diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/include/idmap.h 3_0-vfs/source/include/idmap.h
--- 3_0/source/include/idmap.h	Thu Jan  1 01:00:00 1970
+++ 3_0-vfs/source/include/idmap.h	Wed Apr 23 07:46:33 2003
@@ -0,0 +1,57 @@
+/* 
+   Unix SMB/CIFS implementation.
+
+   Idmap headers
+
+   Copyright (C) Anthony Liguori 2003
+   Copyright (C) Simo Sorce 2003
+   
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+   
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+   
+   You should have received a copy of the GNU Library General Public
+   License along with this library; if not, write to the
+   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA  02111-1307, USA.   
+*/
+
+#define ID_EMPTY	0x00
+#define ID_USERID	0x01
+#define ID_GROUPID	0x02
+#define ID_OTHER	0x04
+
+#define ID_TYPEMASK	0x0f
+
+#define ID_NOMAP	0x10
+#define ID_CACHE	0x20
+
+typedef union unid_t {
+	uid_t uid;
+	gid_t gid;
+} unid_t;
+
+struct DOM_SID;
+/* Filled out by IDMAP backends */
+struct idmap_methods {
+
+	/* Called when backend is first loaded */
+	NTSTATUS (*init)(void);
+
+	NTSTATUS (*get_sid_from_id)(struct DOM_SID *sid, unid_t id, int id_type);
+	NTSTATUS (*get_id_from_sid)(unid_t *id, int *id_type, const struct DOM_SID *sid);
+	NTSTATUS (*set_mapping)(const struct DOM_SID *sid, unid_t id, int id_type);
+
+	/* Called when backend is unloaded */
+	NTSTATUS (*close)(void);
+
+	/* Called to dump backend status */
+	void (*status)(void);
+};
+
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/include/includes.h 3_0-vfs/source/include/includes.h
--- 3_0/source/include/includes.h	Wed Apr 16 18:19:49 2003
+++ 3_0-vfs/source/include/includes.h	Tue Apr 22 16:42:25 2003
@@ -767,6 +767,7 @@ extern int errno;
 #include "debugparse.h"
 
 #include "version.h"
+#include "idmap.h"
 #include "smb.h"
 #include "smbw.h"
 #include "nameserv.h"
@@ -848,11 +849,6 @@ struct functable {
 
 struct printjob;
 
-/***** automatically generated prototypes *****/
-#ifndef NO_PROTO_H
-#include "proto.h"
-#endif
-
 /* String routines */
 
 #include "srvstr.h"
@@ -1275,5 +1271,10 @@ BOOL krb5_get_smb_session_key(krb5_conte
 #undef FALSE
 #endif
 #define FALSE __ERROR__XX__DONT_USE_FALSE
+
+/***** automatically generated prototypes *****/
+#ifndef NO_PROTO_H
+#include "proto.h"
+#endif
 
 #endif /* _INCLUDES_H */
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/include/smb.h 3_0-vfs/source/include/smb.h
--- 3_0/source/include/smb.h	Thu Apr 24 14:28:19 2003
+++ 3_0-vfs/source/include/smb.h	Thu Apr 24 14:30:24 2003
@@ -259,7 +259,7 @@ enum SID_NAME_USE
  *
  * @sa http://msdn.microsoft.com/library/default.asp?url=/library/en-us/security/accctrl_38yn.asp
  **/
-typedef struct sid_info
+typedef struct DOM_SID
 {
   uint8  sid_rev_num;             /**< SID revision number */
   uint8  num_auths;               /**< Number of sub-authorities */
@@ -406,6 +406,7 @@ typedef struct files_struct
 } files_struct;
 
 #include "ntquotas.h"
+#include "sysquotas.h"
 
 /* used to hold an arbitrary blob of data */
 typedef struct data_blob {
@@ -441,18 +442,10 @@ typedef struct
 #include "smb_acls.h"
 #include "vfs.h"
 
-typedef struct smb_vfs_handle_struct
-{
-    void *data;
-    /* Handle on dlopen() call */
-    void *handle;
-    struct smb_vfs_handle_struct  *next, *prev;
-    
-} smb_vfs_handle_struct;
-
 typedef struct connection_struct
 {
 	struct connection_struct *next, *prev;
+	TALLOC_CTX *mem_ctx;
 	unsigned cnum; /* an index passed over the wire */
 	int service;
 	BOOL force_user;
@@ -468,7 +461,8 @@ typedef struct connection_struct
 	char *origpath;
 
 	struct vfs_ops vfs_ops;                   /* Filesystem operations */
-	struct smb_vfs_handle_struct *vfs_private;
+	struct vfs_ops vfs_opaque_ops;			/* OPAQUE Filesystem operations */
+	struct vfs_handle_struct *vfs_handles;		/* for the new plugins */
 
 	char *user; /* name of user who *opened* this connection */
 	uid_t uid; /* uid of user who *opened* this connection */
@@ -992,8 +986,8 @@ struct bitmap {
 #define TRANSACT2_FINDNOTIFYNEXT		0x0C
 #define TRANSACT2_MKDIR				0x0D
 #define TRANSACT2_SESSION_SETUP			0x0E
-#define TRANSACT2_GET_DFS_REFERRAL         0x10
-#define TRANSACT2_REPORT_DFS_INCONSISTANCY 0x11
+#define TRANSACT2_GET_DFS_REFERRAL		0x10
+#define TRANSACT2_REPORT_DFS_INCONSISTANCY	0x11
 
 /* These are the NT transact sub commands. */
 #define NT_TRANSACT_CREATE                1
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/include/smb_macros.h 3_0-vfs/source/include/smb_macros.h
--- 3_0/source/include/smb_macros.h	Thu Apr 24 07:01:00 2003
+++ 3_0-vfs/source/include/smb_macros.h	Thu Apr 24 07:03:24 2003
@@ -247,52 +247,4 @@ copy an IP address from one buffer to an
 
 #define dos_format(fname) string_replace(fname,'/','\\')
 
-/*******************************************************************
- vfs stat wrapper that calls internal2unix.
-********************************************************************/
-
-#define vfs_stat(conn, fname, st) ((conn)->vfs_ops.stat((conn), fname,(st)))
-
-/*******************************************************************
- vfs lstat wrapper that calls internal2unix.
-********************************************************************/
-
-#define vfs_lstat(conn, fname, st) ((conn)->vfs_ops.lstat((conn), fname,(st)))
-
-/*******************************************************************
- vfs fstat wrapper
-********************************************************************/
-
-#define vfs_fstat(fsp, fd, st) ((fsp)->conn->vfs_ops.fstat((fsp),(fd),(st)))
-
-/*******************************************************************
- vfs rmdir wrapper that calls internal2unix.
-********************************************************************/
-
-#define vfs_rmdir(conn,fname) ((conn)->vfs_ops.rmdir((conn),fname))
-
-/*******************************************************************
- vfs Unlink wrapper that calls internal2unix.
-********************************************************************/
-
-#define vfs_unlink(conn, fname) ((conn)->vfs_ops.unlink((conn),fname))
-
-/*******************************************************************
- vfs chmod wrapper that calls internal2unix.
-********************************************************************/
-
-#define vfs_chmod(conn,fname,mode) ((conn)->vfs_ops.chmod((conn),fname,(mode)))
-
-/*******************************************************************
- vfs chown wrapper that calls internal2unix.
-********************************************************************/
-
-#define vfs_chown(conn,fname,uid,gid) ((conn)->vfs_ops.chown((conn),fname,(uid),(gid)))
-
-/*******************************************************************
- A wrapper for vfs_chdir().
-********************************************************************/
-
-#define vfs_chdir(conn,fname) ((conn)->vfs_ops.chdir((conn),fname))
-
 #endif /* _SMB_MACROS_H */
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/include/sysquotas.h 3_0-vfs/source/include/sysquotas.h
--- 3_0/source/include/sysquotas.h	Thu Jan  1 01:00:00 1970
+++ 3_0-vfs/source/include/sysquotas.h	Wed Apr 23 09:03:22 2003
@@ -0,0 +1,199 @@
+/* 
+   Unix SMB/CIFS implementation.
+   SYS QUOTA code constants
+   Copyright (C) Stefan (metze) Metzmacher	2003
+   
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+   
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
+#ifndef _SYSQUOTAS_H
+#define _SYSQUOTAS_H
+
+#ifdef HAVE_SYS_QUOTAS
+
+/* Sometimes we need this on linux for linux/quota.h */
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#ifdef HAVE_ASM_TYPES_H
+#include <asm/types.h>
+#endif
+
+/*
+ * This shouldn't be neccessary - it should be /usr/include/sys/quota.h
+ * Unfortunately, RH7.1 ships with a different quota system using struct mem_dqblk
+ * rather than the struct dqblk defined in /usr/include/sys/quota.h.
+ * This means we must include linux/quota.h to have a hope of working on
+ * RH7.1 systems. And it also means this breaks if the kernel is upgraded
+ * to a Linus 2.4.x (where x > the minor number shipped with RH7.1) until
+ * Linus synchronises with the AC patches. Sometimes I *hate* Linux :-). JRA.
+ */
+#ifdef HAVE_LINUX_QUOTA_H
+#include <linux/quota.h>
+#elif defined(HAVE_SYS_QUOTA_H)
+#include <sys/quota.h>
+#endif
+
+#if defined(HAVE_STRUCT_IF_DQBLK)
+# define SYS_DQBLK if_dqblk
+# define dqb_curblocks dqb_curspace/bsize
+#elif defined(HAVE_STRUCT_MEM_DQBLK)
+# define SYS_DQBLK mem_dqblk
+# define dqb_curblocks dqb_curspace/bsize
+#else /* STRUCT_DQBLK */
+# define SYS_DQBLK dqblk
+#endif
+
+#ifndef Q_SETQLIM
+#define Q_SETQLIM Q_SETQUOTA
+#endif
+
+/*********************************************
+ check for XFS QUOTA MANAGER 
+ *********************************************/
+/* on linux */
+#ifdef HAVE_LINUX_XQM_H
+# include <linux/xqm.h>
+# define HAVE_XFS_QUOTA
+#else
+# ifdef HAVE_XFS_XQM_H
+#  include <xfs/xqm.h>
+#  define HAVE_XFS_QUOTA
+# else
+#  ifdef HAVE_LINUX_DQBLK_XFS_H
+#   include <linux/dqblk_xfs.h>
+#   define HAVE_XFS_QUOTA
+#  endif 
+# endif
+#endif
+/* on IRIX */
+#ifdef Q_XGETQUOTA
+# ifndef HAVE_XFS_QUOTA
+#  define HAVE_XFS_QUOTA
+#  ifndef Q_XQUOTAON
+#   define Q_XQUOTAON Q_QUOTAON
+#  endif /* Q_XQUOTAON */
+#  ifndef Q_XQUOTAOFF
+#   define Q_XQUOTAOFF Q_QUOTAOFF
+#  endif /* Q_XQUOTAOFF */
+#  ifndef Q_XGETQSTAT
+#   define Q_XGETQSTAT Q_GETQSTAT
+#  endif /* Q_XGETQSTAT */
+# endif /* HAVE_XFS_QUOTA */
+#endif /* Q_XGETQUOTA */
+
+#ifdef HAVE_XFS_QUOTA
+/* Linux has BBSIZE in <linux/xfs_fs.h>
+ * or <xfs/xfs_fs.h>
+ * IRIX has BBSIZE in <sys/param.h>
+ */
+#ifdef HAVE_LINUX_XFS_FS_H
+#include <linux/xfs_fs.h>
+#elif defined(HAVE_XFS_XFS_FS_H)
+#include <xfs/xfs_fs.h>
+#endif /* *_XFS_FS_H */
+
+#ifndef BBSHIFT
+#define	BBSHIFT		9
+#endif /* BBSHIFT */
+#ifndef BBSIZE
+#define	BBSIZE		(1<<BBSHIFT)
+#endif /* BBSIZE */
+
+#endif /* HAVE_XFS_QUOTA */
+
+#ifdef LINUX
+# ifndef QUOTABLOCK_SIZE
+#  define QUOTABLOCK_SIZE 1024
+# endif
+/* end LINUX */
+#elif defined(IRIX6)
+# ifndef QUOTABLOCK_SIZE
+#  define QUOTABLOCK_SIZE BBSIZE
+# endif
+/* end IRIX6 */
+#else /* HPUP,... */
+# ifndef QUOTABLOCK_SIZE
+#  define QUOTABLOCK_SIZE DEV_BSIZE
+# endif
+#endif /* HPUP,... */
+
+#if !defined(QUOTAFILENAME) && defined(QFILENAME)
+#define QUOTAFILENAME QFILENAME
+#endif
+
+#ifdef INITQFNAMES
+#define USERQUOTAFILE_EXTENSION ".user"
+#else
+#define USERQUOTAFILE_EXTENSION ""
+#endif
+
+/* this check should be before the QCMD fake! */
+#if defined(QCMD)&&defined(GRPQUOTA)
+#define HAVE_GROUP_QUOTA
+#endif
+
+/* on some systems we have to fake this up ...*/
+#ifndef QCMD
+#define QCMD(cmd,type)	(cmd)
+#endif /* QCMD */
+
+
+#ifdef HAVE_DQB_FSOFTLIMIT
+#define dqb_isoftlimit	dqb_fsoftlimit
+#define dqb_ihardlimit	dqb_fhardlimit
+#define dqb_curinodes	dqb_curfiles
+#endif
+
+/* maybe we can add a configure test for HAVE_CADDR_T,
+ * but it's not needed
+ */
+#ifdef HAVE_CADDR_T
+#define CADDR_T caddr_t
+#else /* CADDR_T */
+#define CADDR_T void*
+#endif /* CADDR_T */
+
+#if defined(HAVE_MNTENT_H)&&defined(HAVE_SETMNTENT)&&defined(HAVE_GETMNTENT)&&defined(HAVE_ENDMNTENT)
+#include <mntent.h>
+#define HAVE_MNTENT 1
+/*#endif defined(HAVE_MNTENT_H)&&defined(HAVE_SETMNTENT)&&defined(HAVE_GETMNTENT)&&defined(HAVE_ENDMNTENT) */
+#elif defined(HAVE_DEVNM_H)&&defined(HAVE_DEVNM)
+#include <devnm.h>
+#endif /* defined(HAVE_DEVNM_H)&&defined(HAVE_DEVNM) */
+
+#endif /* HAVE_SYS_QUOTAS */
+
+/**************************************************
+ Some stuff for the sys_quota api.
+ **************************************************/ 
+
+#define SMB_QUOTAS_NO_LIMIT	((SMB_BIG_UINT)(0))
+#define SMB_QUOTAS_NO_SPACE	((SMB_BIG_UINT)(1))
+
+typedef struct _SMB_DISK_QUOTA {
+	enum SMB_QUOTA_TYPE qtype;
+	SMB_BIG_UINT bsize;
+	SMB_BIG_UINT hardlimit; /* In bsize units. */
+	SMB_BIG_UINT softlimit; /* In bsize units. */
+	SMB_BIG_UINT curblocks; /* In bsize units. */
+	SMB_BIG_UINT ihardlimit; /* inode hard limit. */
+	SMB_BIG_UINT isoftlimit; /* inode soft limit. */
+	SMB_BIG_UINT curinodes; /* Current used inodes. */
+	uint32       qflags;
+} SMB_DISK_QUOTA;
+
+#endif /*_SYSQUOTAS_H */
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/include/vfs.h 3_0-vfs/source/include/vfs.h
--- 3_0/source/include/vfs.h	Wed Apr 16 18:19:49 2003
+++ 3_0-vfs/source/include/vfs.h	Wed Apr 23 12:29:52 2003
@@ -3,6 +3,7 @@
    VFS structures and parameters
    Copyright (C) Tim Potter				1999
    Copyright (C) Alexander Bokovoy			2002
+   Copyright (C) Stefan (metze) Metzmacher		2003
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,50 +46,26 @@
 /* Changed to version 3 for POSIX acl extensions. JRA. */
 /* Changed to version 4 for cascaded VFS interface. Alexander Bokovoy. */
 /* Changed to version 5 for sendfile addition. JRA. */
-#define SMB_VFS_INTERFACE_VERSION 5
+/* Changed to version 6 for the new module system, fixed cascading and quota functions. --metze */
+#define SMB_VFS_INTERFACE_VERSION 6
 
 
-/* Version of supported cascaded interface backward compatibility.
-   (version 5 corresponds to SMB_VFS_INTERFACE_VERSION 5)
-   It is used in vfs_init_custom() to detect VFS modules which conform to cascaded 
-   VFS interface but implement elder version than current version of Samba uses.
-   This allows to use old modules with new VFS interface as far as combined VFS operation
-   set is coherent (will be in most cases). 
-*/
-#define SMB_VFS_INTERFACE_CASCADED 5
+/* to bug old modules witch are trying to compile with the old functions */
+#define vfs_init __ERROR_please_port_this_module_to_SMB_VFS_INTERFACE_VERSION_6(void) { __ERROR_please_port_this_module_to_SMB_VFS_INTERFACE_VERSION_6 };
+#define lp_parm_string __ERROR_please_port_lp_parm_string_to_lp_parm_const_string_or_lp_parm_talloc_string { \
+  __ERROR_please_port_lp_parm_string_to_lp_parm_const_string_or_lp_parm_talloc_string };
+#define lp_vfs_options __ERROR_please_donot_use_lp_vfs_options_anymore_use_lp_parm_xxxx_functions_instead { \
+  __ERROR_please_donot_use_lp_vfs_options_anymore_use_lp_parm_xxxx_functions_instead };
 
 /*
-    Each VFS module must provide following global functions:
-    vfs_init	-- initialization function
-    vfs_done	-- finalization function
-    
-    vfs_init must return proper initialized vfs_op_tuple[] array
-    which describes all operations this module claims to intercept. This function
-    is called whenever module is loaded into smbd process using sys_dlopen().
-    
-    vfs_init must store somewhere vfs_handle reference if module wants to store per-instance
-    private information for further usage. vfs_handle->data should be used to
-    store such information. Do not try to change other fields in this structure
-    or results likely to be unpredictable.
-    
-    vfs_done must perform finalization of the module. In particular,
-    this function must free vfs_ops structure returned to module from smb_vfs_get_opaque_ops()
-    function if it is used (see below). This function is called whenever module 
-    is unloaded from smbd process using sys_dlclose().
-    
-    Prototypes:
-    vfs_op_tuple *vfs_init(const struct vfs_ops *def_vfs_ops,
-			    struct smb_vfs_handle_struct *vfs_handle);
-    void	  vfs_done(connection_struct *conn);
-    
     All intercepted VFS operations must be declared as static functions inside module source
-    in order to keep smbd namespace unpolluted. See source of skel, audit, and recycle bin
+    in order to keep smbd namespace unpolluted. See source of skel, audit, and recycle
     example VFS modules for more details.
-    
 */
 
 /* VFS operations structure */
 
+struct vfs_handle_struct;
 struct connection_struct;
 struct files_struct;
 struct security_descriptor_info;
@@ -97,82 +74,84 @@ struct vfs_ops {
 
 	/* Disk operations */
     
-	int (*connect)(struct connection_struct *conn, const char *service, const char *user);
-	void (*disconnect)(struct connection_struct *conn);
-	SMB_BIG_UINT (*disk_free)(struct connection_struct *conn, const char *path, BOOL small_query, SMB_BIG_UINT *bsize, 
+	int (*connect)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *service, const char *user);
+	void (*disconnect)(struct vfs_handle_struct *handle, struct connection_struct *conn);
+	SMB_BIG_UINT (*disk_free)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, BOOL small_query, SMB_BIG_UINT *bsize, 
 		SMB_BIG_UINT *dfree, SMB_BIG_UINT *dsize);
-    
+	int (*get_quota)(struct vfs_handle_struct *handle, struct connection_struct *conn, enum SMB_QUOTA_TYPE qtype, unid_t id, SMB_DISK_QUOTA *qt);
+	int (*set_quota)(struct vfs_handle_struct *handle, struct connection_struct *conn, enum SMB_QUOTA_TYPE qtype, unid_t id, SMB_DISK_QUOTA *qt);
+
 	/* Directory operations */
 
-	DIR *(*opendir)(struct connection_struct *conn, const char *fname);
-	struct dirent *(*readdir)(struct connection_struct *conn, DIR *dirp);
-	int (*mkdir)(struct connection_struct *conn, const char *path, mode_t mode);
-	int (*rmdir)(struct connection_struct *conn, const char *path);
-	int (*closedir)(struct connection_struct *conn, DIR *dir);
+	DIR *(*opendir)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *fname);
+	struct dirent *(*readdir)(struct vfs_handle_struct *handle, struct connection_struct *conn, DIR *dirp);
+	int (*mkdir)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, mode_t mode);
+	int (*rmdir)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path);
+	int (*closedir)(struct vfs_handle_struct *handle, struct connection_struct *conn, DIR *dir);
     
 	/* File operations */
     
-	int (*open)(struct connection_struct *conn, const char *fname, int flags, mode_t mode);
-	int (*close)(struct files_struct *fsp, int fd);
-	ssize_t (*read)(struct files_struct *fsp, int fd, void *data, size_t n);
-	ssize_t (*write)(struct files_struct *fsp, int fd, const void *data, size_t n);
-	SMB_OFF_T (*lseek)(struct files_struct *fsp, int filedes, SMB_OFF_T offset, int whence);
-	ssize_t (*sendfile)(int tofd, files_struct *fsp, int fromfd, const DATA_BLOB *header, SMB_OFF_T offset, size_t count);
-	int (*rename)(struct connection_struct *conn, const char *old, const char *new);
-	int (*fsync)(struct files_struct *fsp, int fd);
-	int (*stat)(struct connection_struct *conn, const char *fname, SMB_STRUCT_STAT *sbuf);
-	int (*fstat)(struct files_struct *fsp, int fd, SMB_STRUCT_STAT *sbuf);
-	int (*lstat)(struct connection_struct *conn, const char *path, SMB_STRUCT_STAT *sbuf);
-	int (*unlink)(struct connection_struct *conn, const char *path);
-	int (*chmod)(struct connection_struct *conn, const char *path, mode_t mode);
-	int (*fchmod)(struct files_struct *fsp, int fd, mode_t mode);
-	int (*chown)(struct connection_struct *conn, const char *path, uid_t uid, gid_t gid);
-	int (*fchown)(struct files_struct *fsp, int fd, uid_t uid, gid_t gid);
-	int (*chdir)(struct connection_struct *conn, const char *path);
-	char *(*getwd)(struct connection_struct *conn, char *buf);
-	int (*utime)(struct connection_struct *conn, const char *path, struct utimbuf *times);
-	int (*ftruncate)(struct files_struct *fsp, int fd, SMB_OFF_T offset);
-	BOOL (*lock)(struct files_struct *fsp, int fd, int op, SMB_OFF_T offset, SMB_OFF_T count, int type);
-	int (*symlink)(struct connection_struct *conn, const char *oldpath, const char *newpath);
-	int (*readlink)(struct connection_struct *conn, const char *path, char *buf, size_t bufsiz);
-	int (*link)(struct connection_struct *conn, const char *oldpath, const char *newpath);
-	int (*mknod)(struct connection_struct *conn, const char *path, mode_t mode, SMB_DEV_T dev);
-	char *(*realpath)(struct connection_struct *conn, const char *path, char *resolved_path);
+	int (*open)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *fname, int flags, mode_t mode);
+	int (*close)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd);
+	ssize_t (*read)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, void *data, size_t n);
+	ssize_t (*write)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, const void *data, size_t n);
+	SMB_OFF_T (*lseek)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, SMB_OFF_T offset, int whence);
+	ssize_t (*sendfile)(struct vfs_handle_struct *handle, int tofd, files_struct *fsp, int fromfd, const DATA_BLOB *header, SMB_OFF_T offset, size_t count);
+	int (*rename)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *old, const char *new);
+	int (*fsync)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd);
+	int (*stat)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *fname, SMB_STRUCT_STAT *sbuf);
+	int (*fstat)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, SMB_STRUCT_STAT *sbuf);
+	int (*lstat)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, SMB_STRUCT_STAT *sbuf);
+	int (*unlink)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path);
+	int (*chmod)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, mode_t mode);
+	int (*fchmod)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, mode_t mode);
+	int (*chown)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, uid_t uid, gid_t gid);
+	int (*fchown)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, uid_t uid, gid_t gid);
+	int (*chdir)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path);
+	char *(*getwd)(struct vfs_handle_struct *handle, struct connection_struct *conn, char *buf);
+	int (*utime)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, struct utimbuf *times);
+	int (*ftruncate)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, SMB_OFF_T offset);
+	BOOL (*lock)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, int op, SMB_OFF_T offset, SMB_OFF_T count, int type);
+	int (*symlink)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *oldpath, const char *newpath);
+	int (*readlink)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, char *buf, size_t bufsiz);
+	int (*link)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *oldpath, const char *newpath);
+	int (*mknod)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, mode_t mode, SMB_DEV_T dev);
+	char *(*realpath)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, char *resolved_path);
 
 	/* NT ACL operations. */
 
-	size_t (*fget_nt_acl)(struct files_struct *fsp, int fd, struct security_descriptor_info **ppdesc);
-	size_t (*get_nt_acl)(struct files_struct *fsp, const char *name, struct security_descriptor_info **ppdesc);
-	BOOL (*fset_nt_acl)(struct files_struct *fsp, int fd, uint32 security_info_sent, struct security_descriptor_info *psd);
-	BOOL (*set_nt_acl)(struct files_struct *fsp, const char *name, uint32 security_info_sent, struct security_descriptor_info *psd);
+	size_t (*fget_nt_acl)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, struct security_descriptor_info **ppdesc);
+	size_t (*get_nt_acl)(struct vfs_handle_struct *handle, struct files_struct *fsp, const char *name, struct security_descriptor_info **ppdesc);
+	BOOL (*fset_nt_acl)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, uint32 security_info_sent, struct security_descriptor_info *psd);
+	BOOL (*set_nt_acl)(struct vfs_handle_struct *handle, struct files_struct *fsp, const char *name, uint32 security_info_sent, struct security_descriptor_info *psd);
 
 	/* POSIX ACL operations. */
 
-	int (*chmod_acl)(struct connection_struct *conn, const char *name, mode_t mode);
-	int (*fchmod_acl)(struct files_struct *fsp, int fd, mode_t mode);
+	int (*chmod_acl)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *name, mode_t mode);
+	int (*fchmod_acl)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, mode_t mode);
 
-	int (*sys_acl_get_entry)(struct connection_struct *conn, SMB_ACL_T theacl, int entry_id, SMB_ACL_ENTRY_T *entry_p);
-	int (*sys_acl_get_tag_type)(struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d, SMB_ACL_TAG_T *tag_type_p);
-	int (*sys_acl_get_permset)(struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d, SMB_ACL_PERMSET_T *permset_p);
-	void * (*sys_acl_get_qualifier)(struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d);
-	SMB_ACL_T (*sys_acl_get_file)(struct connection_struct *conn, const char *path_p, SMB_ACL_TYPE_T type);
-	SMB_ACL_T (*sys_acl_get_fd)(struct files_struct *fsp, int fd);
-	int (*sys_acl_clear_perms)(struct connection_struct *conn, SMB_ACL_PERMSET_T permset);
-	int (*sys_acl_add_perm)(struct connection_struct *conn, SMB_ACL_PERMSET_T permset, SMB_ACL_PERM_T perm);
-	char * (*sys_acl_to_text)(struct connection_struct *conn, SMB_ACL_T theacl, ssize_t *plen);
-	SMB_ACL_T (*sys_acl_init)(struct connection_struct *conn, int count);
-	int (*sys_acl_create_entry)(struct connection_struct *conn, SMB_ACL_T *pacl, SMB_ACL_ENTRY_T *pentry);
-	int (*sys_acl_set_tag_type)(struct connection_struct *conn, SMB_ACL_ENTRY_T entry, SMB_ACL_TAG_T tagtype);
-	int (*sys_acl_set_qualifier)(struct connection_struct *conn, SMB_ACL_ENTRY_T entry, void *qual);
-	int (*sys_acl_set_permset)(struct connection_struct *conn, SMB_ACL_ENTRY_T entry, SMB_ACL_PERMSET_T permset);
-	int (*sys_acl_valid)(struct connection_struct *conn, SMB_ACL_T theacl );
-	int (*sys_acl_set_file)(struct connection_struct *conn, const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl);
-	int (*sys_acl_set_fd)(struct files_struct *fsp, int fd, SMB_ACL_T theacl);
-	int (*sys_acl_delete_def_file)(struct connection_struct *conn, const char *path);
-	int (*sys_acl_get_perm)(struct connection_struct *conn, SMB_ACL_PERMSET_T permset, SMB_ACL_PERM_T perm);
-	int (*sys_acl_free_text)(struct connection_struct *conn, char *text);
-	int (*sys_acl_free_acl)(struct connection_struct *conn, SMB_ACL_T posix_acl);
-	int (*sys_acl_free_qualifier)(struct connection_struct *conn, void *qualifier, SMB_ACL_TAG_T tagtype);
+	int (*sys_acl_get_entry)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_T theacl, int entry_id, SMB_ACL_ENTRY_T *entry_p);
+	int (*sys_acl_get_tag_type)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d, SMB_ACL_TAG_T *tag_type_p);
+	int (*sys_acl_get_permset)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d, SMB_ACL_PERMSET_T *permset_p);
+	void * (*sys_acl_get_qualifier)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d);
+	SMB_ACL_T (*sys_acl_get_file)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path_p, SMB_ACL_TYPE_T type);
+	SMB_ACL_T (*sys_acl_get_fd)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd);
+	int (*sys_acl_clear_perms)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_PERMSET_T permset);
+	int (*sys_acl_add_perm)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_PERMSET_T permset, SMB_ACL_PERM_T perm);
+	char * (*sys_acl_to_text)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_T theacl, ssize_t *plen);
+	SMB_ACL_T (*sys_acl_init)(struct vfs_handle_struct *handle, struct connection_struct *conn, int count);
+	int (*sys_acl_create_entry)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_T *pacl, SMB_ACL_ENTRY_T *pentry);
+	int (*sys_acl_set_tag_type)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_ENTRY_T entry, SMB_ACL_TAG_T tagtype);
+	int (*sys_acl_set_qualifier)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_ENTRY_T entry, void *qual);
+	int (*sys_acl_set_permset)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_ENTRY_T entry, SMB_ACL_PERMSET_T permset);
+	int (*sys_acl_valid)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_T theacl );
+	int (*sys_acl_set_file)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl);
+	int (*sys_acl_set_fd)(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, SMB_ACL_T theacl);
+	int (*sys_acl_delete_def_file)(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path);
+	int (*sys_acl_get_perm)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_PERMSET_T permset, SMB_ACL_PERM_T perm);
+	int (*sys_acl_free_text)(struct vfs_handle_struct *handle, struct connection_struct *conn, char *text);
+	int (*sys_acl_free_acl)(struct vfs_handle_struct *handle, struct connection_struct *conn, SMB_ACL_T posix_acl);
+	int (*sys_acl_free_qualifier)(struct vfs_handle_struct *handle, struct connection_struct *conn, void *qualifier, SMB_ACL_TAG_T tagtype);
 };
 
 struct vfs_options {
@@ -196,6 +175,8 @@ typedef enum _vfs_op_type {
 	SMB_VFS_OP_CONNECT = 0,
 	SMB_VFS_OP_DISCONNECT,
 	SMB_VFS_OP_DISK_FREE,
+	SMB_VFS_OP_GET_QUOTA,
+	SMB_VFS_OP_SET_QUOTA,
 
 	/* Directory operations */
 
@@ -321,18 +302,54 @@ typedef struct _vfs_op_tuple {
 	vfs_op_layer layer;
 } vfs_op_tuple;
 
-/*
-    Return vfs_ops filled with current opaque VFS operations. This function is designed to
-    be called from VFS module initialization function for those modules which needs 'direct' VFS
-    access (loggers or initiators of file operations other than connection asks for).
-    
-    Returned vfs_ops must be cleaned up in VFS module's finalizer function (vfs_done_<module_name>)
-    using safe_free().
-    
-    Prototype:
-    struct vfs_ops *smb_vfs_get_opaque_ops();
-    
-    This prototype will be available via include/proto.h
-*/
+
+typedef struct vfs_handle_struct {
+	struct vfs_handle_struct  *next, *prev;
+	TALLOC_CTX *mem_ctx;
+	const char *param;
+	struct vfs_ops vfs_def_ops;
+	struct connection_struct *conn;
+	void *data;
+	void (*free_data)(void **data);
+} vfs_handle_struct;
+
+
+#define VFS_HANDLE_GET_OPS(handle,default_vfs_ops,ret) { \
+	if (!(handle)||(((default_vfs_ops)=&(handle)->vfs_def_ops)==NULL)) { \
+		DEBUG(0,("%s() failed to get default_vfs_ops!\n",FUNCTION_MACRO)); \
+		ret; \
+	} \
+}
+
+#define VFS_HANDLE_GET_DATA(handle,datap, type, ret) { \
+	if (!(handle)||((datap=(type *)(handle)->data)==NULL)) { \
+		DEBUG(0,("%s() failed to get vfs_handle->data!\n",FUNCTION_MACRO)); \
+		ret; \
+	} \
+}
+
+#define VFS_HANDLE_SET_DATA(handle,datap, free_fn, type, ret) { \
+	if (!(handle)) { \
+		DEBUG(0,("%s() failed to set handle->data!\n",FUNCTION_MACRO)); \
+		ret; \
+	} else { \
+		if ((handle)->free_data) { \
+			(handle)->free_data(&(handle)->data); \
+		} \
+		(handle)->data = (void *)datap; \
+		(handle)->free_data = free_fn; \
+	} \
+}
+
+#define VFS_HANDLE_FREE_DATA(handle) { \
+	if ((handle) && (handle)->free_data) { \
+		(handle)->free_data(&(handle)->data); \
+	} \
+}
+
+#define VFS_OP(x) ((void *) x)
+
+
+#include "vfs_macros.h"
 
 #endif /* _VFS_H */
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/include/vfs_macros.h 3_0-vfs/source/include/vfs_macros.h
--- 3_0/source/include/vfs_macros.h	Thu Jan  1 01:00:00 1970
+++ 3_0-vfs/source/include/vfs_macros.h	Wed Apr 23 10:02:49 2003
@@ -0,0 +1,192 @@
+/* 
+   Unix SMB/CIFS implementation.
+   VFS wrapper macros
+   Copyright (C) Stefan (metze) Metzmacher	2003
+   
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+   
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
+#ifndef _VFS_MACROS_H
+#define _VFS_MACROS_H
+
+/*********************************************************
+ Don't access conn->vfs_ops directly!!!
+ Use this macros!
+ (Fixes should go also into the vfs_opaque_* macros!)
+**********************************************************/
+
+/* Disk operations */    
+#define vfs_connect(conn, service, user) ((conn)->vfs_ops.connect((conn)->vfs_handles, (conn), (service), (user)))
+#define vfs_disconnect(conn) ((conn)->vfs_ops.disconnect((conn)->vfs_handles, (conn)))
+#define vfs_disk_free(conn, path, small_query, bsize, dfree ,dsize) ((conn)->vfs_ops.disk_free((conn)->vfs_handles, (conn), (path), (small_query), (bsize), (dfree), (dsize)))
+#define vfs_get_quota(conn, qtype, id, qt) ((conn)->vfs_ops.get_quota((conn)->vfs_handles, (qtype), (id), (qt)))
+#define vfs_set_quota(conn, qtype, id, qt) ((conn)->vfs_ops.set_quota((conn)->vfs_handles, (qtype), (id), (qt)))
+
+/* Directory operations */
+#define vfs_opendir(conn, fname) ((conn)->vfs_ops.opendir((conn)->vfs_handles, (conn), (fname)))
+#define vfs_readdir(conn, dirp) ((conn)->vfs_ops.readdir((conn)->vfs_handles, (conn), (dirp)))
+/* vfs_mkdir() is a function in smbd/vfs.c 
+#define vfs_mkdir(conn, path, mode) ((conn)->vfs_ops.mkdir((conn)->vfs_handles,(conn), (path), (mode)))*/
+#define vfs_rmdir(conn, path) ((conn)->vfs_ops.rmdir((conn)->vfs_handles, (conn), (path)))
+#define vfs_closedir(conn, dir) ((conn)->vfs_ops.closedir((conn)->vfs_handles, (conn), dir))
+    
+/* File operations */
+#define vfs_open(conn, fname, flags, mode) ((conn)->vfs_ops.open((conn)->vfs_handles, (conn), (fname), (flags), (mode)))
+#define vfs_close(fsp, fd) ((fsp)->conn->vfs_ops.close((fsp)->conn->vfs_handles, (fsp), (fd)))
+#define vfs_read(fsp, fd, data, n) ((fsp)->conn->vfs_ops.read((fsp)->conn->vfs_handles, (fsp), (fd), (data), (n)))
+#define vfs_write(fsp, fd, data, n) ((fsp)->conn->vfs_ops.write((fsp)->conn->vfs_handles, (fsp), (fd), (data), (n)))
+#define vfs_lseek(fsp, fd, offset, whence) ((fsp)->conn->vfs_ops.lseek((fsp)->conn->vfs_handles, (fsp), (fd), (offset), (whence)))
+#define vfs_sendfile(tofd, fsp, fromfd, header, offset, count) ((fsp)->conn->vfs_ops.sendfile((fsp)->conn->vfs_handles, (tofd), (fsp), (fromfd), (header), (offset), (count)))
+#define vfs_rename(conn, old, new) ((conn)->vfs_ops.rename((conn)->vfs_handles, (conn), (old), (new)))
+#define vfs_fsync(fsp, fd) ((fsp)->conn->vfs_ops.fsync((fsp)->conn->vfs_handles, (fsp), (fd)))
+#define vfs_stat(conn, fname, sbuf) ((conn)->vfs_ops.stat((conn)->vfs_handles, (conn), (fname), (sbuf)))
+#define vfs_fstat(fsp, fd, sbuf) ((fsp)->conn->vfs_ops.fstat((fsp)->conn->vfs_handles, (fsp) ,(fd) ,(sbuf)))
+#define vfs_lstat(conn, path, sbuf) ((conn)->vfs_ops.lstat((conn)->vfs_handles, (conn), (path), (sbuf)))
+#define vfs_unlink(conn, path) ((conn)->vfs_ops.unlink((conn)->vfs_handles, (conn), (path)))
+#define vfs_chmod(conn, path, mode) ((conn)->vfs_ops.chmod((conn)->vfs_handles, (conn), (path), (mode)))
+#define vfs_fchmod(fsp, fd, mode) ((fsp)->conn->vfs_ops.fchmod((fsp)->conn->vfs_handles, (fsp), (fd), (mode)))
+#define vfs_chown(conn, path, uid, gid) ((conn)->vfs_ops.chown((conn)->vfs_handles, (conn), (path), (uid), (gid)))
+#define vfs_fchown(fsp, fd, uid, gid) ((fsp)->conn->vfs_ops.fchown((fsp)->conn->vfs_handles, (fsp), (fd), (uid), (gid)))
+#define vfs_chdir(conn, path) ((conn)->vfs_ops.chdir((conn)->vfs_handles, (conn), (path)))
+#define vfs_getwd(conn, buf) ((conn)->vfs_ops.getwd((conn)->vfs_handles, (conn), (buf)))
+#define vfs_utime(conn, path, times) ((conn)->vfs_ops.utime((conn)->vfs_handles, (conn), (path), (times)))
+#define vfs_ftruncate(fsp, fd, offset) ((fsp)->conn->vfs_ops.ftruncate((fsp)->conn->vfs_handles, (fsp), (fd), (offset)))
+#define vfs_lock(fsp, fd, op, offset, count, type) ((fsp)->conn->vfs_ops.lock((fsp)->conn->vfs_handles, (fsp), (fd) ,(op), (offset), (count), (type)))
+#define vfs_symlink(conn, oldpath, newpath) ((conn)->vfs_ops.symlink((conn)->vfs_handles, (conn), (oldpath), (newpath)))
+#define vfs_readlink(conn, path, buf, bufsiz) ((conn)->vfs_ops.readlink((conn)->vfs_handles, (conn), (path), (buf), (bufsiz)))
+#define vfs_link(conn, oldpath, newpath) ((conn)->vfs_ops.link((conn)->vfs_handles, (conn), (oldpath), (newpath)))
+#define vfs_mknod(conn, path, mode, dev) ((conn)->vfs_ops.mknod((conn)->vfs_handles, (conn), (path), (mode), (dev)))
+#define vfs_realpath(conn, path, resolved_path) ((conn)->vfs_ops.realpath((conn)->vfs_handles, (conn), (path), (resolved_path)))
+
+/* NT ACL operations. */
+#define vfs_fget_nt_acl(fsp, fd, ppdesc) ((fsp)->conn->vfs_ops.fget_nt_acl((fsp)->conn->vfs_handles, (fsp), (fd), (ppdesc)))
+#define vfs_get_nt_acl(fsp, name, ppdesc) ((fsp)->conn->vfs_ops.get_nt_acl((fsp)->conn->vfs_handles, (fsp), (name), (ppdesc)))
+#define vfs_fset_nt_acl(fsp, fd, security_info_sent, psd) ((fsp)->conn->vfs_ops.fset_nt_acl((fsp)->conn->vfs_handles, (fsp), (fd), (security_info_sent), (psd)))
+#define vfs_set_nt_acl(fsp, name, security_info_sent, psd) ((fsp)->conn->vfs_ops.set_nt_acl((fsp)->conn->vfs_handles, (fsp), (name), (security_info_sent), (psd)))
+
+/* POSIX ACL operations. */
+#define vfs_chmod_acl(conn, name, mode) ((conn)->vfs_ops.chmod_acl((conn)->vfs_handles, (conn), (name), (mode)))
+#define vfs_fchmod_acl(fsp, fd, mode) ((fsp)->conn->vfs_ops.fchmod_acl((fsp)->conn->vfs_handles, (fsp), (fd), (mode)))
+
+#define vfs_sys_acl_get_entry(conn, theacl, entry_id, entry_p) ((conn)->vfs_ops.sys_acl_get_entry((conn)->vfs_handles, (conn), (theacl), (entry_id), (entry_p)))
+#define vfs_sys_acl_get_tag_type(conn, entry_d, tag_type_p) ((conn)->vfs_ops.sys_acl_get_tag_type((conn)->vfs_handles, (conn), (entry_d), (tag_type_p)))
+#define vfs_sys_acl_get_permset(conn, entry_d, permset_p) ((conn)->vfs_ops.sys_acl_get_permset((conn)->vfs_handles, (conn), (entry_d), (permset_p)))
+#define vfs_sys_acl_get_qualifier(conn, entry_d) ((conn)->vfs_ops.sys_acl_get_qualifier((conn)->vfs_handles, (conn), (entry_d)))
+#define vfs_sys_acl_get_file(conn, path_p, type) ((conn)->vfs_ops.sys_acl_get_file((conn)->vfs_handles, (conn), (path_p), (type)))
+#define vfs_sys_acl_get_fd(fsp, fd) ((fsp)->conn->vfs_ops.sys_acl_get_fd((fsp)->conn->vfs_handles, (fsp), (fd)))
+#define vfs_sys_acl_clear_perms(conn, permset) ((conn)->vfs_ops.sys_acl_clear_perms((conn)->vfs_handles, (conn), (permset)))
+#define vfs_sys_acl_add_perm(conn, permset, perm) ((conn)->vfs_ops.sys_acl_add_perm((conn)->vfs_handles, (conn), (permset), (perm)))
+#define vfs_sys_acl_to_text(conn, theacl, plen) ((conn)->vfs_ops.sys_acl_to_text((conn)->vfs_handles, (conn), (theacl), (plen)))
+#define vfs_sys_acl_init(conn, count) ((conn)->vfs_ops.sys_acl_init((conn)->vfs_handles, (conn), (count)))
+#define vfs_sys_acl_create_entry(conn, pacl, pentry) ((conn)->vfs_ops.sys_acl_create_entry((conn)->vfs_handles, (conn), (pacl), (pentry)))
+#define vfs_sys_acl_set_tag_type(conn, entry, tagtype) ((conn)->vfs_ops.sys_acl_set_tag_type((conn)->vfs_handles, (conn), (entry), (tagtype)))
+#define vfs_sys_acl_set_qualifier(conn, entry, qual) ((conn)->vfs_ops.sys_acl_set_qualifier((conn)->vfs_handles, (conn), (entry), (qual)))
+#define vfs_sys_acl_set_permset(conn, entry, permset) ((conn)->vfs_ops.sys_acl_set_permset((conn)->vfs_handles, (conn), (entry), (permset)))
+#define vfs_sys_acl_valid(conn, theacl) ((conn)->vfs_ops.sys_acl_valid((conn)->vfs_handles, (conn), (theacl)))
+#define vfs_sys_acl_set_file(conn, name, acltype, theacl) ((conn)->vfs_ops.sys_acl_set_file((conn)->vfs_handles, (conn), (name), (acltype), (theacl)))
+#define vfs_sys_acl_set_fd(fsp, fd, theacl) ((fsp)->conn->vfs_ops.sys_acl_set_fd((fsp)->conn->vfs_handles, (fsp), (fd), (theacl)))
+#define vfs_sys_acl_delete_def_file(conn, path) ((conn)->vfs_ops.sys_acl_delete_def_file((conn)->vfs_handles, (conn), (path)))
+#define vfs_sys_acl_get_perm(conn, permset, perm) ((conn)->vfs_ops.sys_acl_get_perm((conn)->vfs_handles, (conn), (permset), (perm)))
+#define vfs_sys_acl_free_text(conn, text) ((conn)->vfs_ops.sys_acl_free_text((conn)->vfs_handles, (conn), (text)))
+#define vfs_sys_acl_free_acl(conn, posix_acl) ((conn)->vfs_ops.sys_acl_free_acl((conn)->vfs_handles, (conn), (posix_acl)))
+#define vfs_sys_acl_free_qualifier(conn, qualifier, tagtype) ((conn)->vfs_ops.sys_acl_free_qualifier((conn)->vfs_handles, (conn), (qualifier), (tagtype)))
+
+
+/*********************************************************
+ Don't access conn->vfs_opaque_ops directly!!!
+ Use this macros!
+ (Fixes should also go into the vfs_* macros!)
+**********************************************************/
+#if 0 /* FIX ME: the vfs_opaque_ops.*() should get the right vfs_handle_struct... */
+
+/* Disk operations */    
+#define vfs_opaque_connect(conn, service, user) ((conn)->vfs_opaque_ops.connect((conn)->vfs_handles, (conn), (service), (user)))
+#define vfs_opaque_disconnect(conn) ((conn)->vfs_opaque_ops.disconnect((conn)->vfs_handles, (conn)))
+#define vfs_opaque_disk_free(conn, path, small_query, bsize, dfree ,dsize) ((conn)->vfs_opaque_ops.disk_free((conn)->vfs_handles, (conn), (path), (small_query), (bsize), (dfree), (dsize)))
+#define vfs_opaque_get_quota(conn, qtype, id, qt) ((conn)->vfs_opaque_ops.get_quota((conn)->vfs_handles, (qtype), (id), (qt)))
+#define vfs_opaque_set_quota(conn, qtype, id, qt) ((conn)->vfs_opaque_ops.set_quota((conn)->vfs_handles, (qtype), (id), (qt)))
+
+/* Directory operations */
+#define vfs_opaque_opendir(conn, fname) ((conn)->vfs_opaque_ops.opendir((conn)->vfs_handles, (conn), (fname)))
+#define vfs_opaque_readdir(conn, dirp) ((conn)->vfs_opaque_ops.readdir((conn)->vfs_handles, (conn), (dirp)))
+/* vfs_mkdir() is a function in smbd/vfs.c 
+#define vfs_opaque_mkdir(conn, path, mode) ((conn)->vfs_opaque_ops.mkdir((conn)->vfs_handles,(conn), (path), (mode)))*/
+#define vfs_opaque_rmdir(conn, path) ((conn)->vfs_opaque_ops.rmdir((conn)->vfs_handles, (conn), (path)))
+#define vfs_opaque_closedir(conn, dir) ((conn)->vfs_opaque_ops.closedir((conn)->vfs_handles, (conn), dir))
+    
+/* File operations */
+#define vfs_opaque_open(conn, fname, flags, mode) ((conn)->vfs_opaque_ops.open((conn)->vfs_handles, (conn), (fname), (flags), (mode)))
+#define vfs_opaque_close(fsp, fd) ((fsp)->conn->vfs_opaque_ops.close((fsp)->conn->vfs_handles, (fsp), (fd)))
+#define vfs_opaque_read(fsp, fd, data, n) ((fsp)->conn->vfs_opaque_ops.read((fsp)->conn->vfs_handles, (fsp), (fd), (data), (n)))
+#define vfs_opaque_write(fsp, fd, data, n) ((fsp)->conn->vfs_opaque_ops.write((fsp)->conn->vfs_handles, (fsp), (fd), (data), (n)))
+#define vfs_opaque_lseek(fsp, fd, offset, whence) ((fsp)->conn->vfs_opaque_ops.lseek((fsp)->conn->vfs_handles, (fsp), (fd), (offset), (whence)))
+#define vfs_opaque_sendfile(tofd, fsp, fromfd, header, offset, count) ((fsp)->conn->vfs_opaque_ops.sendfile((fsp)->conn->vfs_handles, (tofd), (fsp), (fromfd), (header), (offset), (count)))
+#define vfs_opaque_rename(conn, old, new) ((conn)->vfs_opaque_ops.rename((conn)->vfs_handles, (conn), (old), (new)))
+#define vfs_opaque_fsync(fsp, fd) ((fsp)->conn->vfs_opaque_ops.fsync((fsp)->conn->vfs_handles, (fsp), (fd)))
+#define vfs_opaque_stat(conn, fname, sbuf) ((conn)->vfs_opaque_ops.stat((conn)->vfs_handles, (conn), (fname), (sbuf)))
+#define vfs_opaque_fstat(fsp, fd, sbuf) ((fsp)->conn->vfs_opaque_ops.fstat((fsp)->conn->vfs_handles, (fsp) ,(fd) ,(sbuf)))
+#define vfs_opaque_lstat(conn, path, sbuf) ((conn)->vfs_opaque_ops.lstat((conn)->vfs_handles, (conn), (path), (sbuf)))
+#define vfs_opaque_unlink(conn, path) ((conn)->vfs_opaque_ops.unlink((conn)->vfs_handles, (conn), (path)))
+#define vfs_opaque_chmod(conn, path, mode) ((conn)->vfs_opaque_ops.chmod((conn)->vfs_handles, (conn), (path), (mode)))
+#define vfs_opaque_fchmod(fsp, fd, mode) ((fsp)->conn->vfs_opaque_ops.fchmod((fsp)->conn->vfs_handles, (fsp), (fd), (mode)))
+#define vfs_opaque_chown(conn, path, uid, gid) ((conn)->vfs_opaque_ops.chown((conn)->vfs_handles, (conn), (path), (uid), (gid)))
+#define vfs_opaque_fchown(fsp, fd, uid, gid) ((fsp)->conn->vfs_opaque_ops.fchown((fsp)->conn->vfs_handles, (fsp), (fd), (uid), (gid)))
+#define vfs_opaque_chdir(conn, path) ((conn)->vfs_opaque_ops.chdir((conn)->vfs_handles, (conn), (path)))
+#define vfs_opaque_getwd(conn, buf) ((conn)->vfs_opaque_ops.getwd((conn)->vfs_handles, (conn), (buf)))
+#define vfs_opaque_utime(conn, path, times) ((conn)->vfs_opaque_ops.utime((conn)->vfs_handles, (conn), (path), (times)))
+#define vfs_opaque_ftruncate(fsp, fd, offset) ((fsp)->conn->vfs_opaque_ops.ftruncate((fsp)->conn->vfs_handles, (fsp), (fd), (offset)))
+#define vfs_opaque_lock(fsp, fd, op, offset, count, type) ((fsp)->conn->vfs_opaque_ops.lock((fsp)->conn->vfs_handles, (fsp), (fd) ,(op), (offset), (count), (type)))
+#define vfs_opaque_symlink(conn, oldpath, newpath) ((conn)->vfs_opaque_ops.symlink((conn)->vfs_handles, (conn), (oldpath), (newpath)))
+#define vfs_opaque_readlink(conn, path, buf, bufsiz) ((conn)->vfs_opaque_ops.readlink((conn)->vfs_handles, (conn), (path), (buf), (bufsiz)))
+#define vfs_opaque_link(conn, oldpath, newpath) ((conn)->vfs_opaque_ops.link((conn)->vfs_handles, (conn), (oldpath), (newpath)))
+#define vfs_opaque_mknod(conn, path, mode, dev) ((conn)->vfs_opaque_ops.mknod((conn)->vfs_handles, (conn), (path), (mode), (dev)))
+#define vfs_opaque_realpath(conn, path, resolved_path) ((conn)->vfs_opaque_ops.realpath((conn)->vfs_handles, (conn), (path), (resolved_path)))
+
+/* NT ACL operations. */
+#define vfs_opaque_fget_nt_acl(fsp, fd, ppdesc) ((fsp)->conn->vfs_opaque_ops.fget_nt_acl((fsp)->conn->vfs_handles, (fsp), (fd), (ppdesc)))
+#define vfs_opaque_get_nt_acl(fsp, name, ppdesc) ((fsp)->conn->vfs_opaque_ops.get_nt_acl((fsp)->conn->vfs_handles, (fsp), (name), (ppdesc)))
+#define vfs_opaque_fset_nt_acl(fsp, fd, security_info_sent, psd) ((fsp)->conn->vfs_opaque_ops.fset_nt_acl((fsp)->conn->vfs_handles, (fsp), (fd), (security_info_sent), (psd)))
+#define vfs_opaque_set_nt_acl(fsp, name, security_info_sent, psd) ((fsp)->conn->vfs_opaque_ops.set_nt_acl((fsp)->conn->vfs_handles, (fsp), (name), (security_info_sent), (psd)))
+
+/* POSIX ACL operations. */
+#define vfs_opaque_chmod_acl(conn, name, mode) ((conn)->vfs_opaque_ops.chmod_acl((conn)->vfs_handles, (conn), (name), (mode)))
+#define vfs_opaque_fchmod_acl(fsp, fd, mode) ((fsp)->conn->vfs_opaque_ops.fchmod_acl((fsp)->conn->vfs_handles, (fsp), (fd), (mode)))
+
+#define vfs_opaque_sys_acl_get_entry(conn, theacl, entry_id, entry_p) ((conn)->vfs_opaque_ops.sys_acl_get_entry((conn)->vfs_handles, (conn), (theacl), (entry_id), (entry_p)))
+#define vfs_opaque_sys_acl_get_tag_type(conn, entry_d, tag_type_p) ((conn)->vfs_opaque_ops.sys_acl_get_tag_type((conn)->vfs_handles, (conn), (entry_d), (tag_type_p)))
+#define vfs_opaque_sys_acl_get_permset(conn, entry_d, permset_p) ((conn)->vfs_opaque_ops.sys_acl_get_permset((conn)->vfs_handles, (conn), (entry_d), (permset_p)))
+#define vfs_opaque_sys_acl_get_qualifier(conn, entry_d) ((conn)->vfs_opaque_ops.sys_acl_get_qualifier((conn)->vfs_handles, (conn), (entry_d)))
+#define vfs_opaque_sys_acl_get_file(conn, path_p, type) ((conn)->vfs_opaque_ops.sys_acl_get_file((conn)->vfs_handles, (conn), (path_p), (type)))
+#define vfs_opaque_sys_acl_get_fd(fsp, fd) ((fsp)->conn->vfs_opaque_ops.sys_acl_get_fd((fsp)->conn->vfs_handles, (fsp), (fd)))
+#define vfs_opaque_sys_acl_clear_perms(conn, permset) ((conn)->vfs_opaque_ops.sys_acl_clear_perms((conn)->vfs_handles, (conn), (permset)))
+#define vfs_opaque_sys_acl_add_perm(conn, permset, perm) ((conn)->vfs_opaque_ops.sys_acl_add_perm((conn)->vfs_handles, (conn), (permset), (perm)))
+#define vfs_opaque_sys_acl_to_text(conn, theacl, plen) ((conn)->vfs_opaque_ops.sys_acl_to_text((conn)->vfs_handles, (conn), (theacl), (plen)))
+#define vfs_opaque_sys_acl_init(conn, count) ((conn)->vfs_opaque_ops.sys_acl_init((conn)->vfs_handles, (conn), (count)))
+#define vfs_opaque_sys_acl_create_entry(conn, pacl, pentry) ((conn)->vfs_opaque_ops.sys_acl_create_entry((conn)->vfs_handles, (conn), (pacl), (pentry)))
+#define vfs_opaque_sys_acl_set_tag_type(conn, entry, tagtype) ((conn)->vfs_opaque_ops.sys_acl_set_tag_type((conn)->vfs_handles, (conn), (entry), (tagtype)))
+#define vfs_opaque_sys_acl_set_qualifier(conn, entry, qual) ((conn)->vfs_opaque_ops.sys_acl_set_qualifier((conn)->vfs_handles, (conn), (entry), (qual)))
+#define vfs_opaque_sys_acl_set_permset(conn, entry, permset) ((conn)->vfs_opaque_ops.sys_acl_set_permset((conn)->vfs_handles, (conn), (entry), (permset)))
+#define vfs_opaque_sys_acl_valid(conn, theacl) ((conn)->vfs_opaque_ops.sys_acl_valid((conn)->vfs_handles, (conn), (theacl)))
+#define vfs_opaque_sys_acl_set_file(conn, name, acltype, theacl) ((conn)->vfs_opaque_ops.sys_acl_set_file((conn)->vfs_handles, (conn), (name), (acltype), (theacl)))
+#define vfs_opaque_sys_acl_set_fd(fsp, fd, theacl) ((fsp)->conn->vfs_opaque_ops.sys_acl_set_fd((fsp)->conn->vfs_handles, (fsp), (fd), (theacl)))
+#define vfs_opaque_sys_acl_delete_def_file(conn, path) ((conn)->vfs_opaque_ops.sys_acl_delete_def_file((conn)->vfs_handles, (conn), (path)))
+#define vfs_opaque_sys_acl_get_perm(conn, permset, perm) ((conn)->vfs_opaque_ops.sys_acl_get_perm((conn)->vfs_handles, (conn), (permset), (perm)))
+#define vfs_opaque_sys_acl_free_text(conn, text) ((conn)->vfs_opaque_ops.sys_acl_free_text((conn)->vfs_handles, (conn), (text)))
+#define vfs_opaque_sys_acl_free_acl(conn, posix_acl) ((conn)->vfs_opaque_ops.sys_acl_free_acl((conn)->vfs_handles, (conn), (posix_acl)))
+#define vfs_opaque_sys_acl_free_qualifier(conn, qualifier, tagtype) ((conn)->vfs_opaque_ops.sys_acl_free_qualifier((conn)->vfs_handles, (conn), (qualifier), (tagtype)))
+
+#endif /* FIX ME: the vfs_opaque_ops.*() should get the right vfs_handle_struct... */
+
+#endif /* _VFS_MACROS_H */
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/locking/posix.c 3_0-vfs/source/locking/posix.c
--- 3_0/source/locking/posix.c	Sat Feb 15 18:43:45 2003
+++ 3_0-vfs/source/locking/posix.c	Tue Apr 22 15:58:44 2003
@@ -205,7 +205,7 @@ int fd_close_posix(struct connection_str
 		/*
 		 * No POSIX to worry about, just close.
 		 */
-		ret = conn->vfs_ops.close(fsp,fsp->fd);
+		ret = vfs_close(fsp,fsp->fd);
 		fsp->fd = -1;
 		return ret;
 	}
@@ -259,7 +259,7 @@ int fd_close_posix(struct connection_str
 		DEBUG(10,("fd_close_posix: doing close on %u fd's.\n", (unsigned int)count ));
 
 		for(i = 0; i < count; i++) {
-			if (conn->vfs_ops.close(fsp,fd_array[i]) == -1) {
+			if (vfs_close(fsp,fd_array[i]) == -1) {
 				saved_errno = errno;
 			}
 		}
@@ -278,7 +278,7 @@ int fd_close_posix(struct connection_str
 	 * Finally close the fd associated with this fsp.
 	 */
 
-	ret = conn->vfs_ops.close(fsp,fsp->fd);
+	ret = vfs_close(fsp,fsp->fd);
 
 	if (saved_errno != 0) {
         errno = saved_errno;
@@ -646,11 +646,10 @@ static BOOL posix_lock_in_range(SMB_OFF_
 static BOOL posix_fcntl_lock(files_struct *fsp, int op, SMB_OFF_T offset, SMB_OFF_T count, int type)
 {
 	int ret;
-	struct connection_struct *conn = fsp->conn;
 
 	DEBUG(8,("posix_fcntl_lock %d %d %.0f %.0f %d\n",fsp->fd,op,(double)offset,(double)count,type));
 
-	ret = conn->vfs_ops.lock(fsp,fsp->fd,op,offset,count,type);
+	ret = vfs_lock(fsp,fsp->fd,op,offset,count,type);
 
 	if (!ret && ((errno == EFBIG) || (errno == ENOLCK) || (errno ==  EINVAL))) {
 
@@ -674,7 +673,7 @@ static BOOL posix_fcntl_lock(files_struc
 			DEBUG(0,("Count greater than 31 bits - retrying with 31 bit truncated length.\n"));
 			errno = 0;
 			count &= 0x7fffffff;
-			ret = conn->vfs_ops.lock(fsp,fsp->fd,op,offset,count,type);
+			ret = vfs_lock(fsp,fsp->fd,op,offset,count,type);
 		}
 	}
 
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/modules/vfs_audit.c 3_0-vfs/source/modules/vfs_audit.c
--- 3_0/source/modules/vfs_audit.c	Wed Apr 16 18:19:49 2003
+++ 3_0-vfs/source/modules/vfs_audit.c	Wed Apr 23 12:15:58 2003
@@ -2,8 +2,9 @@
  * Auditing VFS module for samba.  Log selected file operations to syslog
  * facility.
  *
- * Copyright (C) Tim Potter, 1999-2000
- * Copyright (C) Alexander Bokovoy, 2002
+ * Copyright (C) Tim Potter			1999-2000
+ * Copyright (C) Alexander Bokovoy		2002
+ * Copyright (C) Stefan (metze) Metzmacher	2002
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,118 +21,121 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "config.h"
-#include <stdio.h>
-#include <sys/stat.h>
-#ifdef HAVE_UTIME_H
-#include <utime.h>
-#endif
-#ifdef HAVE_DIRENT_H
-#include <dirent.h>
-#endif
-#include <syslog.h>
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-#include <errno.h>
-#include <string.h>
-#include <includes.h>
-#include <vfs.h>
-
-#ifndef SYSLOG_FACILITY
-#define SYSLOG_FACILITY   LOG_USER
-#endif
-
-#ifndef SYSLOG_PRIORITY
-#define SYSLOG_PRIORITY   LOG_NOTICE
-#endif
+
+#include "includes.h"
+
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_VFS
 
 /* Function prototypes */
 
-static int audit_connect(struct connection_struct *conn, const char *svc, const char *user);
-static void audit_disconnect(struct connection_struct *conn);
-static DIR *audit_opendir(struct connection_struct *conn, const char *fname);
-static int audit_mkdir(struct connection_struct *conn, const char *path, mode_t mode);
-static int audit_rmdir(struct connection_struct *conn, const char *path);
-static int audit_open(struct connection_struct *conn, const char *fname, int flags, mode_t mode);
-static int audit_close(struct files_struct *fsp, int fd);
-static int audit_rename(struct connection_struct *conn, const char *old, const char *new);
-static int audit_unlink(struct connection_struct *conn, const char *path);
-static int audit_chmod(struct connection_struct *conn, const char *path, mode_t mode);
-static int audit_chmod_acl(struct connection_struct *conn, const char *name, mode_t mode);
-static int audit_fchmod(struct files_struct *fsp, int fd, mode_t mode);
-static int audit_fchmod_acl(struct files_struct *fsp, int fd, mode_t mode);
+static int audit_connect(vfs_handle_struct *handle, connection_struct *conn, const char *svc, const char *user);
+static void audit_disconnect(vfs_handle_struct *handle, connection_struct *conn);
+static DIR *audit_opendir(vfs_handle_struct *handle, connection_struct *conn, const char *fname);
+static int audit_mkdir(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode);
+static int audit_rmdir(vfs_handle_struct *handle, connection_struct *conn, const char *path);
+static int audit_open(vfs_handle_struct *handle, connection_struct *conn, const char *fname, int flags, mode_t mode);
+static int audit_close(vfs_handle_struct *handle, files_struct *fsp, int fd);
+static int audit_rename(vfs_handle_struct *handle, connection_struct *conn, const char *old, const char *new);
+static int audit_unlink(vfs_handle_struct *handle, connection_struct *conn, const char *path);
+static int audit_chmod(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode);
+static int audit_chmod_acl(vfs_handle_struct *handle, connection_struct *conn, const char *name, mode_t mode);
+static int audit_fchmod(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode);
+static int audit_fchmod_acl(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode);
 
 /* VFS operations */
 
-static struct vfs_ops default_vfs_ops;   /* For passthrough operation */
-static struct smb_vfs_handle_struct *audit_handle;
-
-static vfs_op_tuple audit_ops[] = {
+static vfs_op_tuple audit_op_tuples[] = {
     
 	/* Disk operations */
 
-	{audit_connect, 	SMB_VFS_OP_CONNECT, 	SMB_VFS_LAYER_LOGGER},
-	{audit_disconnect, 	SMB_VFS_OP_DISCONNECT, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_connect), 	SMB_VFS_OP_CONNECT, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_disconnect), 	SMB_VFS_OP_DISCONNECT, 	SMB_VFS_LAYER_LOGGER},
 
 	/* Directory operations */
 
-	{audit_opendir, 	SMB_VFS_OP_OPENDIR, 	SMB_VFS_LAYER_LOGGER},
-	{audit_mkdir, 		SMB_VFS_OP_MKDIR, 	SMB_VFS_LAYER_LOGGER},
-	{audit_rmdir, 		SMB_VFS_OP_RMDIR, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_opendir), 	SMB_VFS_OP_OPENDIR, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_mkdir), 		SMB_VFS_OP_MKDIR, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_rmdir), 		SMB_VFS_OP_RMDIR, 	SMB_VFS_LAYER_LOGGER},
 
 	/* File operations */
 
-	{audit_open, 		SMB_VFS_OP_OPEN, 	SMB_VFS_LAYER_LOGGER},
-	{audit_close, 		SMB_VFS_OP_CLOSE, 	SMB_VFS_LAYER_LOGGER},
-	{audit_rename, 		SMB_VFS_OP_RENAME, 	SMB_VFS_LAYER_LOGGER},
-	{audit_unlink, 		SMB_VFS_OP_UNLINK, 	SMB_VFS_LAYER_LOGGER},
-	{audit_chmod, 		SMB_VFS_OP_CHMOD, 	SMB_VFS_LAYER_LOGGER},
-	{audit_fchmod, 		SMB_VFS_OP_FCHMOD, 	SMB_VFS_LAYER_LOGGER},
-	{audit_chmod_acl, 	SMB_VFS_OP_CHMOD_ACL, 	SMB_VFS_LAYER_LOGGER},
-	{audit_fchmod_acl, 	SMB_VFS_OP_FCHMOD_ACL, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_open), 		SMB_VFS_OP_OPEN, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_close), 		SMB_VFS_OP_CLOSE, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_rename), 		SMB_VFS_OP_RENAME, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_unlink), 		SMB_VFS_OP_UNLINK, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_chmod), 		SMB_VFS_OP_CHMOD, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_fchmod), 		SMB_VFS_OP_FCHMOD, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_chmod_acl), 	SMB_VFS_OP_CHMOD_ACL, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_fchmod_acl), 	SMB_VFS_OP_FCHMOD_ACL, 	SMB_VFS_LAYER_LOGGER},
 	
 	/* Finish VFS operations definition */
 	
-	{NULL, 			SMB_VFS_OP_NOOP, 	SMB_VFS_LAYER_NOOP}
+	{VFS_OP(NULL), 			SMB_VFS_OP_NOOP, 	SMB_VFS_LAYER_NOOP}
 };
 
-/* VFS initialisation function.  Return vfs_op_tuple array back to SAMBA. */
 
-static vfs_op_tuple *audit_init(const struct vfs_ops *def_vfs_ops, 
-			struct smb_vfs_handle_struct *vfs_handle)
+static int audit_syslog_facility(vfs_handle_struct *handle)
 {
-	memcpy(&default_vfs_ops, def_vfs_ops, sizeof(struct vfs_ops));
-	
-	audit_handle = vfs_handle;
+	/* fix me: let this be configurable by:
+	 *	lp_param_enum(SNUM(handle->conn),(handle->param?handle->param:"audit"),"syslog facility",
+	 *		audit_enum_facility,LOG_USER); 
+	 */
+	return LOG_USER;
+}
+
 
-	openlog("smbd_audit", LOG_PID, SYSLOG_FACILITY);
-	syslog(SYSLOG_PRIORITY, "VFS_INIT: vfs_ops loaded\n");
-	return audit_ops;
+static int audit_syslog_priority(vfs_handle_struct *handle)
+{
+	/* fix me: let this be configurable by:
+	 *	lp_param_enum(SNUM(handle->conn),(handle->param?handle->param:"audit"),"syslog priority",
+	 *		audit_enum_priority,LOG_NOTICE); 
+	 */
+	return LOG_NOTICE;
 }
 
 /* Implementation of vfs_ops.  Pass everything on to the default
    operation but log event first. */
 
-static int audit_connect(struct connection_struct *conn, const char *svc, const char *user)
+static int audit_connect(vfs_handle_struct *handle, connection_struct *conn, const char *svc, const char *user)
 {
-	syslog(SYSLOG_PRIORITY, "connect to service %s by user %s\n", 
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	openlog("smbd_audit", LOG_PID, audit_syslog_facility(handle));
+
+	syslog(audit_syslog_priority(handle), "connect to service %s by user %s\n", 
 	       svc, user);
 
-	return default_vfs_ops.connect(conn, svc, user);
+	result = default_vfs_ops->connect(handle->next, conn, svc, user);
+
+	return result;
 }
 
-static void audit_disconnect(struct connection_struct *conn)
+static void audit_disconnect(vfs_handle_struct *handle, connection_struct *conn)
 {
-	syslog(SYSLOG_PRIORITY, "disconnected\n");
-	default_vfs_ops.disconnect(conn);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return);
+	
+	syslog(audit_syslog_priority(handle), "disconnected\n");
+	default_vfs_ops->disconnect(handle->next, conn);
+
+	return;
 }
 
-static DIR *audit_opendir(struct connection_struct *conn, const char *fname)
+static DIR *audit_opendir(vfs_handle_struct *handle, connection_struct *conn, const char *fname)
 {
-	DIR *result = default_vfs_ops.opendir(conn, fname);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	DIR *result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return NULL);
+	
+	result = default_vfs_ops->opendir(handle->next, conn, fname);
 
-	syslog(SYSLOG_PRIORITY, "opendir %s %s%s\n",
+	syslog(audit_syslog_priority(handle), "opendir %s %s%s\n",
 	       fname,
 	       (result == NULL) ? "failed: " : "",
 	       (result == NULL) ? strerror(errno) : "");
@@ -139,11 +143,16 @@ static DIR *audit_opendir(struct connect
 	return result;
 }
 
-static int audit_mkdir(struct connection_struct *conn, const char *path, mode_t mode)
+static int audit_mkdir(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode)
 {
-	int result = default_vfs_ops.mkdir(conn, path, mode);
-
-	syslog(SYSLOG_PRIORITY, "mkdir %s %s%s\n", 
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->mkdir(handle->next, conn, path, mode);
+	
+	syslog(audit_syslog_priority(handle), "mkdir %s %s%s\n", 
 	       path,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -151,11 +160,16 @@ static int audit_mkdir(struct connection
 	return result;
 }
 
-static int audit_rmdir(struct connection_struct *conn, const char *path)
+static int audit_rmdir(vfs_handle_struct *handle, connection_struct *conn, const char *path)
 {
-	int result = default_vfs_ops.rmdir(conn, path);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->rmdir(handle->next, conn, path);
 
-	syslog(SYSLOG_PRIORITY, "rmdir %s %s%s\n", 
+	syslog(audit_syslog_priority(handle), "rmdir %s %s%s\n", 
 	       path, 
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -163,11 +177,16 @@ static int audit_rmdir(struct connection
 	return result;
 }
 
-static int audit_open(struct connection_struct *conn, const char *fname, int flags, mode_t mode)
+static int audit_open(vfs_handle_struct *handle, connection_struct *conn, const char *fname, int flags, mode_t mode)
 {
-	int result = default_vfs_ops.open(conn, fname, flags, mode);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->open(handle->next, conn, fname, flags, mode);
 
-	syslog(SYSLOG_PRIORITY, "open %s (fd %d) %s%s%s\n", 
+	syslog(audit_syslog_priority(handle), "open %s (fd %d) %s%s%s\n", 
 	       fname, result,
 	       ((flags & O_WRONLY) || (flags & O_RDWR)) ? "for writing " : "", 
 	       (result < 0) ? "failed: " : "",
@@ -176,11 +195,16 @@ static int audit_open(struct connection_
 	return result;
 }
 
-static int audit_close(struct files_struct *fsp, int fd)
+static int audit_close(vfs_handle_struct *handle, files_struct *fsp, int fd)
 {
-	int result = default_vfs_ops.close(fsp, fd);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->close(handle->next, fsp, fd);
 
-	syslog(SYSLOG_PRIORITY, "close fd %d %s%s\n",
+	syslog(audit_syslog_priority(handle), "close fd %d %s%s\n",
 	       fd,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -188,11 +212,16 @@ static int audit_close(struct files_stru
 	return result;
 }
 
-static int audit_rename(struct connection_struct *conn, const char *old, const char *new)
+static int audit_rename(vfs_handle_struct *handle, connection_struct *conn, const char *old, const char *new)
 {
-	int result = default_vfs_ops.rename(conn, old, new);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->rename(handle->next, conn, old, new);
 
-	syslog(SYSLOG_PRIORITY, "rename %s -> %s %s%s\n",
+	syslog(audit_syslog_priority(handle), "rename %s -> %s %s%s\n",
 	       old, new,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -200,11 +229,16 @@ static int audit_rename(struct connectio
 	return result;    
 }
 
-static int audit_unlink(struct connection_struct *conn, const char *path)
+static int audit_unlink(vfs_handle_struct *handle, connection_struct *conn, const char *path)
 {
-	int result = default_vfs_ops.unlink(conn, path);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->unlink(handle->next, conn, path);
 
-	syslog(SYSLOG_PRIORITY, "unlink %s %s%s\n",
+	syslog(audit_syslog_priority(handle), "unlink %s %s%s\n",
 	       path,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -212,11 +246,16 @@ static int audit_unlink(struct connectio
 	return result;
 }
 
-static int audit_chmod(struct connection_struct *conn, const char *path, mode_t mode)
+static int audit_chmod(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode)
 {
-	int result = default_vfs_ops.chmod(conn, path, mode);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+
+	result = default_vfs_ops->chmod(handle->next, conn, path, mode);
 
-	syslog(SYSLOG_PRIORITY, "chmod %s mode 0x%x %s%s\n",
+	syslog(audit_syslog_priority(handle), "chmod %s mode 0x%x %s%s\n",
 	       path, mode,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -224,16 +263,16 @@ static int audit_chmod(struct connection
 	return result;
 }
 
-static int audit_chmod_acl(struct connection_struct *conn, const char *path, mode_t mode)
+static int audit_chmod_acl(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->chmod_acl(handle->next, conn, path, mode);
 
-	if ( !default_vfs_ops.chmod_acl )
-		return 0;
-
-	result = default_vfs_ops.chmod_acl(conn, path, mode);
-
-	syslog(SYSLOG_PRIORITY, "chmod_acl %s mode 0x%x %s%s\n",
+	syslog(audit_syslog_priority(handle), "chmod_acl %s mode 0x%x %s%s\n",
 	       path, mode,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -241,11 +280,16 @@ static int audit_chmod_acl(struct connec
 	return result;
 }
 
-static int audit_fchmod(struct files_struct *fsp, int fd, mode_t mode)
+static int audit_fchmod(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode)
 {
-	int result = default_vfs_ops.fchmod(fsp, fd, mode);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->fchmod(handle->next, fsp, fd, mode);
 
-	syslog(SYSLOG_PRIORITY, "fchmod %s mode 0x%x %s%s\n",
+	syslog(audit_syslog_priority(handle), "fchmod %s mode 0x%x %s%s\n",
 	       fsp->fsp_name, mode,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -253,16 +297,16 @@ static int audit_fchmod(struct files_str
 	return result;
 }
 
-static int audit_fchmod_acl(struct files_struct *fsp, int fd, mode_t mode)
+static int audit_fchmod_acl(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->fchmod_acl(handle->next, fsp, fd, mode);
 
-	if ( !default_vfs_ops.fchmod_acl )
-		return 0;
-
-	result = default_vfs_ops.fchmod_acl(fsp, fd, mode);
-
-	syslog(SYSLOG_PRIORITY, "fchmod_acl %s mode 0x%x %s%s\n",
+	syslog(audit_syslog_priority(handle), "fchmod_acl %s mode 0x%x %s%s\n",
 	       fsp->fsp_name, mode,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -272,5 +316,5 @@ static int audit_fchmod_acl(struct files
 
 int vfs_audit_init(void)
 {
-	return smb_register_vfs("audit", audit_init, SMB_VFS_INTERFACE_VERSION);
+	return smb_register_vfs("audit", audit_op_tuples, SMB_VFS_INTERFACE_VERSION);
 }
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/modules/vfs_extd_audit.c 3_0-vfs/source/modules/vfs_extd_audit.c
--- 3_0/source/modules/vfs_extd_audit.c	Wed Apr 16 18:19:49 2003
+++ 3_0-vfs/source/modules/vfs_extd_audit.c	Wed Apr 23 12:17:40 2003
@@ -5,6 +5,7 @@
  * Copyright (C) Tim Potter, 1999-2000
  * Copyright (C) Alexander Bokovoy, 2002
  * Copyright (C) John H Terpstra, 2003
+ * Copyright (C) Stefan (metze) Metzmacher, 2003
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,127 +22,130 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "config.h"
-#include <stdio.h>
-#include <sys/stat.h>
-#ifdef HAVE_UTIME_H
-#include <utime.h>
-#endif
-#ifdef HAVE_DIRENT_H
-#include <dirent.h>
-#endif
-#include <syslog.h>
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-#include <errno.h>
-#include <string.h>
-#include <includes.h>
-#include <vfs.h>
-
-#ifndef SYSLOG_FACILITY
-#define SYSLOG_FACILITY   LOG_USER
-#endif
-
-#ifndef SYSLOG_PRIORITY
-#define SYSLOG_PRIORITY   LOG_NOTICE
-#endif
+
+#include "includes.h"
+
+static int vfs_extd_audit_debug_level = DBGC_VFS;
+
+#undef DBGC_CLASS
+#define DBGC_CLASS vfs_extd_audit_debug_level
 
 /* Function prototypes */
 
-static int audit_connect(struct connection_struct *conn, const char *svc, const char *user);
-static void audit_disconnect(struct connection_struct *conn);
-static DIR *audit_opendir(struct connection_struct *conn, const char *fname);
-static int audit_mkdir(struct connection_struct *conn, const char *path, mode_t mode);
-static int audit_rmdir(struct connection_struct *conn, const char *path);
-static int audit_open(struct connection_struct *conn, const char *fname, int flags, mode_t mode);
-static int audit_close(struct files_struct *fsp, int fd);
-static int audit_rename(struct connection_struct *conn, const char *old, const char *new);
-static int audit_unlink(struct connection_struct *conn, const char *path);
-static int audit_chmod(struct connection_struct *conn, const char *path, mode_t mode);
-static int audit_chmod_acl(struct connection_struct *conn, const char *name, mode_t mode);
-static int audit_fchmod(struct files_struct *fsp, int fd, mode_t mode);
-static int audit_fchmod_acl(struct files_struct *fsp, int fd, mode_t mode);
+static int audit_connect(vfs_handle_struct *handle, connection_struct *conn, const char *svc, const char *user);
+static void audit_disconnect(vfs_handle_struct *handle, connection_struct *conn);
+static DIR *audit_opendir(vfs_handle_struct *handle, connection_struct *conn, const char *fname);
+static int audit_mkdir(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode);
+static int audit_rmdir(vfs_handle_struct *handle, connection_struct *conn, const char *path);
+static int audit_open(vfs_handle_struct *handle, connection_struct *conn, const char *fname, int flags, mode_t mode);
+static int audit_close(vfs_handle_struct *handle, files_struct *fsp, int fd);
+static int audit_rename(vfs_handle_struct *handle, connection_struct *conn, const char *old, const char *new);
+static int audit_unlink(vfs_handle_struct *handle, connection_struct *conn, const char *path);
+static int audit_chmod(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode);
+static int audit_chmod_acl(vfs_handle_struct *handle, connection_struct *conn, const char *name, mode_t mode);
+static int audit_fchmod(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode);
+static int audit_fchmod_acl(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode);
 
 /* VFS operations */
 
-static struct vfs_ops default_vfs_ops;   /* For passthrough operation */
-static struct smb_vfs_handle_struct *audit_handle;
-
-static vfs_op_tuple audit_ops[] = {
+static vfs_op_tuple audit_op_tuples[] = {
     
 	/* Disk operations */
 
-	{audit_connect, 	SMB_VFS_OP_CONNECT, 	SMB_VFS_LAYER_LOGGER},
-	{audit_disconnect, 	SMB_VFS_OP_DISCONNECT, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_connect), 	SMB_VFS_OP_CONNECT, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_disconnect), 	SMB_VFS_OP_DISCONNECT, 	SMB_VFS_LAYER_LOGGER},
 
 	/* Directory operations */
 
-	{audit_opendir, 	SMB_VFS_OP_OPENDIR, 	SMB_VFS_LAYER_LOGGER},
-	{audit_mkdir, 		SMB_VFS_OP_MKDIR, 	SMB_VFS_LAYER_LOGGER},
-	{audit_rmdir, 		SMB_VFS_OP_RMDIR, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_opendir), 	SMB_VFS_OP_OPENDIR, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_mkdir), 		SMB_VFS_OP_MKDIR, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_rmdir), 		SMB_VFS_OP_RMDIR, 	SMB_VFS_LAYER_LOGGER},
 
 	/* File operations */
 
-	{audit_open, 		SMB_VFS_OP_OPEN, 	SMB_VFS_LAYER_LOGGER},
-	{audit_close, 		SMB_VFS_OP_CLOSE, 	SMB_VFS_LAYER_LOGGER},
-	{audit_rename, 		SMB_VFS_OP_RENAME, 	SMB_VFS_LAYER_LOGGER},
-	{audit_unlink, 		SMB_VFS_OP_UNLINK, 	SMB_VFS_LAYER_LOGGER},
-	{audit_chmod, 		SMB_VFS_OP_CHMOD, 	SMB_VFS_LAYER_LOGGER},
-	{audit_fchmod, 		SMB_VFS_OP_FCHMOD, 	SMB_VFS_LAYER_LOGGER},
-	{audit_chmod_acl, 	SMB_VFS_OP_CHMOD_ACL, 	SMB_VFS_LAYER_LOGGER},
-	{audit_fchmod_acl, 	SMB_VFS_OP_FCHMOD_ACL, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_open), 		SMB_VFS_OP_OPEN, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_close), 		SMB_VFS_OP_CLOSE, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_rename), 		SMB_VFS_OP_RENAME, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_unlink), 		SMB_VFS_OP_UNLINK, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_chmod), 		SMB_VFS_OP_CHMOD, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_fchmod), 		SMB_VFS_OP_FCHMOD, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_chmod_acl), 	SMB_VFS_OP_CHMOD_ACL, 	SMB_VFS_LAYER_LOGGER},
+	{VFS_OP(audit_fchmod_acl), 	SMB_VFS_OP_FCHMOD_ACL, 	SMB_VFS_LAYER_LOGGER},
 	
 	/* Finish VFS operations definition */
 	
-	{NULL, 			SMB_VFS_OP_NOOP, 	SMB_VFS_LAYER_NOOP}
+	{VFS_OP(NULL), 			SMB_VFS_OP_NOOP, 	SMB_VFS_LAYER_NOOP}
 };
 
-/* VFS initialisation function.  Return vfs_op_tuple array back to SAMBA. */
 
-static vfs_op_tuple *audit_init(const struct vfs_ops *def_vfs_ops, 
-			struct smb_vfs_handle_struct *vfs_handle)
+static int audit_syslog_facility(vfs_handle_struct *handle)
 {
-	memcpy(&default_vfs_ops, def_vfs_ops, sizeof(struct vfs_ops));
-	
-	audit_handle = vfs_handle;
+	/* fix me: let this be configurable by:
+	 *	lp_param_enum(SNUM(handle->conn),(handle->param?handle->param:"extd_audit"),"syslog facility",
+	 *		audit_enum_facility,LOG_USER); 
+	 */
+	return LOG_USER;
+}
 
-	openlog("smbd_audit", LOG_PID, SYSLOG_FACILITY);
-	syslog(SYSLOG_PRIORITY, "VFS_INIT: vfs_ops loaded\n");
 
-	return audit_ops;
+static int audit_syslog_priority(vfs_handle_struct *handle)
+{
+	/* fix me: let this be configurable by:
+	 *	lp_param_enum(SNUM(handle->conn),(handle->param?handle->param:"extd_audit"),"syslog priority",
+	 *		audit_enum_priority,LOG_NOTICE); 
+	 */
+	return LOG_NOTICE;
 }
 
 /* Implementation of vfs_ops.  Pass everything on to the default
    operation but log event first. */
 
-static int audit_connect(struct connection_struct *conn, const char *svc, const char *user)
+static int audit_connect(vfs_handle_struct *handle, connection_struct *conn, const char *svc, const char *user)
 {
-	syslog(SYSLOG_PRIORITY, "connect to service %s by user %s\n", 
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	openlog("smbd_audit", LOG_PID, audit_syslog_facility(handle));
+
+	syslog(audit_syslog_priority(handle), "connect to service %s by user %s\n", 
 	       svc, user);
 	DEBUG(10, ("Connected to service %s as user %s\n",
 	       svc, user));
 
-	return default_vfs_ops.connect(conn, svc, user);
+	result = default_vfs_ops->connect(handle->next, conn, svc, user);
+
+	return result;
 }
 
-static void audit_disconnect(struct connection_struct *conn)
+static void audit_disconnect(vfs_handle_struct *handle, connection_struct *conn)
 {
-	syslog(SYSLOG_PRIORITY, "disconnected\n");
+	const struct vfs_ops *default_vfs_ops = NULL;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return);
+	
+	syslog(audit_syslog_priority(handle), "disconnected\n");
 	DEBUG(10, ("Disconnected from VFS module extd_audit\n"));
+	default_vfs_ops->disconnect(handle->next, conn);
 
-	default_vfs_ops.disconnect(conn);
+	return;
 }
 
-static DIR *audit_opendir(struct connection_struct *conn, const char *fname)
+static DIR *audit_opendir(vfs_handle_struct *handle, connection_struct *conn, const char *fname)
 {
-	DIR *result = default_vfs_ops.opendir(conn, fname);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	DIR *result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return NULL);
+	
+	result = default_vfs_ops->opendir(handle->next, conn, fname);
 
-	syslog(SYSLOG_PRIORITY, "opendir %s %s%s\n",
+	syslog(audit_syslog_priority(handle), "opendir %s %s%s\n",
 	       fname,
 	       (result == NULL) ? "failed: " : "",
 	       (result == NULL) ? strerror(errno) : "");
-	DEBUG(1, ("vfs_extd_audit: opendir %s %s %s",
+	DEBUG(1, ("vfs_extd_audit: opendir %s %s %s\n",
 	       fname,
 	       (result == NULL) ? "failed: " : "",
 	       (result == NULL) ? strerror(errno) : ""));
@@ -149,11 +153,16 @@ static DIR *audit_opendir(struct connect
 	return result;
 }
 
-static int audit_mkdir(struct connection_struct *conn, const char *path, mode_t mode)
+static int audit_mkdir(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode)
 {
-	int result = default_vfs_ops.mkdir(conn, path, mode);
-
-	syslog(SYSLOG_PRIORITY, "mkdir %s %s%s\n", 
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->mkdir(handle->next, conn, path, mode);
+	
+	syslog(audit_syslog_priority(handle), "mkdir %s %s%s\n", 
 	       path,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -165,11 +174,16 @@ static int audit_mkdir(struct connection
 	return result;
 }
 
-static int audit_rmdir(struct connection_struct *conn, const char *path)
+static int audit_rmdir(vfs_handle_struct *handle, connection_struct *conn, const char *path)
 {
-	int result = default_vfs_ops.rmdir(conn, path);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->rmdir(handle->next, conn, path);
 
-	syslog(SYSLOG_PRIORITY, "rmdir %s %s%s\n", 
+	syslog(audit_syslog_priority(handle), "rmdir %s %s%s\n", 
 	       path, 
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -181,11 +195,16 @@ static int audit_rmdir(struct connection
 	return result;
 }
 
-static int audit_open(struct connection_struct *conn, const char *fname, int flags, mode_t mode)
+static int audit_open(vfs_handle_struct *handle, connection_struct *conn, const char *fname, int flags, mode_t mode)
 {
-	int result = default_vfs_ops.open(conn, fname, flags, mode);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->open(handle->next, conn, fname, flags, mode);
 
-	syslog(SYSLOG_PRIORITY, "open %s (fd %d) %s%s%s\n", 
+	syslog(audit_syslog_priority(handle), "open %s (fd %d) %s%s%s\n", 
 	       fname, result,
 	       ((flags & O_WRONLY) || (flags & O_RDWR)) ? "for writing " : "", 
 	       (result < 0) ? "failed: " : "",
@@ -198,11 +217,16 @@ static int audit_open(struct connection_
 	return result;
 }
 
-static int audit_close(struct files_struct *fsp, int fd)
+static int audit_close(vfs_handle_struct *handle, files_struct *fsp, int fd)
 {
-	int result = default_vfs_ops.close(fsp, fd);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->close(handle->next, fsp, fd);
 
-	syslog(SYSLOG_PRIORITY, "close fd %d %s%s\n",
+	syslog(audit_syslog_priority(handle), "close fd %d %s%s\n",
 	       fd,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -214,11 +238,16 @@ static int audit_close(struct files_stru
 	return result;
 }
 
-static int audit_rename(struct connection_struct *conn, const char *old, const char *new)
+static int audit_rename(vfs_handle_struct *handle, connection_struct *conn, const char *old, const char *new)
 {
-	int result = default_vfs_ops.rename(conn, old, new);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->rename(handle->next, conn, old, new);
 
-	syslog(SYSLOG_PRIORITY, "rename %s -> %s %s%s\n",
+	syslog(audit_syslog_priority(handle), "rename %s -> %s %s%s\n",
 	       old, new,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -230,11 +259,16 @@ static int audit_rename(struct connectio
 	return result;    
 }
 
-static int audit_unlink(struct connection_struct *conn, const char *path)
+static int audit_unlink(vfs_handle_struct *handle, connection_struct *conn, const char *path)
 {
-	int result = default_vfs_ops.unlink(conn, path);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->unlink(handle->next, conn, path);
 
-	syslog(SYSLOG_PRIORITY, "unlink %s %s%s\n",
+	syslog(audit_syslog_priority(handle), "unlink %s %s%s\n",
 	       path,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -246,11 +280,16 @@ static int audit_unlink(struct connectio
 	return result;
 }
 
-static int audit_chmod(struct connection_struct *conn, const char *path, mode_t mode)
+static int audit_chmod(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode)
 {
-	int result = default_vfs_ops.chmod(conn, path, mode);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
 
-	syslog(SYSLOG_PRIORITY, "chmod %s mode 0x%x %s%s\n",
+	result = default_vfs_ops->chmod(handle->next, conn, path, mode);
+
+	syslog(audit_syslog_priority(handle), "chmod %s mode 0x%x %s%s\n",
 	       path, mode,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -262,11 +301,16 @@ static int audit_chmod(struct connection
 	return result;
 }
 
-static int audit_chmod_acl(struct connection_struct *conn, const char *path, mode_t mode)
+static int audit_chmod_acl(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode)
 {
-	int result = default_vfs_ops.chmod_acl(conn, path, mode);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->chmod_acl(handle->next, conn, path, mode);
 
-	syslog(SYSLOG_PRIORITY, "chmod_acl %s mode 0x%x %s%s\n",
+	syslog(audit_syslog_priority(handle), "chmod_acl %s mode 0x%x %s%s\n",
 	       path, mode,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -278,11 +322,16 @@ static int audit_chmod_acl(struct connec
 	return result;
 }
 
-static int audit_fchmod(struct files_struct *fsp, int fd, mode_t mode)
+static int audit_fchmod(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode)
 {
-	int result = default_vfs_ops.fchmod(fsp, fd, mode);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->fchmod(handle->next, fsp, fd, mode);
 
-	syslog(SYSLOG_PRIORITY, "fchmod %s mode 0x%x %s%s\n",
+	syslog(audit_syslog_priority(handle), "fchmod %s mode 0x%x %s%s\n",
 	       fsp->fsp_name, mode,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -294,11 +343,16 @@ static int audit_fchmod(struct files_str
 	return result;
 }
 
-static int audit_fchmod_acl(struct files_struct *fsp, int fd, mode_t mode)
+static int audit_fchmod_acl(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode)
 {
-	int result = default_vfs_ops.fchmod_acl(fsp, fd, mode);
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int result;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+	
+	result = default_vfs_ops->fchmod_acl(handle->next, fsp, fd, mode);
 
-	syslog(SYSLOG_PRIORITY, "fchmod_acl %s mode 0x%x %s%s\n",
+	syslog(audit_syslog_priority(handle), "fchmod_acl %s mode 0x%x %s%s\n",
 	       fsp->fsp_name, mode,
 	       (result < 0) ? "failed: " : "",
 	       (result < 0) ? strerror(errno) : "");
@@ -312,5 +366,18 @@ static int audit_fchmod_acl(struct files
 
 int vfs_extd_audit_init(void)
 {
-	return smb_register_vfs("extd_audit", audit_init, SMB_VFS_INTERFACE_VERSION);
+	BOOL ret = smb_register_vfs("extd_audit", audit_op_tuples, SMB_VFS_INTERFACE_VERSION);
+	
+	if (!ret)
+		return ret;
+
+	vfs_extd_audit_debug_level = debug_add_class("extd_audit");
+	if (vfs_extd_audit_debug_level == -1) {
+		vfs_extd_audit_debug_level = DBGC_VFS;
+		DEBUG(0, ("vfs_extd_audit: Couldn't register custom debugging class!\n"));
+	} else {
+		DEBUG(10, ("vfs_extd_audit: Debug class number of 'extd_audit': %d\n", vfs_extd_audit_debug_level));
+	}
+	
+	return ret;
 }
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/modules/vfs_fake_perms.c-mx.c 3_0-vfs/source/modules/vfs_fake_perms.c-mx.c
--- 3_0/source/modules/vfs_fake_perms.c-mx.c	Thu Jan  1 01:00:00 1970
+++ 3_0-vfs/source/modules/vfs_fake_perms.c-mx.c	Wed Apr 23 12:17:18 2003
@@ -0,0 +1,87 @@
+/* 
+ * Fake Perms VFS module.  Implements passthrough operation of all VFS
+ * calls to disk functions, except for file permissions, which are now
+ * mode 0700 for the current uid/gid.
+ *
+ * Copyright (C) Tim Potter, 1999-2000
+ * Copyright (C) Alexander Bokovoy, 2002
+ * Copyright (C) Andrew Bartlett, 2002
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *  
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *  
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
+#include "includes.h"
+
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_VFS
+
+static int fake_perms_stat(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *fname, SMB_STRUCT_STAT *sbuf)
+{
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int ret = -1;
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1); 
+	
+	ret = default_vfs_ops->stat(handle->next, conn, fname, sbuf);
+	if (ret == 0) {
+		extern struct current_user current_user;
+		
+		if (S_ISDIR(sbuf->st_mode)) {
+			sbuf->st_mode = S_IFDIR | S_IRWXU;
+		} else {
+			sbuf->st_mode = S_IRWXU;
+		}
+		sbuf->st_uid = current_user.uid;
+		sbuf->st_gid = current_user.gid;
+	}
+
+	return ret;
+}
+
+static int fake_perms_fstat(struct vfs_handle_struct *handle, struct files_struct *fsp, int fd, SMB_STRUCT_STAT *sbuf)
+{
+	const struct vfs_ops *default_vfs_ops = NULL;
+	int ret = -1;
+
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+
+	ret = default_vfs_ops->fstat(handle->next, fsp, fd, sbuf);
+	if (ret == 0) {
+		extern struct current_user current_user;
+		
+		if (S_ISDIR(sbuf->st_mode)) {
+			sbuf->st_mode = S_IFDIR | S_IRWXU;
+		} else {
+			sbuf->st_mode = S_IRWXU;
+		}
+		sbuf->st_uid = current_user.uid;
+		sbuf->st_gid = current_user.gid;
+	}
+	return ret;
+}
+
+/* VFS operations structure */
+
+static vfs_op_tuple fake_perms_ops[] = {	
+	{VFS_OP(fake_perms_stat),	SMB_VFS_OP_STAT,	SMB_VFS_LAYER_TRANSPARENT},
+	{VFS_OP(fake_perms_fstat),	SMB_VFS_OP_FSTAT,	SMB_VFS_LAYER_TRANSPARENT},
+
+	{NULL,				SMB_VFS_OP_NOOP,	SMB_VFS_LAYER_NOOP}
+};
+
+int vfs_fake_perms_init(void)
+{
+	return smb_register_vfs("fake_perms", fake_perms_ops, SMB_VFS_INTERFACE_VERSION);
+}
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/modules/vfs_netatalk.c 3_0-vfs/source/modules/vfs_netatalk.c
--- 3_0/source/modules/vfs_netatalk.c	Wed Apr 16 18:19:49 2003
+++ 3_0-vfs/source/modules/vfs_netatalk.c	Wed Apr 23 12:16:56 2003
@@ -2,6 +2,7 @@
  * AppleTalk VFS module for Samba-3.x
  *
  * Copyright (C) Alexei Kotovich, 2002
+ * Copyright (C) Stefan (metze) Metzmacher, 2003
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,22 +19,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "config.h"
-#include <stdio.h>
-#include <sys/stat.h>
-#ifdef HAVE_UTIME_H
-#include <utime.h>
-#endif
-#ifdef HAVE_DIRENT_H
-#include <dirent.h>
-#endif
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-#include <errno.h>
-#include <string.h>
-#include <includes.h>
-#include <vfs.h>
+#include "includes.h"
+
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_VFS
 
 #define APPLEDOUBLE	".AppleDouble"
 #define ADOUBLEMODE	0777
@@ -46,9 +35,6 @@ static int atalk_build_paths(TALLOC_CTX 
 
 static int atalk_unlink_file(const char *path);
 
-static struct vfs_ops default_vfs_ops;	/* For passthrough operation */
-static struct smb_vfs_handle_struct *atalk_handle;
-
 static int atalk_get_path_ptr(char *path)
 {
 	int i   = 0;
@@ -187,11 +173,14 @@ static void atalk_rrmdir(TALLOC_CTX *ctx
 
 /* Directory operations */
 
-DIR *atalk_opendir(struct connection_struct *conn, const char *fname)
+DIR *atalk_opendir(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *fname)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	DIR *ret = 0;
-	
-	ret = default_vfs_ops.opendir(conn, fname);
+
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return NULL);
+
+	ret = default_vfs_ops->opendir(handle->next, conn, fname);
 
 	/*
 	 * when we try to perform delete operation upon file which has fork
@@ -208,12 +197,15 @@ DIR *atalk_opendir(struct connection_str
 	return ret;
 }
 
-static int atalk_rmdir(struct connection_struct *conn, const char *path)
+static int atalk_rmdir(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	BOOL add = False;
 	TALLOC_CTX *ctx = 0;
 	char *dpath;
 
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+
 	if (!conn || !conn->origpath || !path) goto exit_rmdir;
 
 	/* due to there is no way to change bDeleteVetoFiles variable
@@ -233,13 +225,14 @@ static int atalk_rmdir(struct connection
 
 exit_rmdir:
 	talloc_destroy(ctx);
-	return default_vfs_ops.rmdir(conn, path);
+	return default_vfs_ops->rmdir(handle->next, conn, path);
 }
 
 /* File operations */
 
-static int atalk_rename(struct connection_struct *conn, const char *old, const char *new)
+static int atalk_rename(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *old, const char *new)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	int ret = 0;
 	char *adbl_path = 0;
 	char *orig_path = 0;
@@ -247,7 +240,9 @@ static int atalk_rename(struct connectio
 	SMB_STRUCT_STAT orig_info;
 	TALLOC_CTX *ctx;
 
-	ret = default_vfs_ops.rename(conn, old, new);
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+
+	ret = default_vfs_ops->rename(handle->next, conn, old, new);
 
 	if (!conn || !old) return ret;
 
@@ -270,8 +265,9 @@ exit_rename:
 	return ret;
 }
 
-static int atalk_unlink(struct connection_struct *conn, const char *path)
+static int atalk_unlink(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	int ret = 0, i;
 	char *adbl_path = 0;
 	char *orig_path = 0;
@@ -279,7 +275,9 @@ static int atalk_unlink(struct connectio
 	SMB_STRUCT_STAT orig_info;
 	TALLOC_CTX *ctx;
 
-	ret = default_vfs_ops.unlink(conn, path);
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+
+	ret = default_vfs_ops->unlink(handle->next, conn, path);
 
 	if (!conn || !path) return ret;
 
@@ -326,8 +324,9 @@ exit_unlink:	
 	return ret;
 }
 
-static int atalk_chmod(struct connection_struct *conn, const char *path, mode_t mode)
+static int atalk_chmod(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, mode_t mode)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	int ret = 0;
 	char *adbl_path = 0;
 	char *orig_path = 0;
@@ -335,7 +334,9 @@ static int atalk_chmod(struct connection
 	SMB_STRUCT_STAT orig_info;
 	TALLOC_CTX *ctx;
 
-	ret = default_vfs_ops.chmod(conn, path, mode);
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+
+	ret = default_vfs_ops->chmod(handle->next, conn, path, mode);
 
 	if (!conn || !path) return ret;
 
@@ -358,8 +359,9 @@ exit_chmod:	
 	return ret;
 }
 
-static int atalk_chown(struct connection_struct *conn, const char *path, uid_t uid, gid_t gid)
+static int atalk_chown(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *path, uid_t uid, gid_t gid)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	int ret = 0;
 	char *adbl_path = 0;
 	char *orig_path = 0;
@@ -367,7 +369,9 @@ static int atalk_chown(struct connection
 	SMB_STRUCT_STAT orig_info;
 	TALLOC_CTX *ctx;
 
-	ret = default_vfs_ops.chown(conn, path, uid, gid);
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
+
+	ret = default_vfs_ops->chown(handle->next, conn, path, uid, gid);
 
 	if (!conn || !path) return ret;
 
@@ -394,34 +398,22 @@ static vfs_op_tuple atalk_ops[] = {
     
 	/* Directory operations */
 
-	{atalk_opendir, 	SMB_VFS_OP_OPENDIR, 	SMB_VFS_LAYER_TRANSPARENT},
-	{atalk_rmdir, 		SMB_VFS_OP_RMDIR, 	SMB_VFS_LAYER_TRANSPARENT},
+	{VFS_OP(atalk_opendir), 	SMB_VFS_OP_OPENDIR, 	SMB_VFS_LAYER_TRANSPARENT},
+	{VFS_OP(atalk_rmdir), 		SMB_VFS_OP_RMDIR, 	SMB_VFS_LAYER_TRANSPARENT},
 
 	/* File operations */
 
-	{atalk_rename, 		SMB_VFS_OP_RENAME, 	SMB_VFS_LAYER_TRANSPARENT},
-	{atalk_unlink, 		SMB_VFS_OP_UNLINK, 	SMB_VFS_LAYER_TRANSPARENT},
-	{atalk_chmod, 		SMB_VFS_OP_CHMOD, 	SMB_VFS_LAYER_TRANSPARENT},
-	{atalk_chown,		SMB_VFS_OP_CHOWN,	SMB_VFS_LAYER_TRANSPARENT},
+	{VFS_OP(atalk_rename), 		SMB_VFS_OP_RENAME, 	SMB_VFS_LAYER_TRANSPARENT},
+	{VFS_OP(atalk_unlink), 		SMB_VFS_OP_UNLINK, 	SMB_VFS_LAYER_TRANSPARENT},
+	{VFS_OP(atalk_chmod), 		SMB_VFS_OP_CHMOD, 	SMB_VFS_LAYER_TRANSPARENT},
+	{VFS_OP(atalk_chown),		SMB_VFS_OP_CHOWN,	SMB_VFS_LAYER_TRANSPARENT},
 	
 	/* Finish VFS operations definition */
 	
-	{NULL, 			SMB_VFS_OP_NOOP, 	SMB_VFS_LAYER_NOOP}
+	{VFS_OP(NULL), 			SMB_VFS_OP_NOOP, 	SMB_VFS_LAYER_NOOP}
 };
 
-/* VFS initialisation function.  Return vfs_op_tuple array back to SAMBA. */
-static vfs_op_tuple *netatalk_init(const struct vfs_ops *def_vfs_ops,
-  struct smb_vfs_handle_struct *vfs_handle)
-{
-	memcpy(&default_vfs_ops, def_vfs_ops, sizeof(struct vfs_ops));
-	
-	atalk_handle = vfs_handle;
-
-	DEBUG(3, ("ATALK: vfs module loaded\n"));
-	return atalk_ops;
-}
-
 int vfs_netatalk_init(void)
 {
-	return smb_register_vfs("netatalk", netatalk_init, SMB_VFS_INTERFACE_VERSION);
+	return smb_register_vfs("netatalk", atalk_ops, SMB_VFS_INTERFACE_VERSION);
 }
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/modules/vfs_recycle.c 3_0-vfs/source/modules/vfs_recycle.c
--- 3_0/source/modules/vfs_recycle.c	Thu Apr 24 07:01:00 2003
+++ 3_0-vfs/source/modules/vfs_recycle.c	Thu Apr 24 07:21:51 2003
@@ -6,6 +6,7 @@
  * Copyright (C) 2002, Alexander Bokovoy - cascaded VFS adoption,
  * Copyright (C) 2002, Juergen Hasch - added some options.
  * Copyright (C) 2002, Simo Sorce
+ * Copyright (C) 2002, Stefan (metze) Metzmacher
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,45 +31,10 @@ static int vfs_recycle_debug_level = DBG
 
 #undef DBGC_CLASS
 #define DBGC_CLASS vfs_recycle_debug_level
-
-static const char *delimiter = "|";		/* delimiter for options */
-
-/* One per connection */
-
-typedef struct recycle_bin_struct
-{
-	TALLOC_CTX *mem_ctx;
-	char	*repository;		/* name of the recycle bin directory */
-	BOOL	keep_dir_tree;		/* keep directory structure of deleted file in recycle bin */
-	BOOL	versions;		/* create versions of deleted files with identical name */
-	BOOL	touch;			/* touch access date of deleted file */
-	char	*exclude;		/* which files to exclude */
-	char	*exclude_dir;		/* which directories to exclude */
-	char	*noversions;		/* which files to exclude from versioning */
-	SMB_OFF_T maxsize;		/* maximum file size to be saved */
-} recycle_bin_struct;
-
-typedef struct recycle_bin_connections {
-	int conn;
-	recycle_bin_struct *data;
-	struct recycle_bin_connections *next;
-} recycle_bin_connections;
-
-typedef struct recycle_bin_private_data {
-	TALLOC_CTX *mem_ctx;
-	recycle_bin_connections *conns;
-} recycle_bin_private_data;
-
-struct smb_vfs_handle_struct *recycle_bin_private_handle;
-
-/* VFS operations */
-static struct vfs_ops default_vfs_ops;   /* For passthrough operation */
-
-static int recycle_connect(struct connection_struct *conn, const char *service, const char *user);
-static void recycle_disconnect(struct connection_struct *conn);
-static int recycle_unlink(connection_struct *, const char *);
-
-#define VFS_OP(x) ((void *) x)
+ 
+static int recycle_connect(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *service, const char *user);
+static void recycle_disconnect(struct vfs_handle_struct *handle, struct connection_struct *conn);
+static int recycle_unlink(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *name);
 
 static vfs_op_tuple recycle_ops[] = {
 
@@ -82,179 +48,127 @@ static vfs_op_tuple recycle_ops[] = {
 	{NULL,				SMB_VFS_OP_NOOP,	SMB_VFS_LAYER_NOOP}
 };
 
-/**
- * VFS initialisation function.
- *
- * @retval initialised vfs_op_tuple array
- **/
-static vfs_op_tuple *recycle_init(const struct vfs_ops *def_vfs_ops,
-			struct smb_vfs_handle_struct *vfs_handle)
+static int recycle_connect(struct vfs_handle_struct *handle, struct connection_struct *conn, const char *service, const char *user)
 {
-	TALLOC_CTX *mem_ctx = NULL;
+	const struct vfs_ops *default_vfs_ops = NULL;
 
-	DEBUG(10, ("Initializing VFS module recycle\n"));
-	memcpy(&default_vfs_ops, def_vfs_ops, sizeof(struct vfs_ops));
-	vfs_recycle_debug_level = debug_add_class("vfs_recycle_bin");
-	if (vfs_recycle_debug_level == -1) {
-		vfs_recycle_debug_level = DBGC_VFS;
-		DEBUG(0, ("vfs_recycle: Couldn't register custom debugging class!\n"));
-	} else {
-		DEBUG(0, ("vfs_recycle: Debug class number of 'vfs_recycle': %d\n", vfs_recycle_debug_level));
-	}
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
 
-	recycle_bin_private_handle = vfs_handle;
-	if (!(mem_ctx = talloc_init("recycle bin data"))) {
-		DEBUG(0, ("Failed to allocate memory in VFS module recycle_bin\n"));
-		return NULL;
-	}
+	DEBUG(10,("recycle_connect() connect to service[%s] as user[%s].\n",
+		service,user));
 
-	recycle_bin_private_handle->data = talloc(mem_ctx, sizeof(recycle_bin_private_data));
-	if (recycle_bin_private_handle->data == NULL) {
-		DEBUG(0, ("Failed to allocate memory in VFS module recycle_bin\n"));
-		return NULL;
-	}
-	((recycle_bin_private_data *)(recycle_bin_private_handle->data))->mem_ctx = mem_ctx;
-	((recycle_bin_private_data *)(recycle_bin_private_handle->data))->conns = NULL;
+	return default_vfs_ops->connect(handle->next, conn, service, user);		
+}
+
+static void recycle_disconnect(struct vfs_handle_struct *handle, struct connection_struct *conn)
+{
+	const struct vfs_ops *default_vfs_ops = NULL;
+
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return);
 
-	return recycle_ops;
+	DEBUG(10,("recycle_disconnect() connect to service[%s].\n",
+		lp_servicename(SNUM(conn))));
+
+	default_vfs_ops->disconnect(handle->next, conn);	
 }
 
-static int recycle_connect(struct connection_struct *conn, const char *service, const char *user)
+static const char *recycle_repository(struct connection_struct *conn)
 {
-	TALLOC_CTX *ctx = NULL;
-	recycle_bin_struct *recbin;
-	recycle_bin_connections *recconn;
-	recycle_bin_connections *recconnbase;
-	recycle_bin_private_data *recdata;
-	const char *tmp_str;
+	const char *tmp_str = NULL;
+	
 
-	DEBUG(10, ("Called for service %s (%d) as user %s\n", service, SNUM(conn), user));
+	tmp_str = lp_parm_const_string(SNUM(conn), "recycle", "repository",".recycle");
 
-	if (recycle_bin_private_handle)
-		recdata = (recycle_bin_private_data *)(recycle_bin_private_handle->data);
-	else {
-		DEBUG(0, ("Recycle bin not initialized!\n"));
-		return -1;
-	}
+	DEBUG(10, ("recycle: repository = %s\n", tmp_str));
+	
+	return tmp_str;
+}
 
-	if (!(ctx = talloc_init("recycle bin connection"))) {
-		DEBUG(0, ("Failed to allocate memory in VFS module recycle_bin\n"));
-		return -1;
-	}
+static BOOL recycle_keep_dir_tree(struct connection_struct *conn)
+{
+	BOOL ret;
+	
+	ret = lp_parm_bool(SNUM(conn), "recycle", "keeptree", False);
 
-	recbin = talloc_zero(ctx, sizeof(recycle_bin_struct));
-	if (recbin == NULL) {
-		DEBUG(0, ("Failed to allocate memory in VFS module recycle_bin\n"));
-		return -1;
-	}
-	recbin->mem_ctx = ctx;
+	DEBUG(10, ("recycle_bin: keeptree = %s\n", ret?"True":"False"));
+	
+	return ret;
+}
 
-	/* parse configuration options */
-	if ((tmp_str = lp_parm_const_string(SNUM(conn), "vfs_recycle_bin", "repository", ".recycle")) != NULL) {
-		recbin->repository = talloc_sub_conn(recbin->mem_ctx, conn, tmp_str);
-		ALLOC_CHECK(recbin->repository, error);
-		trim_string(recbin->repository, "/", "/");
-		DEBUG(5, ("recycle.bin: repository = %s\n", recbin->repository));
-	} else {
-		DEBUG(0,("recycle.bin: no repository found (fail) !\n"));
-		goto error;
-	}
+static BOOL recycle_versions(struct connection_struct *conn)
+{
+	BOOL ret;
+
+	ret = lp_parm_bool(SNUM(conn), "recycle", "versions", False);
+
+	DEBUG(10, ("recycle: versions = %s\n", ret?"True":"False"));
 	
-	recbin->keep_dir_tree = lp_parm_bool(SNUM(conn), "vfs_recycle_bin", "keeptree", False);
-	DEBUG(5, ("recycle.bin: keeptree = %d\n", recbin->keep_dir_tree));
+	return ret;
+}
+
+static BOOL recycle_touch(struct connection_struct *conn)
+{
+	BOOL ret;
+
+	ret = lp_parm_bool(SNUM(conn), "recycle", "touch", False);
+
+	DEBUG(10, ("recycle: touch = %s\n", ret?"True":"False"));
 	
-	recbin->versions = lp_parm_bool(SNUM(conn), "vfs_recycle_bin", "versions", False);
-	DEBUG(5, ("recycle.bin: versions = %d\n", recbin->versions));
+	return ret;
+}
+
+static const char **recycle_exclude(struct connection_struct *conn)
+{
+	const char **tmp_lp;
 	
-	recbin->touch = lp_parm_bool(SNUM(conn), "vfs_recycle_bin", "touch", False);
-	DEBUG(5, ("recycle.bin: touch = %d\n", recbin->touch));
+	tmp_lp = lp_parm_string_list(SNUM(conn), "recycle", "exclude", NULL);
 
-	recbin->maxsize = lp_parm_ulong(SNUM(conn), "vfs_recycle_bin", "maxsize" , 0);
-	if (recbin->maxsize == 0) {
-		recbin->maxsize = -1;
-		DEBUG(5, ("recycle.bin: maxsize = -infinite-\n"));
-	} else {
-		DEBUG(5, ("recycle.bin: maxsize = %ld\n", (long int)recbin->maxsize));
-	}
+	DEBUG(10, ("recycle: exclude = %s ...\n", tmp_lp?*tmp_lp:""));
+	
+	return tmp_lp;
+}
 
-	if ((tmp_str = lp_parm_const_string(SNUM(conn), "vfs_recycle_bin", "exclude", "")) != NULL) {
-		recbin->exclude = talloc_strdup(recbin->mem_ctx, tmp_str);
-		ALLOC_CHECK(recbin->exclude, error);
-		DEBUG(5, ("recycle.bin: exclude = %s\n", recbin->exclude));
-	}
-	if ((tmp_str = lp_parm_const_string(SNUM(conn), "vfs_recycle_bin", "exclude_dir", "")) != NULL) {
-		recbin->exclude_dir = talloc_strdup(recbin->mem_ctx, tmp_str);
-		ALLOC_CHECK(recbin->exclude_dir, error);
-		DEBUG(5, ("recycle.bin: exclude_dir = %s\n", recbin->exclude_dir));
-	}
-	if ((tmp_str = lp_parm_const_string(SNUM(conn), "vfs_recycle_bin", "noversions", "")) != NULL) {
-		recbin->noversions = talloc_strdup(recbin->mem_ctx, tmp_str);
-		ALLOC_CHECK(recbin->noversions, error);
-		DEBUG(5, ("recycle.bin: noversions = %s\n", recbin->noversions));
-	}
-
-	recconn = talloc(recdata->mem_ctx, sizeof(recycle_bin_connections));
-	if (recconn == NULL) {
-		DEBUG(0, ("Failed to allocate memory in VFS module recycle_bin\n"));
-		goto error;
-	}
-	recconn->conn = SNUM(conn);
-	recconn->data = recbin;
-	recconn->next = NULL;
-	if (recdata->conns) {
-		recconnbase = recdata->conns;
-		while (recconnbase->next != NULL) recconnbase = recconnbase->next;
-		recconnbase->next = recconn;
-	} else {
-		recdata->conns = recconn;
-	}
-	return default_vfs_ops.connect(conn, service, user);
+static const char **recycle_exclude_dir(struct connection_struct *conn)
+{
+	const char **tmp_lp;
+	
+	tmp_lp = lp_parm_string_list(SNUM(conn), "recycle", "exclude_dir", NULL);
 
-error:
-	talloc_destroy(ctx);
-	return -1;
+	DEBUG(10, ("recycle: exclude_dir = %s ...\n", tmp_lp?*tmp_lp:""));
+	
+	return tmp_lp;
 }
 
-static void recycle_disconnect(struct connection_struct *conn)
+static const char **recycle_noversions(struct connection_struct *conn)
 {
-	recycle_bin_private_data *recdata;
-	recycle_bin_connections *recconn;
+	const char **tmp_lp;
+	
+	tmp_lp = lp_parm_string_list(SNUM(conn), "recycle", "noversions", NULL);
 
-	DEBUG(10, ("Disconnecting VFS module recycle bin\n"));
+	DEBUG(10, ("recycle: noversions = %s\n", tmp_lp?*tmp_lp:""));
+	
+	return tmp_lp;
+}
 
-	if (recycle_bin_private_handle)
-		recdata = (recycle_bin_private_data *)(recycle_bin_private_handle->data);
-	else {
-		DEBUG(0, ("Recycle bin not initialized!\n"));
-		return;
-	}
+static int recycle_maxsize(struct connection_struct *conn)
+{
+	int maxsize;
+	
+	maxsize = lp_parm_int(SNUM(conn), "recycle", "maxsize", -1);
 
-	if (recdata) {
-		if (recdata->conns) {
-			if (recdata->conns->conn == SNUM(conn)) {
-				talloc_destroy(recdata->conns->data->mem_ctx);
-				recdata->conns = recdata->conns->next;
-			} else {
-				recconn = recdata->conns;
-				while (recconn->next) {
-					if (recconn->next->conn == SNUM(conn)) {
-						talloc_destroy(recconn->next->data->mem_ctx);
-						recconn->next = recconn->next->next;
-						break;
-					}
-					recconn = recconn->next;
-				}
-			}
-		}
-	}
-	default_vfs_ops.disconnect(conn);
+	DEBUG(10, ("recycle: maxsize = %d\n", maxsize));
+	
+	return maxsize;
 }
 
-static BOOL recycle_directory_exist(connection_struct *conn, const char *dname)
+static BOOL recycle_directory_exist(vfs_handle_struct *handle, const char *dname)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	SMB_STRUCT_STAT st;
 
-	if (default_vfs_ops.stat(conn, dname, &st) == 0) {
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return False);
+
+	if (default_vfs_ops->stat(handle->next, handle->conn, dname, &st) == 0) {
 		if (S_ISDIR(st.st_mode)) {
 			return True;
 		}
@@ -263,11 +177,14 @@ static BOOL recycle_directory_exist(conn
 	return False;
 }
 
-static BOOL recycle_file_exist(connection_struct *conn, const char *fname)
+static BOOL recycle_file_exist(vfs_handle_struct *handle, const char *fname)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	SMB_STRUCT_STAT st;
 
-	if (default_vfs_ops.stat(conn, fname, &st) == 0) {
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return False);
+
+	if (default_vfs_ops->stat(handle->next, handle->conn, fname, &st) == 0) {
 		if (S_ISREG(st.st_mode)) {
 			return True;
 		}
@@ -282,13 +199,18 @@ static BOOL recycle_file_exist(connectio
  * @param fname file name
  * @return size in bytes
  **/
-static SMB_OFF_T recycle_get_file_size(connection_struct *conn, const char *fname)
+static SMB_OFF_T recycle_get_file_size(vfs_handle_struct *handle, const char *fname)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	SMB_STRUCT_STAT st;
-	if (default_vfs_ops.stat(conn, fname, &st) != 0) {
-		DEBUG(0,("recycle.bin: stat for %s returned %s\n", fname, strerror(errno)));
+	
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return (SMB_OFF_T)-1);
+
+	if (default_vfs_ops->stat(handle->next, handle->conn, fname, &st) != 0) {
+		DEBUG(0,("recycle: stat for %s returned %s\n", fname, strerror(errno)));
 		return (SMB_OFF_T)0;
 	}
+
 	return(st.st_size);
 }
 
@@ -298,8 +220,9 @@ static SMB_OFF_T recycle_get_file_size(c
  * @param dname Directory tree to be created
  * @return Returns True for success
  **/
-static BOOL recycle_create_dir(connection_struct *conn, const char *dname)
+static BOOL recycle_create_dir(vfs_handle_struct *handle, const char *dname)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	int len;
 	mode_t mode;
 	char *new_dir = NULL;
@@ -308,6 +231,8 @@ static BOOL recycle_create_dir(connectio
 	char *tok_str;
 	BOOL ret = False;
 
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return False);
+
 	mode = S_IREAD | S_IWRITE | S_IEXEC;
 
 	tmp_str = strdup(dname);
@@ -322,18 +247,18 @@ static BOOL recycle_create_dir(connectio
 	/* Create directory tree if neccessary */
 	for(token = strtok(tok_str, "/"); token; token = strtok(NULL, "/")) {
 		safe_strcat(new_dir, token, len);
-		if (recycle_directory_exist(conn, new_dir))
-			DEBUG(10, ("recycle.bin: dir %s already exists\n", new_dir));
+		if (recycle_directory_exist(handle, new_dir))
+			DEBUG(10, ("recycle: dir %s already exists\n", new_dir));
 		else {
-			DEBUG(5, ("recycle.bin: creating new dir %s\n", new_dir));
-			if (default_vfs_ops.mkdir(conn, new_dir, mode) != 0) {
-				DEBUG(1,("recycle.bin: mkdir failed for %s with error: %s\n", new_dir, strerror(errno)));
+			DEBUG(5, ("recycle: creating new dir %s\n", new_dir));
+			if (default_vfs_ops->mkdir(handle->next, handle->conn, new_dir, mode) != 0) {
+				DEBUG(1,("recycle: mkdir failed for %s with error: %s\n", new_dir, strerror(errno)));
 				ret = False;
 				goto done;
 			}
 		}
 		safe_strcat(new_dir, "/", len);
-		}
+	}
 
 	ret = True;
 done:
@@ -348,30 +273,22 @@ done:
  * @param needle string to be matched exactly to haystack
  * @return True if found
  **/
-static BOOL checkparam(const char *haystack, const char *needle)
+static BOOL checkparam(const char **haystack_list, const char *needle)
 {
-	char *token;
-	char *tok_str;
-	char *tmp_str;
-	BOOL ret = False;
+	int i;
 
-	if (haystack == NULL || strlen(haystack) == 0 || needle == NULL || strlen(needle) == 0) {
+	if (haystack_list == NULL || haystack_list[0] == NULL ||
+		*haystack_list[0] == '\0' || needle == NULL || *needle == '\0') {
 		return False;
 	}
 
-	tmp_str = strdup(haystack);
-	ALLOC_CHECK(tmp_str, done);
-	token = tok_str = tmp_str;
-
-	for(token = strtok(tok_str, delimiter); token; token = strtok(NULL, delimiter)) {
-		if(strcmp(token, needle) == 0) {
-			ret = True;
-			goto done;
+	for(i=0; haystack_list[i] ; i++) {
+		if(strequal(haystack_list[i], needle)) {
+			return True;
 		}
 	}
-done:
-	SAFE_FREE(tmp_str);
-	return ret;
+
+	return False;
 }
 
 /**
@@ -380,110 +297,93 @@ done:
  * @param needle string to be matched exectly to haystack including pattern matching
  * @return True if found
  **/
-static BOOL matchparam(const char *haystack, const char *needle)
+static BOOL matchparam(const char **haystack_list, const char *needle)
 {
-	char *token;
-	char *tok_str;
-	char *tmp_str;
-	BOOL ret = False;
+	int i;
 
-	if (haystack == NULL || strlen(haystack) == 0 || needle == NULL || strlen(needle) == 0) {
+	if (haystack_list == NULL || haystack_list[0] == NULL ||
+		*haystack_list[0] == '\0' || needle == NULL || *needle == '\0') {
 		return False;
 	}
 
-	tmp_str = strdup(haystack);
-	ALLOC_CHECK(tmp_str, done);
-	token = tok_str = tmp_str;
-
-	for(token = strtok(tok_str, delimiter); token; token = strtok(NULL, delimiter)) {
-		if (!unix_wild_match(token, needle)) {
-			ret = True;
-			goto done;
+	for(i=0; haystack_list[i] ; i++) {
+		if(!unix_wild_match((char *)haystack_list[i], (char *)needle)) {
+			return True;
 		}
 	}
-done:
-	SAFE_FREE(tmp_str);
-	return ret;
+
+	return False;
 }
 
 /**
  * Touch access date
  **/
-static void recycle_touch(connection_struct *conn, const char *fname)
+static void recycle_do_touch(vfs_handle_struct *handle, const char *fname)
 {
+	const struct vfs_ops *default_vfs_ops = NULL;
 	SMB_STRUCT_STAT st;
 	struct utimbuf tb;
 	time_t currtime;
 
-	if (default_vfs_ops.stat(conn, fname, &st) != 0) {
-		DEBUG(0,("recycle.bin: stat for %s returned %s\n", fname, strerror(errno)));
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return);
+	
+	if (default_vfs_ops->stat(handle->next, handle->conn, fname, &st) != 0) {
+		DEBUG(0,("recycle: stat for %s returned %s\n", fname, strerror(errno)));
 		return;
 	}
 	currtime = time(&currtime);
 	tb.actime = currtime;
 	tb.modtime = st.st_mtime;
 
-	if (default_vfs_ops.utime(conn, fname, &tb) == -1 )
-		DEBUG(0, ("recycle.bin: touching %s failed, reason = %s\n", fname, strerror(errno)));
+	if (default_vfs_ops->utime(handle->next, handle->conn, fname, &tb) == -1 ) {
+		DEBUG(0, ("recycle: touching %s failed, reason = %s\n", fname, strerror(errno)));
 	}
+}
 
 /**
  * Check if file should be recycled
  **/
-static int recycle_unlink(connection_struct *conn, const char *file_name)
+static int recycle_unlink(vfs_handle_struct *handle, connection_struct *conn, const char *file_name)
 {
-	recycle_bin_private_data *recdata;
-	recycle_bin_connections *recconn;
-	recycle_bin_struct *recbin;
+	const struct vfs_ops *default_vfs_ops = NULL;
 	char *path_name = NULL;
        	char *temp_name = NULL;
 	char *final_name = NULL;
 	const char *base;
-	int i;
-/*	SMB_BIG_UINT dfree, dsize, bsize;	*/
+	char *repository = NULL;
+	int i = 1;
+	int maxsize;
 	SMB_OFF_T file_size; /* space_avail;	*/
 	BOOL exist;
 	int rc = -1;
 
-	recbin = NULL;
-	if (recycle_bin_private_handle) {
-		recdata = (recycle_bin_private_data *)(recycle_bin_private_handle->data);
-		if (recdata) {
-			if (recdata->conns) {
-				recconn = recdata->conns;
-				while (recconn && recconn->conn != SNUM(conn)) recconn = recconn->next;
-				if (recconn != NULL) {
-					recbin = recconn->data;
-				}
-			}
-		}
-	}
-	if (recbin == NULL) {
-		DEBUG(0, ("Recycle bin not initialized!\n"));
-		rc = default_vfs_ops.unlink(conn, file_name);
-		goto done;
-	}
+	VFS_HANDLE_GET_OPS(handle, default_vfs_ops, return -1);
 
-	if(!recbin->repository || *(recbin->repository) == '\0') {
-		DEBUG(3, ("Recycle path not set, purging %s...\n", file_name));
-		rc = default_vfs_ops.unlink(conn, file_name);
+	repository = alloc_sub_conn(conn, (char *)recycle_repository(conn));
+	ALLOC_CHECK(repository, done);
+	/* shouldn't we allow absolute path names here? --metze */
+	trim_string(repository, "/", "/");
+	
+	if(!repository || *(repository) == '\0') {
+		DEBUG(3, ("recycle: repository path not set, purging %s...\n", file_name));
+		rc = default_vfs_ops->unlink(handle->next, conn, file_name);
 		goto done;
 	}
 
 	/* we don't recycle the recycle bin... */
-	if (strncmp(file_name, recbin->repository, strlen(recbin->repository)) == 0) {
-		DEBUG(3, ("File is within recycling bin, unlinking ...\n"));
-		rc = default_vfs_ops.unlink(conn, file_name);
+	if (strncmp(file_name, repository, strlen(repository)) == 0) {
+		DEBUG(3, ("recycle: File is within recycling bin, unlinking ...\n"));
+		rc = default_vfs_ops->unlink(handle->next, conn, file_name);
 		goto done;
 	}
 
-	file_size = recycle_get_file_size(conn, file_name);
+	file_size = recycle_get_file_size(handle, file_name);
 	/* it is wrong to purge filenames only because they are empty imho
 	 *   --- simo
 	 *
 	if(fsize == 0) {
-		DEBUG(3, ("File %s is empty, purging...\n", file_name));
-		rc = default_vfs_ops.unlink(conn,file_name);
+		DEBUG(3, ("recycle: File %s is empty, purging...\n", file_name));
+		rc = default_vfs_ops->unlink(handle->next,conn,file_name);
 		goto done;
 	}
 	 */
@@ -492,20 +392,21 @@ static int recycle_unlink(connection_str
 	 * not greater then maxsize, not the size of the single file, also it is better
 	 * to remove older files
 	 */
-	if(recbin->maxsize > 0 && file_size > recbin->maxsize) {
-		DEBUG(3, ("File %s exceeds maximum recycle size, purging... \n", file_name));
-		rc = default_vfs_ops.unlink(conn, file_name);
+	maxsize = recycle_maxsize(conn);
+	if(maxsize > 0 && file_size > maxsize) {
+		DEBUG(3, ("recycle: File %s exceeds maximum recycle size, purging... \n", file_name));
+		rc = default_vfs_ops->unlink(handle, conn, file_name);
 		goto done;
 	}
 
 	/* FIXME: this is wrong: moving files with rename does not change the disk space
 	 * allocation
 	 *
-	space_avail = default_vfs_ops.disk_free(conn, ".", True, &bsize, &dfree, &dsize) * 1024L;
+	space_avail = default_vfs_ops->disk_free(handle->next, conn, ".", True, &bsize, &dfree, &dsize) * 1024L;
 	DEBUG(5, ("space_avail = %Lu, file_size = %Lu\n", space_avail, file_size));
 	if(space_avail < file_size) {
-		DEBUG(3, ("Not enough diskspace, purging file %s\n", file_name));
-		rc = default_vfs_ops.unlink(conn, file_name);
+		DEBUG(3, ("recycle: Not enough diskspace, purging file %s\n", file_name));
+		rc = default_vfs_ops->unlink(handle->next, conn, file_name);
 		goto done;
 	}
 	 */
@@ -524,13 +425,13 @@ static int recycle_unlink(connection_str
 		base++;
 	}
 
-	DEBUG(10, ("recycle.bin: fname = %s\n", file_name));	/* original filename with path */
-	DEBUG(10, ("recycle.bin: fpath = %s\n", path_name));	/* original path */
-	DEBUG(10, ("recycle.bin: base = %s\n", base));		/* filename without path */
-
-	if (matchparam(recbin->exclude, base)) {
-		DEBUG(3, ("recycle.bin: file %s is excluded \n", base));
-		rc = default_vfs_ops.unlink(conn, file_name);
+	DEBUG(10, ("recycle: fname = %s\n", file_name));	/* original filename with path */
+	DEBUG(10, ("recycle: fpath = %s\n", path_name));	/* original path */
+	DEBUG(10, ("recycle: base = %s\n", base));		/* filename without path */
+
+	if (matchparam(recycle_exclude(conn), base)) {
+		DEBUG(3, ("recycle: file %s is excluded \n", base));
+		rc = default_vfs_ops->unlink(handle->next, conn, file_name);
 		goto done;
 	}
 
@@ -538,72 +439,82 @@ static int recycle_unlink(connection_str
 	 * we shoud check for every level 1, 1/2, 1/2/3, 1/2/3/4 .... 
 	 * 	---simo
 	 */
-	if (checkparam(recbin->exclude_dir, path_name)) {
-		DEBUG(3, ("recycle.bin: directory %s is excluded \n", path_name));
-		rc = default_vfs_ops.unlink(conn, file_name);
+	if (checkparam(recycle_exclude_dir(conn), path_name)) {
+		DEBUG(3, ("recycle: directory %s is excluded \n", path_name));
+		rc = default_vfs_ops->unlink(handle->next, conn, file_name);
 		goto done;
 	}
 
-	/* see if we need to recreate the original directory structure in the recycle bin */
-	if (recbin->keep_dir_tree == True) {
-		asprintf(&temp_name, "%s/%s", recbin->repository, path_name);
+	if (recycle_keep_dir_tree(conn) == True) {
+		asprintf(&temp_name, "%s/%s", repository, path_name);
 	} else {
-		temp_name = strdup(recbin->repository);
+		temp_name = strdup(repository);
 	}
 	ALLOC_CHECK(temp_name, done);
 
-	exist = recycle_directory_exist(conn, temp_name);
+	exist = recycle_directory_exist(handle, temp_name);
 	if (exist) {
-		DEBUG(10, ("recycle.bin: Directory already exists\n"));
+		DEBUG(10, ("recycle: Directory already exists\n"));
 	} else {
-		DEBUG(10, ("recycle.bin: Creating directory %s\n", temp_name));
-		if (recycle_create_dir(conn, temp_name) == False) {
-			DEBUG(3, ("Could not create directory, purging %s...\n", file_name));
-			rc = default_vfs_ops.unlink(conn, file_name);
+		DEBUG(10, ("recycle: Creating directory %s\n", temp_name));
+		if (recycle_create_dir(handle, temp_name) == False) {
+			DEBUG(3, ("recycle: Could not create directory, purging %s...\n", file_name));
+			rc = default_vfs_ops->unlink(handle->next, conn, file_name);
 			goto done;
 		}
 	}
 
 	asprintf(&final_name, "%s/%s", temp_name, base);
 	ALLOC_CHECK(final_name, done);
-	DEBUG(10, ("recycle.bin: recycled file name: %s\n", temp_name));		/* new filename with path */
+	DEBUG(10, ("recycle: recycled file name: %s\n", final_name));		/* new filename with path */
 
 	/* check if we should delete file from recycle bin */
-	if (recycle_file_exist(conn, final_name)) {
-		if (recbin->versions == False || matchparam(recbin->noversions, base) == True) {
-			DEBUG(3, ("recycle.bin: Removing old file %s from recycle bin\n", final_name));
-			if (default_vfs_ops.unlink(conn, final_name) != 0) {
-				DEBUG(1, ("recycle.bin: Error deleting old file: %s\n", strerror(errno)));
+	if (recycle_file_exist(handle, final_name)) {
+		if (recycle_versions(conn) == False || matchparam(recycle_noversions(conn), base) == True) {
+			DEBUG(3, ("recycle: Removing old file %s from recycle bin\n", final_name));
+			if (default_vfs_ops->unlink(handle->next, conn, final_name) != 0) {
+				DEBUG(1, ("recycle: Error deleting old file: %s\n", strerror(errno)));
 			}
 		}
 	}
 
 	/* rename file we move to recycle bin */
 	i = 1;
-	while (recycle_file_exist(conn, final_name)) {
-		snprintf(final_name, PATH_MAX, "%s/Copy #%d of %s", temp_name, i++, base);
+	while (recycle_file_exist(handle, final_name)) {
+		snprintf(final_name, PATH_MAX -1, "%s/Copy #%d of %s", temp_name, i++, base);
 	}
 
-	DEBUG(10, ("recycle.bin: Moving %s to %s\n", file_name, final_name));
-	rc = default_vfs_ops.rename(conn, file_name, final_name);
+	DEBUG(10, ("recycle: Moving %s to %s\n", file_name, final_name));
+	rc = default_vfs_ops->rename(handle->next, conn, file_name, final_name);
 	if (rc != 0) {
-		DEBUG(3, ("recycle.bin: Move error %d (%s), purging file %s (%s)\n", errno, strerror(errno), file_name, final_name));
-		rc = default_vfs_ops.unlink(conn, file_name);
+		DEBUG(3, ("recycle: Move error %d (%s), purging file %s (%s)\n", errno, strerror(errno), file_name, final_name));
+		rc = default_vfs_ops->unlink(handle->next, conn, file_name);
 		goto done;
 	}
 
 	/* touch access date of moved file */
-	if (recbin->touch == True )
-		recycle_touch(conn, final_name);
+	if (recycle_touch(conn) == True )
+		recycle_do_touch(handle, final_name);
 
 done:
 	SAFE_FREE(path_name);
 	SAFE_FREE(temp_name);
 	SAFE_FREE(final_name);
+	SAFE_FREE(repository);
 	return rc;
 }
 
 int vfs_recycle_init(void)
 {
-	return smb_register_vfs("recycle", recycle_init, SMB_VFS_INTERFACE_VERSION);
+	BOOL ret = smb_register_vfs("recycle", recycle_ops, SMB_VFS_INTERFACE_VERSION);
+	
+	vfs_recycle_debug_level = debug_add_class("recycle");
+	if (vfs_recycle_debug_level == -1) {
+		vfs_recycle_debug_level = DBGC_VFS;
+		DEBUG(0, ("vfs_recycle: Couldn't register custom debugging class!\n"));
+	} else {
+		DEBUG(10, ("vfs_recycle: Debug class number of 'recycle': %d\n", vfs_recycle_debug_level));
+	}
+	
+	return ret;
 }
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/msdfs/msdfs.c 3_0-vfs/source/msdfs/msdfs.c
--- 3_0/source/msdfs/msdfs.c	Thu Apr 24 07:01:00 2003
+++ 3_0-vfs/source/msdfs/msdfs.c	Thu Apr 24 07:03:25 2003
@@ -169,14 +169,14 @@ BOOL is_msdfs_link(connection_struct* co
 	if (sbufp == NULL)
 		sbufp = &st;
 
-	if (conn->vfs_ops.lstat(conn, path, sbufp) != 0) {
+	if (vfs_lstat(conn, path, sbufp) != 0) {
 		DEBUG(5,("is_msdfs_link: %s does not exist.\n",path));
 		return False;
 	}
   
 	if (S_ISLNK(sbufp->st_mode)) {
 		/* open the link and read it */
-		referral_len = conn->vfs_ops.readlink(conn, path, referral, 
+		referral_len = vfs_readlink(conn, path, referral, 
 						      sizeof(pstring));
 		if (referral_len == -1) {
 			DEBUG(0,("is_msdfs_link: Error reading msdfs link %s: %s\n", path, strerror(errno)));
@@ -785,10 +785,10 @@ BOOL create_msdfs_link(struct junction_m
 	DEBUG(5,("create_msdfs_link: Creating new msdfs link: %s -> %s\n", path, msdfs_link));
 
 	if(exists)
-		if(conn->vfs_ops.unlink(conn,path)!=0)
+		if(vfs_unlink(conn,path)!=0)
 			return False;
 
-	if(conn->vfs_ops.symlink(conn, msdfs_link, path) < 0) {
+	if(vfs_symlink(conn, msdfs_link, path) < 0) {
 		DEBUG(1,("create_msdfs_link: symlink failed %s -> %s\nError: %s\n", 
 				path, msdfs_link, strerror(errno)));
 		return False;
@@ -805,7 +805,7 @@ BOOL remove_msdfs_link(struct junction_m
 	if(!junction_to_local_path(jn, path, sizeof(path), conn))
 		return False;
      
-	if(conn->vfs_ops.unlink(conn, path)!=0)
+	if(vfs_unlink(conn, path)!=0)
 		return False;
   
 	return True;
@@ -862,7 +862,7 @@ static BOOL form_junctions(int snum, str
 	cnt++;
 	
 	/* Now enumerate all dfs links */
-	dirp = conn->vfs_ops.opendir(conn, connect_path);
+	dirp = vfs_opendir(conn, connect_path);
 	if(!dirp)
 		return False;
 
@@ -881,7 +881,7 @@ static BOOL form_junctions(int snum, str
 		}
 	}
 	
-	conn->vfs_ops.closedir(conn,dirp);
+	vfs_closedir(conn,dirp);
 	*jn_count = cnt;
 	return True;
 }
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/param/loadparm.c 3_0-vfs/source/param/loadparm.c
--- 3_0/source/param/loadparm.c	Thu Apr 24 07:01:01 2003
+++ 3_0-vfs/source/param/loadparm.c	Thu Apr 24 07:03:27 2003
@@ -336,9 +336,7 @@ typedef struct
 	char **printer_admin;
 	char *volume;
 	char *fstype;
-	char **szVfsObjectFile;
-	char *szVfsOptions;
-	char *szVfsPath;
+	char **szVfsObjects;
 	char *szMSDfsProxy;
 	int iMinPrintSpace;
 	int iMaxPrintJobs;
@@ -457,9 +455,7 @@ static service sDefault = {
 	NULL,			/* printer admin */
 	NULL,			/* volume */
 	NULL,			/* fstype */
-	NULL,			/* vfs object */
-	NULL,			/* vfs options */
-	NULL,			/* vfs path */
+	NULL,			/* vfs objects */
 	NULL,                   /* szMSDfsProxy */
 	0,			/* iMinPrintSpace */
 	1000,			/* iMaxPrintJobs */
@@ -1106,10 +1102,9 @@ static struct parm_struct parm_table[] =
 	{"hide local users", P_BOOL, P_GLOBAL, &Globals.bHideLocalUsers, NULL, NULL, FLAG_ADVANCED | FLAG_DEVELOPER},
 
 	{"VFS module options", P_SEP, P_SEPARATOR},
-	
-	{"vfs object", P_LIST, P_LOCAL, &sDefault.szVfsObjectFile, NULL, NULL, FLAG_SHARE},
-	{"vfs options", P_STRING, P_LOCAL, &sDefault.szVfsOptions, NULL, NULL, FLAG_SHARE},
-	{"vfs path", P_STRING, P_LOCAL, &sDefault.szVfsPath, NULL, NULL, FLAG_SHARE},
+
+	{"vfs objects", P_LIST, P_LOCAL, &sDefault.szVfsObjects, NULL, NULL, FLAG_SHARE},
+	{"vfs object", P_LIST, P_LOCAL, &sDefault.szVfsObjects, NULL, NULL, FLAG_SHARE | FLAG_HIDE},
 
 	
 	{"msdfs root", P_BOOL, P_LOCAL, &sDefault.bMSDfsRoot, NULL, NULL, FLAG_SHARE},
@@ -1786,9 +1781,7 @@ FN_LOCAL_LIST(lp_readlist, readlist)
 FN_LOCAL_LIST(lp_writelist, writelist)
 FN_LOCAL_LIST(lp_printer_admin, printer_admin)
 FN_LOCAL_STRING(lp_fstype, fstype)
-FN_LOCAL_LIST(lp_vfsobj, szVfsObjectFile)
-FN_LOCAL_STRING(lp_vfs_options, szVfsOptions)
-FN_LOCAL_STRING(lp_vfs_path, szVfsPath)
+FN_LOCAL_LIST(lp_vfs_objects, szVfsObjects)
 FN_LOCAL_STRING(lp_msdfs_proxy, szMSDfsProxy)
 static FN_LOCAL_STRING(lp_volume, volume)
 FN_LOCAL_STRING(lp_mangled_map, szMangledMap)
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/printing/nt_printing.c 3_0-vfs/source/printing/nt_printing.c
--- 3_0/source/printing/nt_printing.c	Sat Apr 26 10:50:37 2003
+++ 3_0-vfs/source/printing/nt_printing.c	Sat Apr 26 10:52:40 2003
@@ -746,7 +746,7 @@ static int get_file_version(files_struct
 	}
 
 	/* Skip OEM header (if any) and the DOS stub to start of Windows header */
-	if (fsp->conn->vfs_ops.lseek(fsp, fsp->fd, SVAL(buf,DOS_HEADER_LFANEW_OFFSET), SEEK_SET) == (SMB_OFF_T)-1) {
+	if (vfs_lseek(fsp, fsp->fd, SVAL(buf,DOS_HEADER_LFANEW_OFFSET), SEEK_SET) == (SMB_OFF_T)-1) {
 		DEBUG(3,("get_file_version: File [%s] too short, errno = %d\n",
 				fname, errno));
 		/* Assume this isn't an error... the file just looks sort of like a PE/NE file */
@@ -806,7 +806,7 @@ static int get_file_version(files_struct
 				}
 
 				/* Seek to the start of the .rsrc section info */
-				if (fsp->conn->vfs_ops.lseek(fsp, fsp->fd, section_pos, SEEK_SET) == (SMB_OFF_T)-1) {
+				if (vfs_lseek(fsp, fsp->fd, section_pos, SEEK_SET) == (SMB_OFF_T)-1) {
 					DEBUG(3,("get_file_version: PE file [%s] too short for section info, errno = %d\n",
 							fname, errno));
 					goto error_exit;
@@ -899,7 +899,7 @@ static int get_file_version(files_struct
 				 * twice, as it is simpler to read the code. */
 				if (strcmp(&buf[i], VS_SIGNATURE) == 0) {
 					/* Compute skip alignment to next long address */
-					int skip = -(fsp->conn->vfs_ops.lseek(fsp, fsp->fd, 0, SEEK_CUR) - (byte_count - i) +
+					int skip = -(vfs_lseek(fsp, fsp->fd, 0, SEEK_CUR) - (byte_count - i) +
 								 sizeof(VS_SIGNATURE)) & 3;
 					if (IVAL(buf,i+sizeof(VS_SIGNATURE)+skip) != 0xfeef04bd) continue;
 
@@ -992,7 +992,7 @@ static int file_version_is_newer(connect
 			DEBUG(6,("file_version_is_newer: Version info not found [%s], use mod time\n",
 					 old_file));
 			use_version = False;
-			if (fsp->conn->vfs_ops.fstat(fsp, fsp->fd, &st) == -1) goto error_exit;
+			if (vfs_fstat(fsp, fsp->fd, &st) == -1) goto error_exit;
 			old_create_time = st.st_mtime;
 			DEBUGADD(6,("file_version_is_newer: mod time = %ld sec\n", old_create_time));
 		}
@@ -1021,7 +1021,7 @@ static int file_version_is_newer(connect
 			DEBUG(6,("file_version_is_newer: Version info not found [%s], use mod time\n",
 					 new_file));
 			use_version = False;
-			if (fsp->conn->vfs_ops.fstat(fsp, fsp->fd, &st) == -1) goto error_exit;
+			if (vfs_fstat(fsp, fsp->fd, &st) == -1) goto error_exit;
 			new_create_time = st.st_mtime;
 			DEBUGADD(6,("file_version_is_newer: mod time = %ld sec\n", new_create_time));
 		}
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/printing/printfsp.c 3_0-vfs/source/printing/printfsp.c
--- 3_0/source/printing/printfsp.c	Thu Feb  6 10:34:17 2003
+++ 3_0-vfs/source/printing/printfsp.c	Tue Apr 22 15:34:33 2003
@@ -80,7 +80,7 @@ files_struct *print_fsp_open(connection_
 	string_set(&fsp->fsp_name,print_job_fname(SNUM(conn),jobid));
 	fsp->wbmpx_ptr = NULL;      
 	fsp->wcp = NULL; 
-	conn->vfs_ops.fstat(fsp,fsp->fd, &sbuf);
+	vfs_fstat(fsp,fsp->fd, &sbuf);
 	fsp->mode = sbuf.st_mode;
 	fsp->inode = sbuf.st_ino;
 	fsp->dev = sbuf.st_dev;
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/rpc_server/srv_srvsvc_nt.c 3_0-vfs/source/rpc_server/srv_srvsvc_nt.c
--- 3_0/source/rpc_server/srv_srvsvc_nt.c	Mon Apr  7 07:58:03 2003
+++ 3_0-vfs/source/rpc_server/srv_srvsvc_nt.c	Tue Apr 22 15:47:53 2003
@@ -1887,7 +1887,7 @@ WERROR _srv_net_file_query_secdesc(pipes
 		}
 	}
 
-	sd_size = conn->vfs_ops.get_nt_acl(fsp, fsp->fsp_name, &psd);
+	sd_size = vfs_get_nt_acl(fsp, fsp->fsp_name, &psd);
 
 	if (sd_size == 0) {
 		DEBUG(3,("_srv_net_file_query_secdesc: Unable to get NT ACL for file %s\n", filename));
@@ -1991,7 +1991,7 @@ WERROR _srv_net_file_set_secdesc(pipes_s
 		}
 	}
 
-	ret = conn->vfs_ops.set_nt_acl(fsp, fsp->fsp_name, q_u->sec_info, q_u->sec_desc);
+	ret = vfs_set_nt_acl(fsp, fsp->fsp_name, q_u->sec_info, q_u->sec_desc);
 
 	if (ret == False) {
 		DEBUG(3,("_srv_net_file_set_secdesc: Unable to set NT ACL on file %s\n", filename));
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/close.c 3_0-vfs/source/smbd/close.c
--- 3_0/source/smbd/close.c	Tue Apr 22 06:46:32 2003
+++ 3_0-vfs/source/smbd/close.c	Tue Apr 22 14:36:30 2003
@@ -185,7 +185,7 @@ static int close_normal_file(files_struc
 	if (normal_close && delete_on_close) {
 		DEBUG(5,("close_file: file %s. Delete on close was set - deleting file.\n",
 			fsp->fsp_name));
-		if(fsp->conn->vfs_ops.unlink(conn,fsp->fsp_name) != 0) {
+		if(vfs_unlink(conn,fsp->fsp_name) != 0) {
 			/*
 			 * This call can potentially fail as another smbd may have
 			 * had the file open with delete on close set and deleted
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/conn.c 3_0-vfs/source/smbd/conn.c
--- 3_0/source/smbd/conn.c	Wed Apr 16 18:19:49 2003
+++ 3_0-vfs/source/smbd/conn.c	Tue Apr 22 14:22:02 2003
@@ -93,6 +93,7 @@ thinking the server is still available.
 ****************************************************************************/
 connection_struct *conn_new(void)
 {
+	TALLOC_CTX *mem_ctx;
 	connection_struct *conn;
 	int i;
 
@@ -103,10 +104,16 @@ connection_struct *conn_new(void)
 		return NULL;
 	}
 
-	conn = (connection_struct *)malloc(sizeof(*conn));
-	if (!conn) return NULL;
+	if ((mem_ctx=talloc_init("connection_struct"))==NULL) {
+		DEBUG(0,("talloc_init(connection_struct) failed!\n"));
+		return NULL;
+	}
 
-	ZERO_STRUCTP(conn);
+	if ((conn=(connection_struct *)talloc_zero(mem_ctx, sizeof(*conn)))==NULL) {
+		DEBUG(0,("talloc_zero() failed!\n"));
+		return NULL;
+	}
+	conn->mem_ctx = mem_ctx;
 	conn->cnum = i;
 
 	bitmap_set(bmap, i);
@@ -195,27 +202,16 @@ void conn_clear_vuid_cache(uint16 vuid)
 
 void conn_free(connection_struct *conn)
 {
- 	smb_vfs_handle_struct *handle, *thandle;
- 	void (*done_fptr)(connection_struct *the_conn);
+ 	vfs_handle_struct *handle = NULL, *thandle = NULL;
+ 	TALLOC_CTX *mem_ctx = NULL;
 
 	/* Free vfs_connection_struct */
-	handle = conn->vfs_private;
+	handle = conn->vfs_handles;
 	while(handle) {
-		/* Only call dlclose for the old modules */
-		if (handle->handle) {
-			/* Close dlopen() handle */
-			done_fptr = (void (*)(connection_struct *))sys_dlsym(handle->handle, "vfs_done");
-
-			if (done_fptr == NULL) {
-				DEBUG(3, ("No vfs_done() symbol found in module with handle %p, ignoring\n", handle->handle));
-			} else {
-				done_fptr(conn);
-			}
-			sys_dlclose(handle->handle);
-		}
-		DLIST_REMOVE(conn->vfs_private, handle);
+		DLIST_REMOVE(conn->vfs_handles, handle);
 		thandle = handle->next;
-		SAFE_FREE(handle);
+		if (handle->free_data)
+			handle->free_data(&handle->data);
 		handle = thandle;
 	}
 
@@ -238,8 +234,9 @@ void conn_free(connection_struct *conn)
 	bitmap_clear(bmap, conn->cnum);
 	num_open--;
 
+	mem_ctx = conn->mem_ctx;
 	ZERO_STRUCTP(conn);
-	SAFE_FREE(conn);
+	talloc_destroy(mem_ctx);
 }
 
 
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/dir.c 3_0-vfs/source/smbd/dir.c
--- 3_0/source/smbd/dir.c	Mon Apr 14 16:32:14 2003
+++ 3_0-vfs/source/smbd/dir.c	Tue Apr 22 11:45:51 2003
@@ -643,7 +643,7 @@ BOOL get_dir_entry(connection_struct *co
 			pstrcpy(pathreal,path);
 			pstrcat(path,fname);
 			pstrcat(pathreal,dname);
-			if (conn->vfs_ops.stat(conn, pathreal, &sbuf) != 0) {
+			if (vfs_stat(conn, pathreal, &sbuf) != 0) {
 				DEBUG(5,("Couldn't stat 1 [%s]. Error = %s\n",path, strerror(errno) ));
 				continue;
 			}
@@ -715,7 +715,7 @@ static BOOL user_can_read_file(connectio
 		return False;
 
 	/* Get NT ACL -allocated in main loop talloc context. No free needed here. */
-	sd_size = conn->vfs_ops.fget_nt_acl(fsp, fsp->fd, &psd);
+	sd_size = vfs_fget_nt_acl(fsp, fsp->fd, &psd);
 	close_file(fsp, True);
 
 	/* No access if SD get failed. */
@@ -768,7 +768,7 @@ static BOOL user_can_write_file(connecti
 		return False;
 
 	/* Get NT ACL -allocated in main loop talloc context. No free needed here. */
-	sd_size = conn->vfs_ops.fget_nt_acl(fsp, fsp->fd, &psd);
+	sd_size = vfs_fget_nt_acl(fsp, fsp->fd, &psd);
 	close_file(fsp, False);
 
 	/* No access if SD get failed. */
@@ -811,7 +811,7 @@ void *OpenDir(connection_struct *conn, c
 {
 	Dir *dirp;
 	const char *n;
-	DIR *p = conn->vfs_ops.opendir(conn,name);
+	DIR *p = vfs_opendir(conn,name);
 	int used=0;
 
 	if (!p)
@@ -819,7 +819,7 @@ void *OpenDir(connection_struct *conn, c
 	dirp = (Dir *)malloc(sizeof(Dir));
 	if (!dirp) {
 		DEBUG(0,("Out of memory in OpenDir\n"));
-		conn->vfs_ops.closedir(conn,p);
+		vfs_closedir(conn,p);
 		return(NULL);
 	}
 	dirp->pos = dirp->numentries = dirp->mallocsize = 0;
@@ -912,7 +912,7 @@ void *OpenDir(connection_struct *conn, c
 		dirp->numentries++;
 	}
 
-	conn->vfs_ops.closedir(conn,p);
+	vfs_closedir(conn,p);
 	return((void *)dirp);
 }
 
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/dosmode.c 3_0-vfs/source/smbd/dosmode.c
--- 3_0/source/smbd/dosmode.c	Mon Feb 24 08:05:56 2003
+++ 3_0-vfs/source/smbd/dosmode.c	Tue Apr 22 14:23:25 2003
@@ -262,7 +262,7 @@ int file_chmod(connection_struct *conn,c
 		if (!fsp)
 			return -1;
 		become_root();
-		ret = conn->vfs_ops.fchmod(fsp, fsp->fd, unixmode);
+		ret = vfs_fchmod(fsp, fsp->fd, unixmode);
 		unbecome_root();
 		close_file_fchmod(fsp);
 	}
@@ -283,7 +283,7 @@ int file_utime(connection_struct *conn, 
 
   errno = 0;
 
-  if(conn->vfs_ops.utime(conn,fname, times) == 0)
+  if(vfs_utime(conn,fname, times) == 0)
     return 0;
 
   if((errno != EPERM) && (errno != EACCES))
@@ -311,7 +311,7 @@ int file_utime(connection_struct *conn, 
 			 current_user.ngroups,current_user.groups)))) {
 		  /* We are allowed to become root and change the filetime. */
 		  become_root();
-		  ret = conn->vfs_ops.utime(conn,fname, times);
+		  ret = vfs_utime(conn,fname, times);
 		  unbecome_root();
 	  }
   }
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/fileio.c 3_0-vfs/source/smbd/fileio.c
--- 3_0/source/smbd/fileio.c	Fri Dec 20 14:03:49 2002
+++ 3_0-vfs/source/smbd/fileio.c	Tue Apr 22 11:53:28 2003
@@ -32,7 +32,7 @@ static SMB_OFF_T seek_file(files_struct 
 {
 	SMB_OFF_T seek_ret;
 
-	seek_ret = fsp->conn->vfs_ops.lseek(fsp,fsp->fd,pos,SEEK_SET);
+	seek_ret = vfs_lseek(fsp,fsp->fd,pos,SEEK_SET);
 
 	if(seek_ret == -1) {
 		DEBUG(0,("seek_file: (%s) sys_lseek failed. Error was %s\n",
@@ -101,7 +101,7 @@ ssize_t read_file(files_struct *fsp,char
 #ifdef DMF_FIX
 		int numretries = 3;
 tryagain:
-		readret = fsp->conn->vfs_ops.read(fsp,fsp->fd,data,n);
+		readret = vfs_read(fsp,fsp->fd,data,n);
 		if (readret == -1) {
 			if ((errno == EAGAIN) && numretries) {
 				DEBUG(3,("read_file EAGAIN retry in 10 seconds\n"));
@@ -112,7 +112,7 @@ tryagain:
 			return -1;
 		}
 #else /* NO DMF fix. */
-		readret = fsp->conn->vfs_ops.read(fsp,fsp->fd,data,n);
+		readret = vfs_read(fsp,fsp->fd,data,n);
 		if (readret == -1)
 			return -1;
 #endif
@@ -181,7 +181,7 @@ ssize_t write_file(files_struct *fsp, ch
 		SMB_STRUCT_STAT st;
 		fsp->modified = True;
 
-		if (fsp->conn->vfs_ops.fstat(fsp,fsp->fd,&st) == 0) {
+		if (vfs_fstat(fsp,fsp->fd,&st) == 0) {
 			int dosmode = dos_mode(fsp->conn,fsp->fsp_name,&st);
 			fsp->size = (SMB_BIG_UINT)st.st_size;
 			if (MAP_ARCHIVE(fsp->conn) && !IS_DOS_ARCHIVE(dosmode))
@@ -760,7 +760,7 @@ void sync_file(connection_struct *conn, 
 {
 	if(lp_strict_sync(SNUM(conn)) && fsp->fd != -1) {
 		flush_write_cache(fsp, SYNC_FLUSH);
-		conn->vfs_ops.fsync(fsp,fsp->fd);
+		vfs_fsync(fsp,fsp->fd);
 	}
 }
 
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/filename.c 3_0-vfs/source/smbd/filename.c
--- 3_0/source/smbd/filename.c	Tue Mar 18 15:29:05 2003
+++ 3_0-vfs/source/smbd/filename.c	Tue Apr 22 14:23:58 2003
@@ -416,7 +416,7 @@ BOOL check_name(pstring name,connection_
 #ifdef S_ISLNK
 	if (!lp_symlinks(SNUM(conn))) {
 		SMB_STRUCT_STAT statbuf;
-		if ( (conn->vfs_ops.lstat(conn,name,&statbuf) != -1) &&
+		if ( (vfs_lstat(conn,name,&statbuf) != -1) &&
 				(S_ISLNK(statbuf.st_mode)) ) {
 			DEBUG(3,("check_name: denied: file path name %s is a symlink\n",name));
 			ret=0; 
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/nttrans.c 3_0-vfs/source/smbd/nttrans.c
--- 3_0/source/smbd/nttrans.c	Mon Apr 14 16:32:14 2003
+++ 3_0-vfs/source/smbd/nttrans.c	Tue Apr 22 14:12:00 2003
@@ -1032,7 +1032,7 @@ static NTSTATUS set_sd(files_struct *fsp
 	if (psd->off_dacl==0)
 		security_info_sent &= ~DACL_SECURITY_INFORMATION;
 	
-	ret = fsp->conn->vfs_ops.fset_nt_acl( fsp, fsp->fd, security_info_sent, psd);
+	ret = vfs_fset_nt_acl( fsp, fsp->fd, security_info_sent, psd);
 	
 	if (!ret) {
 		talloc_destroy(mem_ctx);
@@ -1561,7 +1561,7 @@ static int call_nt_transact_query_securi
 	if (!lp_nt_acl_support(SNUM(conn)))
 		sd_size = get_null_nt_acl(mem_ctx, &psd);
 	else
-		sd_size = conn->vfs_ops.fget_nt_acl(fsp, fsp->fd, &psd);
+		sd_size = vfs_fget_nt_acl(fsp, fsp->fd, &psd);
 
 	if (sd_size == 0) {
 		talloc_destroy(mem_ctx);
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/open.c 3_0-vfs/source/smbd/open.c
--- 3_0/source/smbd/open.c	Thu Apr 24 14:28:20 2003
+++ 3_0-vfs/source/smbd/open.c	Thu Apr 24 14:30:25 2003
@@ -38,13 +38,13 @@ static int fd_open(struct connection_str
 		flags |= O_NOFOLLOW;
 #endif
 
-	fd = conn->vfs_ops.open(conn,fname,flags,mode);
+	fd = vfs_open(conn,fname,flags,mode);
 
 	/* Fix for files ending in '.' */
 	if((fd == -1) && (errno == ENOENT) &&
 	   (strchr_m(fname,'.')==NULL)) {
 		pstrcat(fname,".");
-		fd = conn->vfs_ops.open(conn,fname,flags,mode);
+		fd = vfs_open(conn,fname,flags,mode);
 	}
 
 	DEBUG(10,("fd_open: name %s, flags = 0%o mode = 0%o, fd = %d. %s\n", fname,
@@ -259,7 +259,7 @@ static int truncate_unless_locked(struct
 		unix_ERR_ntstatus = dos_to_ntstatus(ERRDOS, ERRlock);
 		return -1;
 	} else {
-		return conn->vfs_ops.ftruncate(fsp,fsp->fd,0); 
+		return vfs_ftruncate(fsp,fsp->fd,0); 
 	}
 }
 
@@ -1150,7 +1150,7 @@ flags=0x%X flags2=0x%X mode=0%o returned
 
 		int saved_errno = errno; /* We might get ENOSYS in the next call.. */
 
-		if (conn->vfs_ops.fchmod_acl(fsp, fsp->fd, mode) == -1 && errno == ENOSYS)
+		if (vfs_fchmod_acl(fsp, fsp->fd, mode) == -1 && errno == ENOSYS)
 			errno = saved_errno; /* Ignore ENOSYS */
 
 	} else if (new_mode) {
@@ -1161,7 +1161,7 @@ flags=0x%X flags2=0x%X mode=0%o returned
 
 		if (conn->vfs_ops.fchmod_acl != NULL) {
 			int saved_errno = errno; /* We might get ENOSYS in the next call.. */
-			ret = conn->vfs_ops.fchmod_acl(fsp, fsp->fd, new_mode);
+			ret = vfs_fchmod_acl(fsp, fsp->fd, new_mode);
 
 			if (ret == -1 && errno == ENOSYS) {
 				errno = saved_errno; /* Ignore ENOSYS */
@@ -1172,7 +1172,7 @@ flags=0x%X flags2=0x%X mode=0%o returned
 			}
 		}
 
-		if ((ret == -1) && (conn->vfs_ops.fchmod(fsp, fsp->fd, new_mode) == -1))
+		if ((ret == -1) && (vfs_fchmod(fsp, fsp->fd, new_mode) == -1))
 			DEBUG(5, ("open_file_shared: failed to reset attributes of file %s to 0%o\n",
 				fname, (int)new_mode));
 	}
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/posix_acls.c 3_0-vfs/source/smbd/posix_acls.c
--- 3_0/source/smbd/posix_acls.c	Tue Apr 22 06:46:32 2003
+++ 3_0-vfs/source/smbd/posix_acls.c	Tue Apr 22 15:29:37 2003
@@ -158,9 +158,9 @@ static mode_t convert_permset_to_mode_t(
 {
 	mode_t ret = 0;
 
-	ret |= (conn->vfs_ops.sys_acl_get_perm(conn, permset, SMB_ACL_READ) ? S_IRUSR : 0);
-	ret |= (conn->vfs_ops.sys_acl_get_perm(conn, permset, SMB_ACL_WRITE) ? S_IWUSR : 0);
-	ret |= (conn->vfs_ops.sys_acl_get_perm(conn, permset, SMB_ACL_EXECUTE) ? S_IXUSR : 0);
+	ret |= (vfs_sys_acl_get_perm(conn, permset, SMB_ACL_READ) ? S_IRUSR : 0);
+	ret |= (vfs_sys_acl_get_perm(conn, permset, SMB_ACL_WRITE) ? S_IWUSR : 0);
+	ret |= (vfs_sys_acl_get_perm(conn, permset, SMB_ACL_EXECUTE) ? S_IXUSR : 0);
 
 	return ret;
 }
@@ -190,18 +190,18 @@ static mode_t unix_perms_to_acl_perms(mo
 
 static int map_acl_perms_to_permset(connection_struct *conn, mode_t mode, SMB_ACL_PERMSET_T *p_permset)
 {
-	if (conn->vfs_ops.sys_acl_clear_perms(conn, *p_permset) ==  -1)
+	if (vfs_sys_acl_clear_perms(conn, *p_permset) ==  -1)
 		return -1;
 	if (mode & S_IRUSR) {
-		if (conn->vfs_ops.sys_acl_add_perm(conn, *p_permset, SMB_ACL_READ) == -1)
+		if (vfs_sys_acl_add_perm(conn, *p_permset, SMB_ACL_READ) == -1)
 			return -1;
 	}
 	if (mode & S_IWUSR) {
-		if (conn->vfs_ops.sys_acl_add_perm(conn, *p_permset, SMB_ACL_WRITE) == -1)
+		if (vfs_sys_acl_add_perm(conn, *p_permset, SMB_ACL_WRITE) == -1)
 			return -1;
 	}
 	if (mode & S_IXUSR) {
-		if (conn->vfs_ops.sys_acl_add_perm(conn, *p_permset, SMB_ACL_EXECUTE) == -1)
+		if (vfs_sys_acl_add_perm(conn, *p_permset, SMB_ACL_EXECUTE) == -1)
 			return -1;
 	}
 	return 0;
@@ -659,13 +659,13 @@ static BOOL ensure_canon_entry_valid(can
 		int entry_id = SMB_ACL_FIRST_ENTRY;
 
 		if(fsp->is_directory || fsp->fd == -1) {
-			current_posix_acl = conn->vfs_ops.sys_acl_get_file(conn, fsp->fsp_name, SMB_ACL_TYPE_ACCESS);
+			current_posix_acl = vfs_sys_acl_get_file(conn, fsp->fsp_name, SMB_ACL_TYPE_ACCESS);
 		} else {
-			current_posix_acl = conn->vfs_ops.sys_acl_get_fd(fsp, fsp->fd);
+			current_posix_acl = vfs_sys_acl_get_fd(fsp, fsp->fd);
 		}
 
 		if (current_posix_acl) {
-			while (conn->vfs_ops.sys_acl_get_entry(conn, current_posix_acl, entry_id, &entry) == 1) {
+			while (vfs_sys_acl_get_entry(conn, current_posix_acl, entry_id, &entry) == 1) {
 				SMB_ACL_TAG_T tagtype;
 				SMB_ACL_PERMSET_T permset;
 
@@ -674,10 +674,10 @@ static BOOL ensure_canon_entry_valid(can
 					entry_id = SMB_ACL_NEXT_ENTRY;
 
 				/* Is this a MASK entry ? */
-				if (conn->vfs_ops.sys_acl_get_tag_type(conn, entry, &tagtype) == -1)
+				if (vfs_sys_acl_get_tag_type(conn, entry, &tagtype) == -1)
 					continue;
 
-				if (conn->vfs_ops.sys_acl_get_permset(conn, entry, &permset) == -1)
+				if (vfs_sys_acl_get_permset(conn, entry, &permset) == -1)
 					continue;
 
 				switch(tagtype) {
@@ -695,7 +695,7 @@ static BOOL ensure_canon_entry_valid(can
 						break;
 				}
 			}
-			conn->vfs_ops.sys_acl_free_acl(conn, current_posix_acl);
+			vfs_sys_acl_free_acl(conn, current_posix_acl);
 		} else {
 			DEBUG(10,("ensure_canon_entry_valid: failed to retrieve current ACL of %s\n",
 				fsp->fsp_name));
@@ -1658,7 +1658,7 @@ static canon_ace *canonicalise_acl( file
 	SMB_ACL_ENTRY_T entry;
 	size_t ace_count;
 
-	while ( posix_acl && (conn->vfs_ops.sys_acl_get_entry(conn, posix_acl, entry_id, &entry) == 1)) {
+	while ( posix_acl && (vfs_sys_acl_get_entry(conn, posix_acl, entry_id, &entry) == 1)) {
 		SMB_ACL_TAG_T tagtype;
 		SMB_ACL_PERMSET_T permset;
 		DOM_SID sid;
@@ -1670,10 +1670,10 @@ static canon_ace *canonicalise_acl( file
 			entry_id = SMB_ACL_NEXT_ENTRY;
 
 		/* Is this a MASK entry ? */
-		if (conn->vfs_ops.sys_acl_get_tag_type(conn, entry, &tagtype) == -1)
+		if (vfs_sys_acl_get_tag_type(conn, entry, &tagtype) == -1)
 			continue;
 
-		if (conn->vfs_ops.sys_acl_get_permset(conn, entry, &permset) == -1)
+		if (vfs_sys_acl_get_permset(conn, entry, &permset) == -1)
 			continue;
 
 		/* Decide which SID to use based on the ACL type. */
@@ -1686,7 +1686,7 @@ static canon_ace *canonicalise_acl( file
 				break;
 			case SMB_ACL_USER:
 				{
-					uid_t *puid = (uid_t *)conn->vfs_ops.sys_acl_get_qualifier(conn, entry);
+					uid_t *puid = (uid_t *)vfs_sys_acl_get_qualifier(conn, entry);
 					if (puid == NULL) {
 						DEBUG(0,("canonicalise_acl: Failed to get uid.\n"));
 						continue;
@@ -1703,7 +1703,7 @@ static canon_ace *canonicalise_acl( file
 					uid_to_sid( &sid, *puid);
 					unix_ug.uid = *puid;
 					owner_type = UID_ACE;
-					conn->vfs_ops.sys_acl_free_qualifier(conn, (void *)puid,tagtype);
+					vfs_sys_acl_free_qualifier(conn, (void *)puid,tagtype);
 					break;
 				}
 			case SMB_ACL_GROUP_OBJ:
@@ -1714,7 +1714,7 @@ static canon_ace *canonicalise_acl( file
 				break;
 			case SMB_ACL_GROUP:
 				{
-					gid_t *pgid = (gid_t *)conn->vfs_ops.sys_acl_get_qualifier(conn, entry);
+					gid_t *pgid = (gid_t *)vfs_sys_acl_get_qualifier(conn, entry);
 					if (pgid == NULL) {
 						DEBUG(0,("canonicalise_acl: Failed to get gid.\n"));
 						continue;
@@ -1722,7 +1722,7 @@ static canon_ace *canonicalise_acl( file
 					gid_to_sid( &sid, *pgid);
 					unix_ug.gid = *pgid;
 					owner_type = GID_ACE;
-					conn->vfs_ops.sys_acl_free_qualifier(conn, (void *)pgid,tagtype);
+					vfs_sys_acl_free_qualifier(conn, (void *)pgid,tagtype);
 					break;
 				}
 			case SMB_ACL_MASK:
@@ -1807,7 +1807,7 @@ static BOOL set_canon_ace_list(files_str
 {
 	connection_struct *conn = fsp->conn;
 	BOOL ret = False;
-	SMB_ACL_T the_acl = conn->vfs_ops.sys_acl_init(conn, (int)count_canon_ace_list(the_ace) + 1);
+	SMB_ACL_T the_acl = vfs_sys_acl_init(conn, (int)count_canon_ace_list(the_ace) + 1);
 	canon_ace *p_ace;
 	int i;
 	SMB_ACL_ENTRY_T mask_entry;
@@ -1865,7 +1865,7 @@ static BOOL set_canon_ace_list(files_str
 		 * Get the entry for this ACE.
 		 */
 
-		if (conn->vfs_ops.sys_acl_create_entry(conn, &the_acl, &the_entry) == -1) {
+		if (vfs_sys_acl_create_entry(conn, &the_acl, &the_entry) == -1) {
 			DEBUG(0,("set_canon_ace_list: Failed to create entry %d. (%s)\n",
 				i, strerror(errno) ));
 			goto done;
@@ -1891,7 +1891,7 @@ static BOOL set_canon_ace_list(files_str
 		 * First tell the entry what type of ACE this is.
 		 */
 
-		if (conn->vfs_ops.sys_acl_set_tag_type(conn, the_entry, p_ace->type) == -1) {
+		if (vfs_sys_acl_set_tag_type(conn, the_entry, p_ace->type) == -1) {
 			DEBUG(0,("set_canon_ace_list: Failed to set tag type on entry %d. (%s)\n",
 				i, strerror(errno) ));
 			goto done;
@@ -1903,7 +1903,7 @@ static BOOL set_canon_ace_list(files_str
 		 */
 
 		if ((p_ace->type == SMB_ACL_USER) || (p_ace->type == SMB_ACL_GROUP)) {
-			if (conn->vfs_ops.sys_acl_set_qualifier(conn, the_entry,(void *)&p_ace->unix_ug.uid) == -1) {
+			if (vfs_sys_acl_set_qualifier(conn, the_entry,(void *)&p_ace->unix_ug.uid) == -1) {
 				DEBUG(0,("set_canon_ace_list: Failed to set qualifier on entry %d. (%s)\n",
 					i, strerror(errno) ));
 				goto done;
@@ -1914,7 +1914,7 @@ static BOOL set_canon_ace_list(files_str
 		 * Convert the mode_t perms in the canon_ace to a POSIX permset.
 		 */
 
-		if (conn->vfs_ops.sys_acl_get_permset(conn, the_entry, &the_permset) == -1) {
+		if (vfs_sys_acl_get_permset(conn, the_entry, &the_permset) == -1) {
 			DEBUG(0,("set_canon_ace_list: Failed to get permset on entry %d. (%s)\n",
 				i, strerror(errno) ));
 			goto done;
@@ -1930,7 +1930,7 @@ static BOOL set_canon_ace_list(files_str
 		 * ..and apply them to the entry.
 		 */
 
-		if (conn->vfs_ops.sys_acl_set_permset(conn, the_entry, the_permset) == -1) {
+		if (vfs_sys_acl_set_permset(conn, the_entry, the_permset) == -1) {
 			DEBUG(0,("set_canon_ace_list: Failed to add permset on entry %d. (%s)\n",
 				i, strerror(errno) ));
 			goto done;
@@ -1941,17 +1941,17 @@ static BOOL set_canon_ace_list(files_str
 	}
 
 	if (needs_mask && !got_mask_entry) {
-		if (conn->vfs_ops.sys_acl_create_entry(conn, &the_acl, &mask_entry) == -1) {
+		if (vfs_sys_acl_create_entry(conn, &the_acl, &mask_entry) == -1) {
 			DEBUG(0,("set_canon_ace_list: Failed to create mask entry. (%s)\n", strerror(errno) ));
 			goto done;
 		}
 
-		if (conn->vfs_ops.sys_acl_set_tag_type(conn, mask_entry, SMB_ACL_MASK) == -1) {
+		if (vfs_sys_acl_set_tag_type(conn, mask_entry, SMB_ACL_MASK) == -1) {
 			DEBUG(0,("set_canon_ace_list: Failed to set tag type on mask entry. (%s)\n",strerror(errno) ));
 			goto done;
 		}
 
-		if (conn->vfs_ops.sys_acl_get_permset(conn, mask_entry, &mask_permset) == -1) {
+		if (vfs_sys_acl_get_permset(conn, mask_entry, &mask_permset) == -1) {
 			DEBUG(0,("set_canon_ace_list: Failed to get mask permset. (%s)\n", strerror(errno) ));
 			goto done;
 		}
@@ -1961,7 +1961,7 @@ static BOOL set_canon_ace_list(files_str
 			goto done;
 		}
 
-		if (conn->vfs_ops.sys_acl_set_permset(conn, mask_entry, mask_permset) == -1) {
+		if (vfs_sys_acl_set_permset(conn, mask_entry, mask_permset) == -1) {
 			DEBUG(0,("set_canon_ace_list: Failed to add mask permset. (%s)\n", strerror(errno) ));
 			goto done;
 		}
@@ -1971,7 +1971,7 @@ static BOOL set_canon_ace_list(files_str
 	 * Check if the ACL is valid.
 	 */
 
-	if (conn->vfs_ops.sys_acl_valid(conn, the_acl) == -1) {
+	if (vfs_sys_acl_valid(conn, the_acl) == -1) {
 		DEBUG(0,("set_canon_ace_list: ACL type (%s) is invalid for set (%s).\n",
 				the_acl_type == SMB_ACL_TYPE_DEFAULT ? "directory default" : "file",
 				strerror(errno) ));
@@ -1983,7 +1983,7 @@ static BOOL set_canon_ace_list(files_str
 	 */
 
 	if(default_ace || fsp->is_directory || fsp->fd == -1) {
-		if (conn->vfs_ops.sys_acl_set_file(conn, fsp->fsp_name, the_acl_type, the_acl) == -1) {
+		if (vfs_sys_acl_set_file(conn, fsp->fsp_name, the_acl_type, the_acl) == -1) {
 			/*
 			 * Some systems allow all the above calls and only fail with no ACL support
 			 * when attempting to apply the acl. HPUX with HFS is an example of this. JRA.
@@ -2002,7 +2002,7 @@ static BOOL set_canon_ace_list(files_str
 			goto done;
 		}
 	} else {
-		if (conn->vfs_ops.sys_acl_set_fd(fsp, fsp->fd, the_acl) == -1) {
+		if (vfs_sys_acl_set_fd(fsp, fsp->fd, the_acl) == -1) {
 			/*
 			 * Some systems allow all the above calls and only fail with no ACL support
 			 * when attempting to apply the acl. HPUX with HFS is an example of this. JRA.
@@ -2026,7 +2026,7 @@ static BOOL set_canon_ace_list(files_str
   done:
 
 	if (the_acl != NULL)
-	    conn->vfs_ops.sys_acl_free_acl(conn, the_acl);
+	    vfs_sys_acl_free_acl(conn, the_acl);
 
 	return ret;
 }
@@ -2057,8 +2057,8 @@ SMB_ACL_T free_empty_sys_acl(connection_
 
 	if (!acl)
 		return NULL;
-	if (conn->vfs_ops.sys_acl_get_entry(conn, acl, SMB_ACL_FIRST_ENTRY, &entry) != 1) {
-		conn->vfs_ops.sys_acl_free_acl(conn, acl);
+	if (vfs_sys_acl_get_entry(conn, acl, SMB_ACL_FIRST_ENTRY, &entry) != 1) {
+		vfs_sys_acl_free_acl(conn, acl);
 		return NULL;
 	}
 	return acl;
@@ -2231,14 +2231,14 @@ size_t get_nt_acl(files_struct *fsp, SEC
 		 * Get the ACL from the path.
 		 */
 
-		posix_acl = conn->vfs_ops.sys_acl_get_file(conn, fsp->fsp_name, SMB_ACL_TYPE_ACCESS);
+		posix_acl = vfs_sys_acl_get_file(conn, fsp->fsp_name, SMB_ACL_TYPE_ACCESS);
 
 		/*
 		 * If it's a directory get the default POSIX ACL.
 		 */
 
 		if(fsp->is_directory) {
-			dir_acl = conn->vfs_ops.sys_acl_get_file(conn, fsp->fsp_name, SMB_ACL_TYPE_DEFAULT);
+			dir_acl = vfs_sys_acl_get_file(conn, fsp->fsp_name, SMB_ACL_TYPE_DEFAULT);
 			dir_acl = free_empty_sys_acl(conn, dir_acl);
 		}
 
@@ -2251,7 +2251,7 @@ size_t get_nt_acl(files_struct *fsp, SEC
 		/*
 		 * Get the ACL from the fd.
 		 */
-		posix_acl = conn->vfs_ops.sys_acl_get_fd(fsp, fsp->fd);
+		posix_acl = vfs_sys_acl_get_fd(fsp, fsp->fd);
 	}
 
 	DEBUG(5,("get_nt_acl : file ACL %s, directory ACL %s\n",
@@ -2479,9 +2479,9 @@ size_t get_nt_acl(files_struct *fsp, SEC
  done:
 
 	if (posix_acl)
-		conn->vfs_ops.sys_acl_free_acl(conn, posix_acl);
+		vfs_sys_acl_free_acl(conn, posix_acl);
 	if (dir_acl)
-		conn->vfs_ops.sys_acl_free_acl(conn, dir_acl);
+		vfs_sys_acl_free_acl(conn, dir_acl);
 	free_canon_ace_list(file_ace);
 	free_canon_ace_list(dir_ace);
 	SAFE_FREE(nt_ace_list);
@@ -2527,7 +2527,7 @@ static int try_chown(connection_struct *
 
 	become_root();
 	/* Keep the current file gid the same. */
-	ret = vfswrap_fchown(fsp, fsp->fd, uid, (gid_t)-1);
+	ret = vfs_fchown(fsp, fsp->fd, uid, (gid_t)-1);
 	unbecome_root();
 
 	close_file_fchmod(fsp);
@@ -2697,7 +2697,7 @@ BOOL set_nt_acl(files_struct *fsp, uint3
 					 * No default ACL - delete one if it exists.
 					 */
 
-					if (conn->vfs_ops.sys_acl_delete_def_file(conn, fsp->fsp_name) == -1) {
+					if (vfs_sys_acl_delete_def_file(conn, fsp->fsp_name) == -1) {
 						DEBUG(3,("set_nt_acl: sys_acl_delete_def_file failed (%s)\n", strerror(errno)));
 						free_canon_ace_list(file_ace_list);
 						free_canon_ace_list(dir_ace_list);
@@ -2726,7 +2726,7 @@ BOOL set_nt_acl(files_struct *fsp, uint3
 					DEBUG(3,("set_nt_acl: chmod %s. perms = 0%o.\n",
 						fsp->fsp_name, (unsigned int)posix_perms ));
 
-					if(conn->vfs_ops.chmod(conn,fsp->fsp_name, posix_perms) == -1) {
+					if(vfs_chmod(conn,fsp->fsp_name, posix_perms) == -1) {
 						DEBUG(3,("set_nt_acl: chmod %s, 0%o failed. Error = %s.\n",
 								fsp->fsp_name, (unsigned int)posix_perms, strerror(errno) ));
 						free_canon_ace_list(file_ace_list);
@@ -2768,7 +2768,7 @@ static int chmod_acl_internals( connecti
 	SMB_ACL_ENTRY_T entry;
 	int num_entries = 0;
 
-	while ( conn->vfs_ops.sys_acl_get_entry(conn, posix_acl, entry_id, &entry) == 1) {
+	while ( vfs_sys_acl_get_entry(conn, posix_acl, entry_id, &entry) == 1) {
 		SMB_ACL_TAG_T tagtype;
 		SMB_ACL_PERMSET_T permset;
 		mode_t perms;
@@ -2777,10 +2777,10 @@ static int chmod_acl_internals( connecti
 		if (entry_id == SMB_ACL_FIRST_ENTRY)
 			entry_id = SMB_ACL_NEXT_ENTRY;
 
-		if (conn->vfs_ops.sys_acl_get_tag_type(conn, entry, &tagtype) == -1)
+		if (vfs_sys_acl_get_tag_type(conn, entry, &tagtype) == -1)
 			return -1;
 
-		if (conn->vfs_ops.sys_acl_get_permset(conn, entry, &permset) == -1)
+		if (vfs_sys_acl_get_permset(conn, entry, &permset) == -1)
 			return -1;
 
 		num_entries++;
@@ -2811,7 +2811,7 @@ static int chmod_acl_internals( connecti
 		if (map_acl_perms_to_permset(conn, perms, &permset) == -1)
 			return -1;
 
-		if (conn->vfs_ops.sys_acl_set_permset(conn, entry, permset) == -1)
+		if (vfs_sys_acl_set_permset(conn, entry, permset) == -1)
 			return -1;
 	}
 
@@ -2837,17 +2837,17 @@ static int copy_access_acl(connection_st
 	SMB_ACL_T posix_acl = NULL;
 	int ret = -1;
 
-	if ((posix_acl = conn->vfs_ops.sys_acl_get_file(conn, from, SMB_ACL_TYPE_ACCESS)) == NULL)
+	if ((posix_acl = vfs_sys_acl_get_file(conn, from, SMB_ACL_TYPE_ACCESS)) == NULL)
 		return -1;
 
 	if ((ret = chmod_acl_internals(conn, posix_acl, mode)) == -1)
 		goto done;
 
-	ret = conn->vfs_ops.sys_acl_set_file(conn, to, SMB_ACL_TYPE_ACCESS, posix_acl);
+	ret = vfs_sys_acl_set_file(conn, to, SMB_ACL_TYPE_ACCESS, posix_acl);
 
  done:
 
-	conn->vfs_ops.sys_acl_free_acl(conn, posix_acl);
+	vfs_sys_acl_free_acl(conn, posix_acl);
 	return ret;
 }
 
@@ -2889,17 +2889,17 @@ int fchmod_acl(files_struct *fsp, int fd
 	SMB_ACL_T posix_acl = NULL;
 	int ret = -1;
 
-	if ((posix_acl = conn->vfs_ops.sys_acl_get_fd(fsp, fd)) == NULL)
+	if ((posix_acl = vfs_sys_acl_get_fd(fsp, fd)) == NULL)
 		return -1;
 
 	if ((ret = chmod_acl_internals(conn, posix_acl, mode)) == -1)
 		goto done;
 
-	ret = conn->vfs_ops.sys_acl_set_fd(fsp, fd, posix_acl);
+	ret = vfs_sys_acl_set_fd(fsp, fd, posix_acl);
 
   done:
 
-	conn->vfs_ops.sys_acl_free_acl(conn, posix_acl);
+	vfs_sys_acl_free_acl(conn, posix_acl);
 	return ret;
 }
 
@@ -2909,14 +2909,14 @@ int fchmod_acl(files_struct *fsp, int fd
 
 BOOL directory_has_default_acl(connection_struct *conn, const char *fname)
 {
-        SMB_ACL_T dir_acl = conn->vfs_ops.sys_acl_get_file( conn, fname, SMB_ACL_TYPE_DEFAULT);
+        SMB_ACL_T dir_acl = vfs_sys_acl_get_file( conn, fname, SMB_ACL_TYPE_DEFAULT);
         BOOL has_acl = False;
         SMB_ACL_ENTRY_T entry;
 
-        if (dir_acl != NULL && (conn->vfs_ops.sys_acl_get_entry(conn, dir_acl, SMB_ACL_FIRST_ENTRY, &entry) == 1))
+        if (dir_acl != NULL && (vfs_sys_acl_get_entry(conn, dir_acl, SMB_ACL_FIRST_ENTRY, &entry) == 1))
                 has_acl = True;
 
 	if (dir_acl)
-	        conn->vfs_ops.sys_acl_free_acl(conn, dir_acl);
+	        vfs_sys_acl_free_acl(conn, dir_acl);
         return has_acl;
 }
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/reply.c 3_0-vfs/source/smbd/reply.c
--- 3_0/source/smbd/reply.c	Thu Apr 24 07:01:02 2003
+++ 3_0-vfs/source/smbd/reply.c	Thu Apr 24 07:03:31 2003
@@ -553,7 +553,7 @@ int reply_dskattr(connection_struct *con
 	SMB_BIG_UINT dfree,dsize,bsize;
 	START_PROFILE(SMBdskattr);
 
-	conn->vfs_ops.disk_free(conn,".",True,&bsize,&dfree,&dsize);
+	vfs_disk_free(conn,".",True,&bsize,&dfree,&dsize);
   
 	outsize = set_message(outbuf,5,0,True);
 	
@@ -1227,7 +1227,7 @@ static NTSTATUS can_delete(char *fname,c
 	if (!CAN_WRITE(conn))
 		return NT_STATUS_MEDIA_WRITE_PROTECTED;
 
-	if (conn->vfs_ops.lstat(conn,fname,&sbuf) != 0)
+	if (vfs_lstat(conn,fname,&sbuf) != 0)
 		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
 
 	fmode = dos_mode(conn,fname,&sbuf);
@@ -1429,7 +1429,7 @@ void send_file_readbraw(connection_struc
 		header.length = 4;
 		header.free = NULL;
 
-		if ( conn->vfs_ops.sendfile( smbd_server_fd(), fsp, fsp->fd, &header, startpos, nread) == -1) {
+		if ( vfs_sendfile( smbd_server_fd(), fsp, fsp->fd, &header, startpos, nread) == -1) {
 			/*
 			 * Special hack for broken Linux with no 64 bit clean sendfile. If we
 			 * return ENOSYS then pretend we just got a normal read.
@@ -1750,7 +1750,7 @@ int send_file_readX(connection_struct *c
 		header.length = data - outbuf;
 		header.free = NULL;
 
-		if ( conn->vfs_ops.sendfile( smbd_server_fd(), fsp, fsp->fd, &header, startpos, smb_maxcnt) == -1) {
+		if ( vfs_sendfile( smbd_server_fd(), fsp, fsp->fd, &header, startpos, smb_maxcnt) == -1) {
 			/*
 			 * Special hack for broken Linux with no 64 bit clean sendfile. If we
 			 * return ENOSYS then pretend we just got a normal read.
@@ -2262,7 +2262,7 @@ int reply_lseek(connection_struct *conn,
 			break;
 	}
 
-	if((res = conn->vfs_ops.lseek(fsp,fsp->fd,startpos,umode)) == -1) {
+	if((res = vfs_lseek(fsp,fsp->fd,startpos,umode)) == -1) {
 		/*
 		 * Check for the special case where a seek before the start
 		 * of the file sets the offset to zero. Added in the CIFS spec,
@@ -2274,7 +2274,7 @@ int reply_lseek(connection_struct *conn,
 
 			if(umode == SEEK_CUR) {
 
-				if((current_pos = conn->vfs_ops.lseek(fsp,fsp->fd,0,SEEK_CUR)) == -1) {
+				if((current_pos = vfs_lseek(fsp,fsp->fd,0,SEEK_CUR)) == -1) {
 					END_PROFILE(SMBlseek);
 					return(UNIXERROR(ERRDOS,ERRnoaccess));
 				}
@@ -2294,7 +2294,7 @@ int reply_lseek(connection_struct *conn,
 			}
  
 			if(current_pos < 0)
-				res = conn->vfs_ops.lseek(fsp,fsp->fd,0,SEEK_SET);
+				res = vfs_lseek(fsp,fsp->fd,0,SEEK_SET);
 		}
 
 		if(res == -1) {
@@ -2901,7 +2901,7 @@ static BOOL recursive_rmdir(connection_s
 		pstrcat(fullname, "/");
 		pstrcat(fullname, dname);
 
-		if(conn->vfs_ops.lstat(conn,fullname, &st) != 0) {
+		if(vfs_lstat(conn,fullname, &st) != 0) {
 			ret = True;
 			break;
 		}
@@ -2974,7 +2974,7 @@ BOOL rmdir_internals(connection_struct *
 					pstrcat(fullname, "/");
 					pstrcat(fullname, dname);
                      
-					if(conn->vfs_ops.lstat(conn,fullname, &st) != 0)
+					if(vfs_lstat(conn,fullname, &st) != 0)
 						break;
 					if(st.st_mode & S_IFDIR) {
 						if(lp_recursive_veto_delete(SNUM(conn))) {
@@ -3284,7 +3284,7 @@ directory = %s, newname = %s, newname_la
 			return NT_STATUS_OBJECT_NAME_COLLISION;
 		}
 
-		if(conn->vfs_ops.rename(conn,directory, newname) == 0) {
+		if(vfs_rename(conn,directory, newname) == 0) {
 			DEBUG(3,("rename_internals: succeeded doing rename on %s -> %s\n",
 				directory,newname));
 			return NT_STATUS_OK;	
@@ -3351,7 +3351,7 @@ directory = %s, newname = %s, newname_la
 					continue;
 				}
 				
-				if (!conn->vfs_ops.rename(conn,fname,destname))
+				if (!vfs_rename(conn,fname,destname))
 					count++;
 				DEBUG(3,("rename_internals: doing rename on %s -> %s\n",fname,destname));
 			}
@@ -3457,7 +3457,7 @@ static BOOL copy_file(char *src,char *de
 	}
 
 	if ((ofun&3) == 1) {
-		if(conn->vfs_ops.lseek(fsp2,fsp2->fd,0,SEEK_END) == -1) {
+		if(vfs_lseek(fsp2,fsp2->fd,0,SEEK_END) == -1) {
 			DEBUG(0,("copy_file: error - vfs lseek returned error %s\n", strerror(errno) ));
 			/*
 			 * Stop the copy from occurring.
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/service.c 3_0-vfs/source/smbd/service.c
--- 3_0/source/smbd/service.c	Thu Apr 24 07:01:02 2003
+++ 3_0-vfs/source/smbd/service.c	Thu Apr 24 07:03:32 2003
@@ -705,7 +705,7 @@ static connection_struct *make_connectio
 	/* Invoke VFS make connection hook */
 
 	if (conn->vfs_ops.connect) {
-		if (conn->vfs_ops.connect(conn, lp_servicename(snum), user) < 0) {
+		if (vfs_connect(conn, lp_servicename(snum), user) < 0) {
 			DEBUG(0,("make_connection: VFS make connection failed!\n"));
 			change_to_root_user();
 			conn_free(conn);
@@ -876,7 +876,7 @@ void close_cnum(connection_struct *conn,
 
 	    /* Call VFS disconnect hook */
 	    
-	    conn->vfs_ops.disconnect(conn);
+	    vfs_disconnect(conn);
 	    
 	}
 
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/trans2.c 3_0-vfs/source/smbd/trans2.c
--- 3_0/source/smbd/trans2.c	Fri Apr 25 07:24:37 2003
+++ 3_0-vfs/source/smbd/trans2.c	Fri Apr 25 07:26:45 2003
@@ -1338,7 +1338,7 @@ static int call_trans2qfsinfo(connection
 		{
 			SMB_BIG_UINT dfree,dsize,bsize;
 			data_len = 18;
-			conn->vfs_ops.disk_free(conn,".",False,&bsize,&dfree,&dsize);	
+			vfs_disk_free(conn,".",False,&bsize,&dfree,&dsize);	
 			SIVAL(pdata,l1_idFileSystem,st.st_dev);
 			SIVAL(pdata,l1_cSectorUnit,bsize/512);
 			SIVAL(pdata,l1_cUnit,dsize);
@@ -1406,7 +1406,7 @@ static int call_trans2qfsinfo(connection
 		{
 			SMB_BIG_UINT dfree,dsize,bsize,block_size,sectors_per_unit,bytes_per_sector;
 			data_len = 24;
-			conn->vfs_ops.disk_free(conn,".",False,&bsize,&dfree,&dsize);
+			vfs_disk_free(conn,".",False,&bsize,&dfree,&dsize);
 			block_size = lp_block_size(snum);
 			if (bsize < block_size) {
 				SMB_BIG_UINT factor = block_size/bsize;
@@ -1436,7 +1436,7 @@ cBytesSector=%u, cUnitTotal=%u, cUnitAva
 		{
 			SMB_BIG_UINT dfree,dsize,bsize,block_size,sectors_per_unit,bytes_per_sector;
 			data_len = 32;
-			conn->vfs_ops.disk_free(conn,".",False,&bsize,&dfree,&dsize);
+			vfs_disk_free(conn,".",False,&bsize,&dfree,&dsize);
 			block_size = lp_block_size(snum);
 			if (bsize < block_size) {
 				SMB_BIG_UINT factor = block_size/bsize;
@@ -1628,7 +1628,7 @@ static int call_trans2qfilepathinfo(conn
 				DEBUG(3,("fstat of fnum %d failed (%s)\n", fsp->fnum, strerror(errno)));
 				return(UNIXERROR(ERRDOS,ERRbadfid));
 			}
-			if((pos = fsp->conn->vfs_ops.lseek(fsp,fsp->fd,0,SEEK_CUR)) == -1)
+			if((pos = vfs_lseek(fsp,fsp->fd,0,SEEK_CUR)) == -1)
 				return(UNIXERROR(ERRDOS,ERRnoaccess));
 
 			delete_pending = fsp->delete_on_close;
@@ -2062,7 +2062,7 @@ static int call_trans2qfilepathinfo(conn
 #else
 				return(UNIXERROR(ERRDOS,ERRbadlink));
 #endif
-				len = conn->vfs_ops.readlink(conn,fullpathname, buffer, sizeof(pstring)-1);     /* read link */
+				len = vfs_readlink(conn,fullpathname, buffer, sizeof(pstring)-1);     /* read link */
 				if (len == -1)
 					return(UNIXERROR(ERRDOS,ERRnoaccess));
 				buffer[len] = 0;
@@ -2186,7 +2186,7 @@ static int ensure_link_is_safe(connectio
 		pstrcpy(link_dest, "./");
 	}
 		
-	if (conn->vfs_ops.realpath(conn,link_dest,resolved_name) == NULL)
+	if (vfs_realpath(conn,link_dest,resolved_name) == NULL)
 		return -1;
 
 	pstrcpy(link_dest, resolved_name);
@@ -2617,7 +2617,7 @@ size = %.0f, uid = %u, gid = %u, raw per
 0%o for file %s\n", (double)dev, unixmode, fname ));
 
 				/* Ok - do the mknod. */
-				if (conn->vfs_ops.mknod(conn,dos_to_unix_static(fname), unixmode, dev) != 0)
+				if (vfs_mknod(conn,dos_to_unix_static(fname), unixmode, dev) != 0)
 					return(UNIXERROR(ERRDOS,ERRnoaccess));
 
 				inherit_access_acl(conn, fname, unixmode);
@@ -2685,7 +2685,7 @@ size = %.0f, uid = %u, gid = %u, raw per
 			DEBUG(10,("call_trans2setfilepathinfo: SMB_SET_FILE_UNIX_LINK doing symlink %s -> %s\n",
 				fname, link_dest ));
 
-			if (conn->vfs_ops.symlink(conn,link_dest,fname) != 0)
+			if (vfs_symlink(conn,link_dest,fname) != 0)
 				return(UNIXERROR(ERRDOS,ERRnoaccess));
 			SSVAL(params,0,0);
 			send_trans2_replies(outbuf, bufsize, params, 2, *ppdata, 0);
@@ -2710,7 +2710,7 @@ size = %.0f, uid = %u, gid = %u, raw per
 			DEBUG(10,("call_trans2setfilepathinfo: SMB_SET_FILE_UNIX_LINK doing hard link %s -> %s\n",
 				fname, link_dest ));
 
-			if (conn->vfs_ops.link(conn,link_dest,fname) != 0)
+			if (vfs_link(conn,link_dest,fname) != 0)
 				return(UNIXERROR(ERRDOS,ERRnoaccess));
 			SSVAL(params,0,0);
 			send_trans2_replies(outbuf, bufsize, params, 2, *ppdata, 0);
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/vfs-wrap.c 3_0-vfs/source/smbd/vfs-wrap.c
--- 3_0/source/smbd/vfs-wrap.c	Fri Sep 27 07:48:09 2002
+++ 3_0-vfs/source/smbd/vfs-wrap.c	Tue Apr 22 16:27:46 2003
@@ -30,18 +30,18 @@
    is sure to try and execute them.  These stubs are used to prevent
    this possibility. */
 
-int vfswrap_dummy_connect(connection_struct *conn, const char *service, const char *user)
+int vfswrap_dummy_connect(vfs_handle_struct *handle, connection_struct *conn, const char *service, const char *user)
 {
     return 0;    /* Return >= 0 for success */
 }
 
-void vfswrap_dummy_disconnect(connection_struct *conn)
+void vfswrap_dummy_disconnect(vfs_handle_struct *handle, connection_struct *conn)
 {
 }
 
 /* Disk operations */
 
-SMB_BIG_UINT vfswrap_disk_free(connection_struct *conn, const char *path, BOOL small_query, SMB_BIG_UINT *bsize, 
+SMB_BIG_UINT vfswrap_disk_free(vfs_handle_struct *handle, connection_struct *conn, const char *path, BOOL small_query, SMB_BIG_UINT *bsize, 
 			       SMB_BIG_UINT *dfree, SMB_BIG_UINT *dsize)
 {
     SMB_BIG_UINT result;
@@ -52,7 +52,7 @@ SMB_BIG_UINT vfswrap_disk_free(connectio
     
 /* Directory operations */
 
-DIR *vfswrap_opendir(connection_struct *conn, const char *fname)
+DIR *vfswrap_opendir(vfs_handle_struct *handle, connection_struct *conn, const char *fname)
 {
     DIR *result;
 
@@ -62,7 +62,7 @@ DIR *vfswrap_opendir(connection_struct *
     return result;
 }
 
-struct dirent *vfswrap_readdir(connection_struct *conn, DIR *dirp)
+struct dirent *vfswrap_readdir(vfs_handle_struct *handle, connection_struct *conn, DIR *dirp)
 {
     struct dirent *result;
 
@@ -72,7 +72,7 @@ struct dirent *vfswrap_readdir(connectio
     return result;
 }
 
-int vfswrap_mkdir(connection_struct *conn, const char *path, mode_t mode)
+int vfswrap_mkdir(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode)
 {
 	int result;
 	BOOL has_dacl = False;
@@ -94,7 +94,7 @@ int vfswrap_mkdir(connection_struct *con
 		 */
 		int saved_errno = errno; /* We may get ENOSYS */
 		if (conn->vfs_ops.chmod_acl != NULL) {
-			if ((conn->vfs_ops.chmod_acl(conn, path, mode) == -1) && (errno == ENOSYS))
+			if ((vfs_chmod_acl(conn, path, mode) == -1) && (errno == ENOSYS))
 				errno = saved_errno;
 		}
 	}
@@ -103,7 +103,7 @@ int vfswrap_mkdir(connection_struct *con
     return result;
 }
 
-int vfswrap_rmdir(connection_struct *conn, const char *path)
+int vfswrap_rmdir(vfs_handle_struct *handle, connection_struct *conn, const char *path)
 {
     int result;
 
@@ -113,7 +113,7 @@ int vfswrap_rmdir(connection_struct *con
     return result;
 }
 
-int vfswrap_closedir(connection_struct *conn, DIR *dirp)
+int vfswrap_closedir(vfs_handle_struct *handle, connection_struct *conn, DIR *dirp)
 {
     int result;
 
@@ -125,7 +125,7 @@ int vfswrap_closedir(connection_struct *
 
 /* File operations */
     
-int vfswrap_open(connection_struct *conn, const char *fname, int flags, mode_t mode)
+int vfswrap_open(vfs_handle_struct *handle, connection_struct *conn, const char *fname, int flags, mode_t mode)
 {
     int result;
 
@@ -135,7 +135,7 @@ int vfswrap_open(connection_struct *conn
     return result;
 }
 
-int vfswrap_close(files_struct *fsp, int fd)
+int vfswrap_close(vfs_handle_struct *handle, files_struct *fsp, int fd)
 {
     int result;
 
@@ -146,7 +146,7 @@ int vfswrap_close(files_struct *fsp, int
     return result;
 }
 
-ssize_t vfswrap_read(files_struct *fsp, int fd, void *data, size_t n)
+ssize_t vfswrap_read(vfs_handle_struct *handle, files_struct *fsp, int fd, void *data, size_t n)
 {
     ssize_t result;
 
@@ -156,7 +156,7 @@ ssize_t vfswrap_read(files_struct *fsp, 
     return result;
 }
 
-ssize_t vfswrap_write(files_struct *fsp, int fd, const void *data, size_t n)
+ssize_t vfswrap_write(vfs_handle_struct *handle, files_struct *fsp, int fd, const void *data, size_t n)
 {
     ssize_t result;
 
@@ -166,7 +166,7 @@ ssize_t vfswrap_write(files_struct *fsp,
     return result;
 }
 
-SMB_OFF_T vfswrap_lseek(files_struct *fsp, int filedes, SMB_OFF_T offset, int whence)
+SMB_OFF_T vfswrap_lseek(vfs_handle_struct *handle, files_struct *fsp, int filedes, SMB_OFF_T offset, int whence)
 {
 	SMB_OFF_T result = 0;
 
@@ -192,7 +192,7 @@ SMB_OFF_T vfswrap_lseek(files_struct *fs
 	return result;
 }
 
-ssize_t vfswrap_sendfile(int tofd, struct files_struct *fsp, int fromfd, const DATA_BLOB *hdr,
+ssize_t vfswrap_sendfile(vfs_handle_struct *handle, int tofd, files_struct *fsp, int fromfd, const DATA_BLOB *hdr,
 			SMB_OFF_T offset, size_t n)
 {
 	ssize_t result;
@@ -203,7 +203,7 @@ ssize_t vfswrap_sendfile(int tofd, struc
 	return result;
 }
 
-int vfswrap_rename(connection_struct *conn, const char *old, const char *new)
+int vfswrap_rename(vfs_handle_struct *handle, connection_struct *conn, const char *old, const char *new)
 {
     int result;
 
@@ -213,7 +213,7 @@ int vfswrap_rename(connection_struct *co
     return result;
 }
 
-int vfswrap_fsync(files_struct *fsp, int fd)
+int vfswrap_fsync(vfs_handle_struct *handle, files_struct *fsp, int fd)
 {
 #ifdef HAVE_FSYNC
     int result;
@@ -228,7 +228,7 @@ int vfswrap_fsync(files_struct *fsp, int
 #endif
 }
 
-int vfswrap_stat(connection_struct *conn, const char *fname, SMB_STRUCT_STAT *sbuf)
+int vfswrap_stat(vfs_handle_struct *handle, connection_struct *conn, const char *fname, SMB_STRUCT_STAT *sbuf)
 {
     int result;
 
@@ -238,7 +238,7 @@ int vfswrap_stat(connection_struct *conn
     return result;
 }
 
-int vfswrap_fstat(files_struct *fsp, int fd, SMB_STRUCT_STAT *sbuf)
+int vfswrap_fstat(vfs_handle_struct *handle, files_struct *fsp, int fd, SMB_STRUCT_STAT *sbuf)
 {
     int result;
 
@@ -248,7 +248,7 @@ int vfswrap_fstat(files_struct *fsp, int
     return result;
 }
 
-int vfswrap_lstat(connection_struct *conn, const char *path, SMB_STRUCT_STAT *sbuf)
+int vfswrap_lstat(vfs_handle_struct *handle, connection_struct *conn, const char *path, SMB_STRUCT_STAT *sbuf)
 {
     int result;
 
@@ -258,7 +258,7 @@ int vfswrap_lstat(connection_struct *con
     return result;
 }
 
-int vfswrap_unlink(connection_struct *conn, const char *path)
+int vfswrap_unlink(vfs_handle_struct *handle, connection_struct *conn, const char *path)
 {
     int result;
 
@@ -268,7 +268,7 @@ int vfswrap_unlink(connection_struct *co
     return result;
 }
 
-int vfswrap_chmod(connection_struct *conn, const char *path, mode_t mode)
+int vfswrap_chmod(vfs_handle_struct *handle, connection_struct *conn, const char *path, mode_t mode)
 {
     int result;
 
@@ -283,7 +283,7 @@ int vfswrap_chmod(connection_struct *con
 	
 	if (conn->vfs_ops.chmod_acl != NULL) {
 		int saved_errno = errno; /* We might get ENOSYS */
-		if ((result = conn->vfs_ops.chmod_acl(conn, path, mode)) == 0) {
+		if ((result = vfs_chmod_acl(conn, path, mode)) == 0) {
 			END_PROFILE(syscall_chmod);
 			return result;
 		}
@@ -296,7 +296,7 @@ int vfswrap_chmod(connection_struct *con
     return result;
 }
 
-int vfswrap_fchmod(files_struct *fsp, int fd, mode_t mode)
+int vfswrap_fchmod(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode)
 {
     int result;
 	struct vfs_ops *vfs_ops = &fsp->conn->vfs_ops;
@@ -311,7 +311,7 @@ int vfswrap_fchmod(files_struct *fsp, in
 	
 	if (vfs_ops->fchmod_acl != NULL) {
 		int saved_errno = errno; /* We might get ENOSYS */
-		if ((result = vfs_ops->fchmod_acl(fsp, fd, mode)) == 0) {
+		if ((result = vfs_fchmod_acl(fsp, fd, mode)) == 0) {
 			END_PROFILE(syscall_chmod);
 			return result;
 		}
@@ -324,7 +324,7 @@ int vfswrap_fchmod(files_struct *fsp, in
     return result;
 }
 
-int vfswrap_chown(connection_struct *conn, const char *path, uid_t uid, gid_t gid)
+int vfswrap_chown(vfs_handle_struct *handle, connection_struct *conn, const char *path, uid_t uid, gid_t gid)
 {
     int result;
 
@@ -334,7 +334,7 @@ int vfswrap_chown(connection_struct *con
     return result;
 }
 
-int vfswrap_fchown(files_struct *fsp, int fd, uid_t uid, gid_t gid)
+int vfswrap_fchown(vfs_handle_struct *handle, files_struct *fsp, int fd, uid_t uid, gid_t gid)
 {
     int result;
 
@@ -345,7 +345,7 @@ int vfswrap_fchown(files_struct *fsp, in
     return result;
 }
 
-int vfswrap_chdir(connection_struct *conn, const char *path)
+int vfswrap_chdir(vfs_handle_struct *handle, connection_struct *conn, const char *path)
 {
     int result;
 
@@ -355,7 +355,7 @@ int vfswrap_chdir(connection_struct *con
     return result;
 }
 
-char *vfswrap_getwd(connection_struct *conn, char *path)
+char *vfswrap_getwd(vfs_handle_struct *handle, connection_struct *conn, char *path)
 {
     char *result;
 
@@ -365,7 +365,7 @@ char *vfswrap_getwd(connection_struct *c
     return result;
 }
 
-int vfswrap_utime(connection_struct *conn, const char *path, struct utimbuf *times)
+int vfswrap_utime(vfs_handle_struct *handle, connection_struct *conn, const char *path, struct utimbuf *times)
 {
     int result;
 
@@ -380,18 +380,17 @@ int vfswrap_utime(connection_struct *con
  allocate is set.
 **********************************************************************/
 
-static int strict_allocate_ftruncate(files_struct *fsp, int fd, SMB_OFF_T len)
+static int strict_allocate_ftruncate(vfs_handle_struct *handle, files_struct *fsp, int fd, SMB_OFF_T len)
 {
-	struct vfs_ops *vfs_ops = &fsp->conn->vfs_ops;
 	SMB_STRUCT_STAT st;
-	SMB_OFF_T currpos = vfs_ops->lseek(fsp, fd, 0, SEEK_CUR);
+	SMB_OFF_T currpos = vfs_lseek(fsp, fd, 0, SEEK_CUR);
 	unsigned char zero_space[4096];
 	SMB_OFF_T space_to_write;
 
 	if (currpos == -1)
 		return -1;
 
-	if (vfs_ops->fstat(fsp, fd, &st) == -1)
+	if (vfs_fstat(fsp, fd, &st) == -1)
 		return -1;
 
 	space_to_write = len - st.st_size;
@@ -409,7 +408,7 @@ static int strict_allocate_ftruncate(fil
 		return sys_ftruncate(fd, len);
 
 	/* Write out the real space on disk. */
-	if (vfs_ops->lseek(fsp, fd, st.st_size, SEEK_SET) != st.st_size)
+	if (vfs_lseek(fsp, fd, st.st_size, SEEK_SET) != st.st_size)
 		return -1;
 
 	space_to_write = len - st.st_size;
@@ -419,7 +418,7 @@ static int strict_allocate_ftruncate(fil
 		SMB_OFF_T retlen;
 		SMB_OFF_T current_len_to_write = MIN(sizeof(zero_space),space_to_write);
 
-		retlen = vfs_ops->write(fsp,fsp->fd,(char *)zero_space,current_len_to_write);
+		retlen = vfs_write(fsp,fsp->fd,(char *)zero_space,current_len_to_write);
 		if (retlen <= 0)
 			return -1;
 
@@ -427,16 +426,15 @@ static int strict_allocate_ftruncate(fil
 	}
 
 	/* Seek to where we were */
-	if (vfs_ops->lseek(fsp, fd, currpos, SEEK_SET) != currpos)
+	if (vfs_lseek(fsp, fd, currpos, SEEK_SET) != currpos)
 		return -1;
 
 	return 0;
 }
 
-int vfswrap_ftruncate(files_struct *fsp, int fd, SMB_OFF_T len)
+int vfswrap_ftruncate(vfs_handle_struct *handle, files_struct *fsp, int fd, SMB_OFF_T len)
 {
 	int result = -1;
-	struct vfs_ops *vfs_ops = &fsp->conn->vfs_ops;
 	SMB_STRUCT_STAT st;
 	char c = 0;
 	SMB_OFF_T currpos;
@@ -444,7 +442,7 @@ int vfswrap_ftruncate(files_struct *fsp,
 	START_PROFILE(syscall_ftruncate);
 
 	if (lp_strict_allocate(SNUM(fsp->conn))) {
-		result = strict_allocate_ftruncate(fsp, fd, len);
+		result = strict_allocate_ftruncate(handle, fsp, fd, len);
 		END_PROFILE(syscall_ftruncate);
 		return result;
 	}
@@ -462,7 +460,7 @@ int vfswrap_ftruncate(files_struct *fsp,
 	/* According to W. R. Stevens advanced UNIX prog. Pure 4.3 BSD cannot
 	   extend a file with ftruncate. Provide alternate implementation
 	   for this */
-	currpos = vfs_ops->lseek(fsp, fd, 0, SEEK_CUR);
+	currpos = vfs_lseek(fsp, fd, 0, SEEK_CUR);
 	if (currpos == -1) {
 		goto done;
 	}
@@ -471,7 +469,7 @@ int vfswrap_ftruncate(files_struct *fsp,
 	   size in which case the ftruncate above should have
 	   succeeded or shorter, in which case seek to len - 1 and
 	   write 1 byte of zero */
-	if (vfs_ops->fstat(fsp, fd, &st) == -1) {
+	if (vfs_fstat(fsp, fd, &st) == -1) {
 		goto done;
 	}
 
@@ -492,14 +490,14 @@ int vfswrap_ftruncate(files_struct *fsp,
 		goto done;
 	}
 
-	if (vfs_ops->lseek(fsp, fd, len-1, SEEK_SET) != len -1)
+	if (vfs_lseek(fsp, fd, len-1, SEEK_SET) != len -1)
 		goto done;
 
-	if (vfs_ops->write(fsp, fd, &c, 1)!=1)
+	if (vfs_write(fsp, fd, &c, 1)!=1)
 		goto done;
 
 	/* Seek to where we were */
-	if (vfs_ops->lseek(fsp, fd, currpos, SEEK_SET) != currpos)
+	if (vfs_lseek(fsp, fd, currpos, SEEK_SET) != currpos)
 		goto done;
 	result = 0;
 
@@ -509,7 +507,7 @@ int vfswrap_ftruncate(files_struct *fsp,
 	return result;
 }
 
-BOOL vfswrap_lock(files_struct *fsp, int fd, int op, SMB_OFF_T offset, SMB_OFF_T count, int type)
+BOOL vfswrap_lock(vfs_handle_struct *handle, files_struct *fsp, int fd, int op, SMB_OFF_T offset, SMB_OFF_T count, int type)
 {
     BOOL result;
 
@@ -520,7 +518,7 @@ BOOL vfswrap_lock(files_struct *fsp, int
     return result;
 }
 
-int vfswrap_symlink(connection_struct *conn, const char *oldpath, const char *newpath)
+int vfswrap_symlink(vfs_handle_struct *handle, connection_struct *conn, const char *oldpath, const char *newpath)
 {
     int result;
 
@@ -530,7 +528,7 @@ int vfswrap_symlink(connection_struct *c
     return result;
 }
 
-int vfswrap_readlink(connection_struct *conn, const char *path, char *buf, size_t bufsiz)
+int vfswrap_readlink(vfs_handle_struct *handle, connection_struct *conn, const char *path, char *buf, size_t bufsiz)
 {
     int result;
 
@@ -540,7 +538,7 @@ int vfswrap_readlink(connection_struct *
     return result;
 }
 
-int vfswrap_link(connection_struct *conn, const char *oldpath, const char *newpath)
+int vfswrap_link(vfs_handle_struct *handle, connection_struct *conn, const char *oldpath, const char *newpath)
 {
 	int result;
 
@@ -550,7 +548,7 @@ int vfswrap_link(connection_struct *conn
 	return result;
 }
 
-int vfswrap_mknod(connection_struct *conn, const char *pathname, mode_t mode, SMB_DEV_T dev)
+int vfswrap_mknod(vfs_handle_struct *handle, connection_struct *conn, const char *pathname, mode_t mode, SMB_DEV_T dev)
 {
 	int result;
 
@@ -560,7 +558,7 @@ int vfswrap_mknod(connection_struct *con
 	return result;
 }
 
-char *vfswrap_realpath(connection_struct *conn, const char *path, char *resolved_path)
+char *vfswrap_realpath(vfs_handle_struct *handle, connection_struct *conn, const char *path, char *resolved_path)
 {
 	char *result;
 
@@ -570,7 +568,7 @@ char *vfswrap_realpath(connection_struct
 	return result;
 }
 
-size_t vfswrap_fget_nt_acl(files_struct *fsp, int fd, SEC_DESC **ppdesc)
+size_t vfswrap_fget_nt_acl(vfs_handle_struct *handle, files_struct *fsp, int fd, SEC_DESC **ppdesc)
 {
 	size_t result;
 
@@ -580,7 +578,7 @@ size_t vfswrap_fget_nt_acl(files_struct 
 	return result;
 }
 
-size_t vfswrap_get_nt_acl(files_struct *fsp, const char *name, SEC_DESC **ppdesc)
+size_t vfswrap_get_nt_acl(vfs_handle_struct *handle, files_struct *fsp, const char *name, SEC_DESC **ppdesc)
 {
 	size_t result;
 
@@ -590,7 +588,7 @@ size_t vfswrap_get_nt_acl(files_struct *
 	return result;
 }
 
-BOOL vfswrap_fset_nt_acl(files_struct *fsp, int fd, uint32 security_info_sent, SEC_DESC *psd)
+BOOL vfswrap_fset_nt_acl(vfs_handle_struct *handle, files_struct *fsp, int fd, uint32 security_info_sent, SEC_DESC *psd)
 {
 	BOOL result;
 
@@ -600,7 +598,7 @@ BOOL vfswrap_fset_nt_acl(files_struct *f
 	return result;
 }
 
-BOOL vfswrap_set_nt_acl(files_struct *fsp, const char *name, uint32 security_info_sent, SEC_DESC *psd)
+BOOL vfswrap_set_nt_acl(vfs_handle_struct *handle, files_struct *fsp, const char *name, uint32 security_info_sent, SEC_DESC *psd)
 {
 	BOOL result;
 
@@ -610,7 +608,7 @@ BOOL vfswrap_set_nt_acl(files_struct *fs
 	return result;
 }
 
-int vfswrap_chmod_acl(connection_struct *conn, const char *name, mode_t mode)
+int vfswrap_chmod_acl(vfs_handle_struct *handle, connection_struct *conn, const char *name, mode_t mode)
 {
 	int result;
 
@@ -620,7 +618,7 @@ int vfswrap_chmod_acl(connection_struct 
 	return result;
 }
 
-int vfswrap_fchmod_acl(files_struct *fsp, int fd, mode_t mode)
+int vfswrap_fchmod_acl(vfs_handle_struct *handle, files_struct *fsp, int fd, mode_t mode)
 {
 	int result;
 
@@ -630,112 +628,124 @@ int vfswrap_fchmod_acl(files_struct *fsp
 	return result;
 }
 
-int vfswrap_sys_acl_get_entry(struct connection_struct *conn, SMB_ACL_T theacl, int entry_id, SMB_ACL_ENTRY_T *entry_p)
+int vfswrap_sys_acl_get_entry(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_T theacl, int entry_id, SMB_ACL_ENTRY_T *entry_p)
 {
 	return sys_acl_get_entry(theacl, entry_id, entry_p);
 }
 
-int vfswrap_sys_acl_get_tag_type(struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d, SMB_ACL_TAG_T *tag_type_p)
+int vfswrap_sys_acl_get_tag_type(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_ENTRY_T entry_d, SMB_ACL_TAG_T *tag_type_p)
 {
 	return sys_acl_get_tag_type(entry_d, tag_type_p);
 }
 
-int vfswrap_sys_acl_get_permset(struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d, SMB_ACL_PERMSET_T *permset_p)
+int vfswrap_sys_acl_get_permset(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_ENTRY_T entry_d, SMB_ACL_PERMSET_T *permset_p)
 {
 	return sys_acl_get_permset(entry_d, permset_p);
 }
 
-void * vfswrap_sys_acl_get_qualifier(struct connection_struct *conn, SMB_ACL_ENTRY_T entry_d)
+void * vfswrap_sys_acl_get_qualifier(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_ENTRY_T entry_d)
 {
 	return sys_acl_get_qualifier(entry_d);
 }
 
-SMB_ACL_T vfswrap_sys_acl_get_file(struct connection_struct *conn, const char *path_p, SMB_ACL_TYPE_T type)
+SMB_ACL_T vfswrap_sys_acl_get_file(vfs_handle_struct *handle, connection_struct *conn, const char *path_p, SMB_ACL_TYPE_T type)
 {
 	return sys_acl_get_file(path_p, type);
 }
 
-SMB_ACL_T vfswrap_sys_acl_get_fd(struct files_struct *fsp, int fd)
+SMB_ACL_T vfswrap_sys_acl_get_fd(vfs_handle_struct *handle, files_struct *fsp, int fd)
 {
 	return sys_acl_get_fd(fd);
 }
 
-int vfswrap_sys_acl_clear_perms(struct connection_struct *conn, SMB_ACL_PERMSET_T permset)
+int vfswrap_sys_acl_clear_perms(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_PERMSET_T permset)
 {
 	return sys_acl_clear_perms(permset);
 }
 
-int vfswrap_sys_acl_add_perm(struct connection_struct *conn, SMB_ACL_PERMSET_T permset, SMB_ACL_PERM_T perm)
+int vfswrap_sys_acl_add_perm(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_PERMSET_T permset, SMB_ACL_PERM_T perm)
 {
 	return sys_acl_add_perm(permset, perm);
 }
 
-char * vfswrap_sys_acl_to_text(struct connection_struct *conn, SMB_ACL_T theacl, ssize_t *plen)
+char * vfswrap_sys_acl_to_text(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_T theacl, ssize_t *plen)
 {
 	return sys_acl_to_text(theacl, plen);
 }
 
-SMB_ACL_T vfswrap_sys_acl_init(struct connection_struct *conn, int count)
+SMB_ACL_T vfswrap_sys_acl_init(vfs_handle_struct *handle, connection_struct *conn, int count)
 {
 	return sys_acl_init(count);
 }
 
-int vfswrap_sys_acl_create_entry(struct connection_struct *conn, SMB_ACL_T *pacl, SMB_ACL_ENTRY_T *pentry)
+int vfswrap_sys_acl_create_entry(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_T *pacl, SMB_ACL_ENTRY_T *pentry)
 {
 	return sys_acl_create_entry(pacl, pentry);
 }
 
-int vfswrap_sys_acl_set_tag_type(struct connection_struct *conn, SMB_ACL_ENTRY_T entry, SMB_ACL_TAG_T tagtype)
+int vfswrap_sys_acl_set_tag_type(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_ENTRY_T entry, SMB_ACL_TAG_T tagtype)
 {
 	return sys_acl_set_tag_type(entry, tagtype);
 }
 
-int vfswrap_sys_acl_set_qualifier(struct connection_struct *conn, SMB_ACL_ENTRY_T entry, void *qual)
+int vfswrap_sys_acl_set_qualifier(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_ENTRY_T entry, void *qual)
 {
 	return sys_acl_set_qualifier(entry, qual);
 }
 
-int vfswrap_sys_acl_set_permset(struct connection_struct *conn, SMB_ACL_ENTRY_T entry, SMB_ACL_PERMSET_T permset)
+int vfswrap_sys_acl_set_permset(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_ENTRY_T entry, SMB_ACL_PERMSET_T permset)
 {
 	return sys_acl_set_permset(entry, permset);
 }
 
-int vfswrap_sys_acl_valid(struct connection_struct *conn, SMB_ACL_T theacl )
+int vfswrap_sys_acl_valid(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_T theacl )
 {
 	return sys_acl_valid(theacl );
 }
 
-int vfswrap_sys_acl_set_file(struct connection_struct *conn, const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl)
+int vfswrap_sys_acl_set_file(vfs_handle_struct *handle, connection_struct *conn, const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl)
 {
 	return sys_acl_set_file(name, acltype, theacl);
 }
 
-int vfswrap_sys_acl_set_fd(struct files_struct *fsp, int fd, SMB_ACL_T theacl)
+int vfswrap_sys_acl_set_fd(vfs_handle_struct *handle, files_struct *fsp, int fd, SMB_ACL_T theacl)
 {
 	return sys_acl_set_fd(fd, theacl);
 }
 
-int vfswrap_sys_acl_delete_def_file(struct connection_struct *conn, const char *path)
+int vfswrap_sys_acl_delete_def_file(vfs_handle_struct *handle, connection_struct *conn, const char *path)
 {
 	return sys_acl_delete_def_file(path);
 }
 
-int vfswrap_sys_acl_get_perm(struct connection_struct *conn, SMB_ACL_PERMSET_T permset, SMB_ACL_PERM_T perm)
+int vfswrap_sys_acl_get_perm(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_PERMSET_T permset, SMB_ACL_PERM_T perm)
 {
 	return sys_acl_get_perm(permset, perm);
 }
 
-int vfswrap_sys_acl_free_text(struct connection_struct *conn, char *text)
+int vfswrap_sys_acl_free_text(vfs_handle_struct *handle, connection_struct *conn, char *text)
 {
 	return sys_acl_free_text(text);
 }
 
-int vfswrap_sys_acl_free_acl(struct connection_struct *conn, SMB_ACL_T posix_acl)
+int vfswrap_sys_acl_free_acl(vfs_handle_struct *handle, connection_struct *conn, SMB_ACL_T posix_acl)
 {
 	return sys_acl_free_acl(posix_acl);
 }
 
-int vfswrap_sys_acl_free_qualifier(struct connection_struct *conn, void *qualifier, SMB_ACL_TAG_T tagtype)
+int vfswrap_sys_acl_free_qualifier(vfs_handle_struct *handle, connection_struct *conn, void *qualifier, SMB_ACL_TAG_T tagtype)
 {
 	return sys_acl_free_qualifier(qualifier, tagtype);
+}
+
+int vfswrap_get_quota(vfs_handle_struct *handle, connection_struct *conn, enum SMB_QUOTA_TYPE qtype, unid_t id, SMB_DISK_QUOTA *qt)
+{
+	errno = ENOSYS;
+	return -1;	
+}
+
+int vfswrap_set_quota(vfs_handle_struct *handle, connection_struct *conn, enum SMB_QUOTA_TYPE qtype, unid_t id, SMB_DISK_QUOTA *qt)
+{
+	errno = ENOSYS;
+	return -1;
 }
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/smbd/vfs.c 3_0-vfs/source/smbd/vfs.c
--- 3_0/source/smbd/vfs.c	Tue Apr 22 06:46:32 2003
+++ 3_0-vfs/source/smbd/vfs.c	Sat Apr 26 14:33:43 2003
@@ -28,9 +28,9 @@
 #define DBGC_CLASS DBGC_VFS
 
 struct vfs_init_function_entry {
-   char *name;
-   vfs_op_tuple *ops, *(*init)(const struct vfs_ops *, struct smb_vfs_handle_struct *);
-   struct vfs_init_function_entry *prev, *next;
+	char *name;
+ 	vfs_op_tuple *vfs_ops;
+	struct vfs_init_function_entry *prev, *next;
 };
 
 static struct vfs_init_function_entry *backends = NULL;
@@ -42,12 +42,6 @@ struct vfs_syminfo {
 	void *fptr;
 };
 
-/*
-  Opaque (final) vfs operations. This is a combination of first-met opaque vfs operations
-  across all currently processed modules.  */
-
-static vfs_op_tuple vfs_opaque_ops[SMB_VFS_OP_LAST];
-
 /* Default vfs hooks.  WARNING: The order of these initialisers is
    very important.  They must be in the same order as defined in
    vfs.h.  Change at your own peril. */
@@ -59,6 +53,8 @@ static struct vfs_ops default_vfs_ops = 
 	vfswrap_dummy_connect,
 	vfswrap_dummy_disconnect,
 	vfswrap_disk_free,
+	vfswrap_get_quota,
+	vfswrap_set_quota,
 
 	/* Directory operations */
 
@@ -140,116 +136,65 @@ static struct vfs_ops default_vfs_ops = 
 
 static struct vfs_init_function_entry *vfs_find_backend_entry(const char *name)
 {
-   struct vfs_init_function_entry *entry = backends;
-   pstring stripped;
-
-   module_path_get_name(name, stripped);
+	struct vfs_init_function_entry *entry = backends;
+	pstring stripped;
 
-   while(entry) {
-       if (strequal(entry->name, stripped)) return entry;
-       entry = entry->next;
-   }
-
-   return NULL;
-}
-
-BOOL smb_register_vfs(const char *name, vfs_op_tuple *(*init)(const struct vfs_ops *, struct smb_vfs_handle_struct *), int version)
-{
-   struct vfs_init_function_entry *entry = backends;
-   
-   if ((version < SMB_VFS_INTERFACE_CASCADED)) {
-       DEBUG(0, ("vfs_init() returned wrong interface version info (was %d, should be no less than %d)\n",
-           version, SMB_VFS_INTERFACE_VERSION ));
-       return False;
-   }
-  
-   if ((version < SMB_VFS_INTERFACE_VERSION)) {
-       DEBUG(0, ("Warning: vfs_init() states that module confirms interface version #%d, current interface version is #%d.\n\
-		Proceeding in compatibility mode, new operations (since version #%d) will fallback to default ones.\n",
-		version, SMB_VFS_INTERFACE_VERSION, version ));
-	   return False;
-   }
-   
-   while(entry) {
-       if (strequal(entry->name, name)) {
-           DEBUG(0,("VFS module %s already loaded!\n", name));
-           return False;
-       }
-       entry = entry->next;
-   }
-
-   entry = smb_xmalloc(sizeof(struct vfs_init_function_entry));
-   entry->name = smb_xstrdup(name);
-   entry->init = init;
-
-   DLIST_ADD(backends, entry);
-   DEBUG(5, ("Successfully added vfs backend '%s'\n", name));
-   return True;
-}
-
-/****************************************************************************
-  initialise default vfs hooks
-****************************************************************************/
-
-static void vfs_init_default(connection_struct *conn)
-{
-	DEBUG(3, ("Initialising default vfs hooks\n"));
+	module_path_get_name(name, stripped);
+ 
+	while(entry) {
+		if (strequal(entry->name, stripped)) return entry;
+		entry = entry->next;
+	}
 
-	memcpy(&conn->vfs_ops, &default_vfs_ops, sizeof(struct vfs_ops));
-	conn->vfs_private = NULL;
+	return NULL;
 }
 
-/***************************************************************************
- Function to load old VFS modules. Should go away after a while.
- **************************************************************************/
-
-static vfs_op_tuple *vfs_load_old_plugin(connection_struct *conn, const char *vfs_object)
+BOOL smb_register_vfs(const char *name, vfs_op_tuple *vfs_ops, int version)
 {
-	int vfs_version = -1;
-	vfs_op_tuple *ops, *(*init_fptr)(int *, const struct vfs_ops *, struct smb_vfs_handle_struct *);
-	/* Open object file */
+	struct vfs_init_function_entry *entry = backends;
 
-	if ((conn->vfs_private->handle = sys_dlopen(vfs_object, RTLD_NOW)) == NULL) {
-		DEBUG(0, ("Error opening %s: %s\n", vfs_object, sys_dlerror()));
-		return NULL;
+	if (!name || !name[0] || !vfs_ops) {
+		DEBUG(0,("smb_register_vfs() called with NULL pointer or empty name!\n"));
+		return False;
 	}
 
-	/* Get handle on vfs_init() symbol */
-
-	init_fptr = (vfs_op_tuple *(*)(int *, const struct vfs_ops *, struct smb_vfs_handle_struct *))sys_dlsym(conn->vfs_private->handle, "vfs_init");
-
-	if (init_fptr == NULL) {
-		DEBUG(0, ("No vfs_init() symbol found in %s\n", vfs_object));
-		sys_dlclose(conn->vfs_private->handle);
-		return NULL;
-	}
+ 	if ((version != SMB_VFS_INTERFACE_VERSION)) {
+		DEBUG(0, ("Failed to register vfs module '%s'.\n"
+		          "The module was compiled against SMB_VFS_INTERFACE_VERSION %d,\n"
+		          "current SMB_VFS_INTERFACE_VERSION is %d.\n"
+		          "Please recompile against the current Samba Version!\n",  
+			name, version, SMB_VFS_INTERFACE_VERSION));
+		return False;
+  	}
 
-	/* Initialise vfs_ops structure */
-	if ((ops = init_fptr(&vfs_version, &conn->vfs_ops, conn->vfs_private)) == NULL) {
-		DEBUG(0, ("vfs_init() function from %s failed\n", vfs_object));
-		sys_dlclose(conn->vfs_private->handle);
-		return NULL;
+	while(entry) {
+		if (strequal(entry->name, name)) {
+			DEBUG(0,("VFS module %s already loaded!\n", name));
+			return False;
+		}
+		entry = entry->next;
 	}
 
-	if ((vfs_version < SMB_VFS_INTERFACE_CASCADED)) {
-		DEBUG(0, ("vfs_init() returned wrong interface version info (was %d, should be no less than %d)\n",
-				  vfs_version, SMB_VFS_INTERFACE_VERSION ));
-		sys_dlclose(conn->vfs_private->handle);
-		return NULL;
-	}
+	entry = smb_xmalloc(sizeof(struct vfs_init_function_entry));
+	entry->name = smb_xstrdup(name);
+	entry->vfs_ops = vfs_ops;
 
-	if ((vfs_version < SMB_VFS_INTERFACE_VERSION)) {
-		DEBUG(0, ("Warning: vfs_init() states that module confirms interface version #%d, current interface version is #%d.\n\
-				  Proceeding in compatibility mode, new operations (since version #%d) will fallback to default ones.\n",
-				  vfs_version, SMB_VFS_INTERFACE_VERSION, vfs_version ));
-		sys_dlclose(conn->vfs_private->handle);
-		return NULL;
-	}
-	
-	return ops;
+	DLIST_ADD(backends, entry);
+	DEBUG(5, ("Successfully added vfs backend '%s'\n", name));
+	return True;
 }
 
+/****************************************************************************
+  initialise default vfs hooks
+****************************************************************************/
 
+static void vfs_init_default(connection_struct *conn)
+{
+	DEBUG(3, ("Initialising default vfs hooks\n"));
+
+	memcpy(&conn->vfs_ops, &default_vfs_ops, sizeof(struct vfs_ops));
+	memcpy(&conn->vfs_opaque_ops, &default_vfs_ops, sizeof(struct vfs_ops));
+}
 
 /****************************************************************************
   initialise custom vfs hooks
@@ -258,51 +203,80 @@ static vfs_op_tuple *vfs_load_old_plugin
 BOOL vfs_init_custom(connection_struct *conn, const char *vfs_object)
 {
 	vfs_op_tuple *ops;
+	char *module_name = NULL;
+	char *module_param = NULL, *p;
 	int i;
+	vfs_handle_struct *handle;
 	struct vfs_init_function_entry *entry;
-
-	DEBUG(3, ("Initialising custom vfs hooks from %s\n", vfs_object));
+	
+	if (!conn||!vfs_object||!vfs_object[0]) {
+		DEBUG(0,("vfs_init_custon() called with NULL pointer or emtpy vfs_object!\n"));
+		return False;
+	}
 
 	if(!backends) static_init_vfs;
 
+	DEBUG(3, ("Initialising custom vfs hooks from [%s]\n", vfs_object));
+
+	module_name = smb_xstrdup(vfs_object);
+
+	p = strchr(module_name, ':');
+
+	if (p) {
+		*p = 0;
+		module_param = p+1;
+		trim_string(module_param, " ", " ");
+	}
+
+	trim_string(module_name, " ", " ");
+
 	/* First, try to load the module with the new module system */
-	if((entry = vfs_find_backend_entry(vfs_object)) || 
-	   (smb_probe_module("vfs", vfs_object) && 
-		(entry = vfs_find_backend_entry(vfs_object)))) {
+	if((entry = vfs_find_backend_entry(module_name)) || 
+	   (smb_probe_module("vfs", module_name) && 
+		(entry = vfs_find_backend_entry(module_name)))) {
 
-		DEBUG(3,("Successfully loaded %s with the new modules system\n", vfs_object));
+		DEBUGADD(5,("Successfully loaded vfs module [%s] with the new modules system\n", vfs_object));
 		
-		if ((ops = entry->init(&conn->vfs_ops, conn->vfs_private)) == NULL) {
-			DEBUG(0, ("vfs init function from %s failed\n", vfs_object));
-			return False;
-		}
+	 	if ((ops = entry->vfs_ops) == NULL) {
+	 		DEBUG(0, ("entry->vfs_ops==NULL for [%s] failed\n", vfs_object));
+	 		SAFE_FREE(module_name);
+	 		return False;
+	 	}
 	} else {
-		/* If that doesn't work, fall back to the old system 
-		 * (This part should go away after a while, it's only here 
-		 * for backwards compatibility) */
-		DEBUG(2, ("Can't load module %s with new modules system, falling back to compatibility\n",
-			vfs_object));
-		if ((ops = vfs_load_old_plugin(conn, vfs_object)) == NULL) {
-			DEBUG(0, ("vfs init function from %s failed\n", vfs_object));
-			return False;
-		}
+		DEBUG(0,("Can't find a vfs module [%s]\n",vfs_object));
+		SAFE_FREE(module_name);
+		return False;
 	}
 
+	handle = (vfs_handle_struct *)talloc_zero(conn->mem_ctx,sizeof(vfs_handle_struct));
+	if (!handle) {
+		DEBUG(0,("talloc_zero() failed!\n"));
+		SAFE_FREE(module_name);
+		return False;
+	}
+	memcpy(&handle->vfs_def_ops, &conn->vfs_ops, sizeof(struct vfs_ops));
+	handle->conn = conn;
+	if (module_param) {
+		handle->param = talloc_strdup(conn->mem_ctx, module_param);
+	}
+	DLIST_ADD(conn->vfs_handles, handle);
+
  	for(i=0; ops[i].op != NULL; i++) {
- 	  DEBUG(3, ("Checking operation #%d (type %d, layer %d)\n", i, ops[i].type, ops[i].layer));
+ 	  DEBUG(5, ("Checking operation #%d (type %d, layer %d)\n", i, ops[i].type, ops[i].layer));
  	  if(ops[i].layer == SMB_VFS_LAYER_OPAQUE) {
  	    /* Check whether this operation was already made opaque by different module */
- 	    if(vfs_opaque_ops[ops[i].type].op == ((void**)&default_vfs_ops)[ops[i].type]) {
+ 	    if(((void**)&conn->vfs_opaque_ops)[ops[i].type] == ((void**)&default_vfs_ops)[ops[i].type]) {
  	      /* No, it isn't overloaded yet. Overload. */
- 	      DEBUG(3, ("Making operation type %d opaque [module %s]\n", ops[i].type, vfs_object));
- 	      vfs_opaque_ops[ops[i].type] = ops[i];
+ 	      DEBUGADD(5, ("Making operation type %d opaque [module %s]\n", ops[i].type, vfs_object));
+ 	      ((void**)&conn->vfs_opaque_ops)[ops[i].type] = ops[i].op;
  	    }
  	  }
  	  /* Change current VFS disposition*/
- 	  DEBUG(3, ("Accepting operation type %d from module %s\n", ops[i].type, vfs_object));
+ 	  DEBUGADD(5, ("Accepting operation type %d from module %s\n", ops[i].type, vfs_object));
  	  ((void**)&conn->vfs_ops)[ops[i].type] = ops[i].op;
 	}
 
+	SAFE_FREE(module_name);
 	return True;
 }
 
@@ -313,71 +287,31 @@ BOOL vfs_init_custom(connection_struct *
 BOOL smbd_vfs_init(connection_struct *conn)
 {
 	const char **vfs_objects;
-	char *vfs_module, *vfs_path;
 	unsigned int i = 0;
 	int j = 0;
-	struct smb_vfs_handle_struct *handle;
 	
 	/* Normal share - initialise with disk access functions */
 	vfs_init_default(conn);
-	vfs_objects = lp_vfsobj(SNUM(conn));
+	vfs_objects = lp_vfs_objects(SNUM(conn));
 
 	/* Override VFS functions if 'vfs object' was not specified*/
 	if (!vfs_objects || !vfs_objects[0])
 		return True;
-
-	for(i=0; i<SMB_VFS_OP_LAST; i++) {
-		vfs_opaque_ops[i].op = ((void**)&default_vfs_ops)[i];
-		vfs_opaque_ops[i].type = i;
-		vfs_opaque_ops[i].layer = SMB_VFS_LAYER_OPAQUE;
+	
+	for (i=0; vfs_objects[i] ;) {
+		i++;
 	}
 
-	vfs_path = lp_vfs_path(SNUM(conn));
-	
-	for (i=0; vfs_objects[i]; i++); /* count passed modules */
-        
 	for (j=i-1; j >= 0; j--) {
-		conn->vfs_private = NULL;
-		handle = (struct smb_vfs_handle_struct *) smb_xmalloc(sizeof(smb_vfs_handle_struct));
-		/* Loadable object file */
-		handle->handle = NULL;
-		DLIST_ADD(conn->vfs_private, handle);
-		vfs_module = NULL;
-		if (vfs_path && *vfs_path) {
-			asprintf(&vfs_module, "%s/%s", vfs_path, vfs_objects[j]);
-		} else {
-			asprintf(&vfs_module, "%s", vfs_objects[j]);
-		}
-		if (!vfs_init_custom(conn, vfs_module)) {
-			DEBUG(0, ("smbd_vfs_init: vfs_init_custom failed for %s\n", vfs_module));
-			SAFE_FREE(vfs_module);
-			DLIST_REMOVE(conn->vfs_private, handle);
-			SAFE_FREE(handle);
+		if (!vfs_init_custom(conn, vfs_objects[j])) {
+			DEBUG(0, ("smbd_vfs_init: vfs_init_custom failed for %s\n", vfs_objects[j]));
 			return False;
 		}
-		SAFE_FREE(vfs_module);
 	}
 	return True;
 }
 
 /*******************************************************************
- Create vfs_ops reflecting current vfs_opaque_ops
-*******************************************************************/
-
-struct vfs_ops *smb_vfs_get_opaque_ops(void)
-{
-  int i;
-  struct vfs_ops *ops;
-
-  ops = smb_xmalloc(sizeof(struct vfs_ops));
-
-  for(i=0; i<SMB_VFS_OP_LAST; i++) {
-    ((void**)ops)[i] = vfs_opaque_ops[i].op;
-  }
-  return ops;
-}
-
-/*******************************************************************
  Check if directory exists.
 ********************************************************************/
 
@@ -400,15 +334,6 @@ BOOL vfs_directory_exist(connection_stru
 }
 
 /*******************************************************************
- vfs getwd wrapper 
-********************************************************************/
-
-static char *vfs_getwd(connection_struct *conn, char *path)
-{
-	return conn->vfs_ops.getwd(conn,path);
-}
-
-/*******************************************************************
  vfs mkdir wrapper 
 ********************************************************************/
 
@@ -417,7 +342,7 @@ int vfs_mkdir(connection_struct *conn, c
 	int ret;
 	SMB_STRUCT_STAT sbuf;
 
-	if(!(ret=conn->vfs_ops.mkdir(conn,name,mode))) {
+	if(!(ret=conn->vfs_ops.mkdir(conn->vfs_handles, conn, name,mode))) {
 
 		inherit_access_acl(conn, name, mode);
 
@@ -479,7 +404,7 @@ ssize_t vfs_read_data(files_struct *fsp,
 
 	while (total < byte_count)
 	{
-		ssize_t ret = fsp->conn->vfs_ops.read(fsp, fsp->fd, buf + total,
+		ssize_t ret = fsp->conn->vfs_ops.read(fsp->conn->vfs_handles, fsp, fsp->fd, buf + total,
 					byte_count - total);
 
 		if (ret == 0) return total;
@@ -504,7 +429,7 @@ ssize_t vfs_write_data(files_struct *fsp
 	ssize_t ret;
 
 	while (total < N) {
-		ret = fsp->conn->vfs_ops.write(fsp,fsp->fd,buffer + total,N - total);
+		ret = fsp->conn->vfs_ops.write(fsp->conn->vfs_handles,fsp,fsp->fd,buffer + total,N - total);
 
 		if (ret == -1)
 			return -1;
@@ -527,7 +452,6 @@ int vfs_allocate_file_space(files_struct
 	int ret;
 	SMB_STRUCT_STAT st;
 	connection_struct *conn = fsp->conn;
-	struct vfs_ops *vfs_ops = &conn->vfs_ops;
 	SMB_BIG_UINT space_avail;
 	SMB_BIG_UINT bsize,dfree,dsize;
 
@@ -558,7 +482,7 @@ int vfs_allocate_file_space(files_struct
 				fsp->fsp_name, (double)st.st_size ));
 
 		flush_write_cache(fsp, SIZECHANGE_FLUSH);
-		if ((ret = vfs_ops->ftruncate(fsp, fsp->fd, (SMB_OFF_T)len)) != -1) {
+		if ((ret = vfs_ftruncate(fsp, fsp->fd, (SMB_OFF_T)len)) != -1) {
 			set_filelen_write_cache(fsp, len);
 		}
 		return ret;
@@ -571,7 +495,7 @@ int vfs_allocate_file_space(files_struct
 
 	len -= st.st_size;
 	len /= 1024; /* Len is now number of 1k blocks needed. */
-	space_avail = conn->vfs_ops.disk_free(conn,fsp->fsp_name,False,&bsize,&dfree,&dsize);
+	space_avail = vfs_disk_free(conn,fsp->fsp_name,False,&bsize,&dfree,&dsize);
 
 	DEBUG(10,("vfs_allocate_file_space: file %s, grow. Current size %.0f, needed blocks = %.0f, space avail = %.0f\n",
 			fsp->fsp_name, (double)st.st_size, (double)len, (double)space_avail ));
@@ -597,7 +521,7 @@ int vfs_set_filelen(files_struct *fsp, S
 	release_level_2_oplocks_on_change(fsp);
 	DEBUG(10,("vfs_set_filelen: ftruncate %s to len %.0f\n", fsp->fsp_name, (double)len));
 	flush_write_cache(fsp, SIZECHANGE_FLUSH);
-	if ((ret = fsp->conn->vfs_ops.ftruncate(fsp, fsp->fd, len)) != -1)
+	if ((ret = fsp->conn->vfs_ops.ftruncate(fsp->conn->vfs_handles, fsp, fsp->fd, len)) != -1)
 		set_filelen_write_cache(fsp, len);
 
 	return ret;
@@ -612,12 +536,12 @@ static files_struct *out_fsp;
 
 static ssize_t read_fn(int fd, void *buf, size_t len)
 {
-	return in_fsp->conn->vfs_ops.read(in_fsp, fd, buf, len);
+	return in_fsp->conn->vfs_ops.read(in_fsp->conn->vfs_handles, in_fsp, fd, buf, len);
 }
 
 static ssize_t write_fn(int fd, const void *buf, size_t len)
 {
-	return out_fsp->conn->vfs_ops.write(out_fsp, fd, buf, len);
+	return out_fsp->conn->vfs_ops.write(out_fsp->conn->vfs_handles, out_fsp, fd, buf, len);
 }
 
 SMB_OFF_T vfs_transfer_file(files_struct *in, files_struct *out, SMB_OFF_T n)
@@ -634,13 +558,13 @@ SMB_OFF_T vfs_transfer_file(files_struct
 
 char *vfs_readdirname(connection_struct *conn, void *p)
 {
-	struct dirent *ptr;
+	struct dirent *ptr= NULL;
 	char *dname;
 
 	if (!p)
 		return(NULL);
 
-	ptr = (struct dirent *)conn->vfs_ops.readdir(conn,p);
+	ptr = (struct dirent *)vfs_readdir(conn,p);
 	if (!ptr)
 		return(NULL);
 
@@ -659,72 +583,6 @@ char *vfs_readdirname(connection_struct 
 	return(dname);
 }
 
-/* VFS options not quite working yet */
-
-#if 0
-
-/***************************************************************************
-  handle the interpretation of the vfs option parameter
- *************************************************************************/
-static BOOL handle_vfs_option(char *pszParmValue, char **ptr)
-{
-    struct vfs_options *new_option, **options = (struct vfs_options **)ptr;
-    int i;
-
-    /* Create new vfs option */
-
-    new_option = (struct vfs_options *)malloc(sizeof(*new_option));
-    if (new_option == NULL) {
-	return False;
-    }
-
-    ZERO_STRUCTP(new_option);
-
-    /* Get name and value */
-
-    new_option->name = strtok(pszParmValue, "=");
-
-    if (new_option->name == NULL) {
-	return False;
-    }
-
-    while(isspace(*new_option->name)) {
-	new_option->name++;
-    }
-
-    for (i = strlen(new_option->name); i > 0; i--) {
-	if (!isspace(new_option->name[i - 1])) break;
-    }
-
-    new_option->name[i] = '\0';
-    new_option->name = strdup(new_option->name);
-
-    new_option->value = strtok(NULL, "=");
-
-    if (new_option->value != NULL) {
-
-	while(isspace(*new_option->value)) {
-	    new_option->value++;
-	}
-	
-	for (i = strlen(new_option->value); i > 0; i--) {
-	    if (!isspace(new_option->value[i - 1])) break;
-	}
-	
-	new_option->value[i] = '\0';
-	new_option->value = strdup(new_option->value);
-    }
-
-    /* Add to list */
-
-    DLIST_ADD(*options, new_option);
-
-    return True;
-}
-
-#endif
-
-
 /*******************************************************************
  A wrapper for vfs_chdir().
 ********************************************************************/
@@ -740,7 +598,7 @@ int vfs_ChDir(connection_struct *conn, c
 	if (*path == '/' && strcsequal(LastDir,path))
 		return(0);
 
-	DEBUG(3,("vfs_ChDir to %s\n",path));
+	DEBUG(4,("vfs_ChDir to %s\n",path));
 
 	res = vfs_chdir(conn,path);
 	if (!res)
@@ -908,7 +766,7 @@ static BOOL readlink_check(connection_st
 		realdir[reallen] = 0;
 	}
 
-	if (conn->vfs_ops.readlink(conn, name, flink, sizeof(pstring) -1) != -1) {
+	if (vfs_readlink(conn, name, flink, sizeof(pstring) -1) != -1) {
 		DEBUG(3,("reduce_name: file path name %s is a symlink\nChecking it's path\n", name));
 		if (*flink == '/') {
 			pstrcpy(cleanlink, flink);
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/torture/cmd_vfs.c 3_0-vfs/source/torture/cmd_vfs.c
--- 3_0/source/torture/cmd_vfs.c	Mon Dec  2 16:11:47 2002
+++ 3_0-vfs/source/torture/cmd_vfs.c	Wed Apr 23 10:23:47 2003
@@ -23,37 +23,28 @@
 #include "includes.h"
 #include "vfstest.h"
 
-static char *null_string = "";
+static const char *null_string = "";
 
-static NTSTATUS cmd_load_module(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_load_module(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
-	struct smb_vfs_handle_struct *handle;
-	char *path = lp_vfs_path(0);
-	char name[PATH_MAX];
+	int i;
 	
 	if (argc != 2) {
-		printf("Usage: load <module path>\n");
+		printf("Usage: load <modules>\n");
 		return NT_STATUS_OK;
 	}
 
-	if (path != NULL && *path != '\0') {
-		snprintf(name, PATH_MAX, "%s/%s", path, argv[1]);
-	} else {
-		snprintf(name, PATH_MAX, "%s", argv[1]);
-	}
-	vfs->conn->vfs_private = NULL;
-	handle = (struct smb_vfs_handle_struct *) smb_xmalloc(sizeof(smb_vfs_handle_struct));
-	handle->handle = NULL;
-	DLIST_ADD(vfs->conn->vfs_private, handle)
-	if (!vfs_init_custom(vfs->conn, name)) {
-		DEBUG(0, ("load: error=-1 (vfs_init_custom failed for %s)\n", argv[1]));
-		return NT_STATUS_UNSUCCESSFUL;
+	for (i=1;i<argc;i++) {
+		if (!vfs_init_custom(vfs->conn, argv[i])) {
+			DEBUG(0, ("load: (vfs_init_custom failed for %s)\n", argv[i]));
+			return NT_STATUS_UNSUCCESSFUL;
+		}
 	}
 	printf("load: ok\n");
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_populate(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_populate(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	char c;
 	size_t size;
@@ -73,7 +64,7 @@ static NTSTATUS cmd_populate(struct vfs_
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_show_data(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_show_data(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	size_t offset;
 	size_t len;
@@ -101,19 +92,19 @@ static NTSTATUS cmd_show_data(struct vfs
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_connect(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_connect(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
-	vfs->conn->vfs_ops.connect(vfs->conn, lp_servicename(vfs->conn->service), "vfstest");
+	vfs_connect(vfs->conn, lp_servicename(vfs->conn->service), "vfstest");
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_disconnect(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_disconnect(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
-	vfs->conn->vfs_ops.disconnect(vfs->conn);
+	vfs_disconnect(vfs->conn);
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_disk_free(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_disk_free(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	SMB_BIG_UINT diskfree, bsize, dfree, dsize;
 	if (argc != 2) {
@@ -121,7 +112,7 @@ static NTSTATUS cmd_disk_free(struct vfs
 		return NT_STATUS_OK;
 	}
 
-	diskfree = vfs->conn->vfs_ops.disk_free(vfs->conn, argv[1], False, &bsize, &dfree, &dsize);
+	diskfree = vfs_disk_free(vfs->conn, argv[1], False, &bsize, &dfree, &dsize);
 	printf("disk_free: %lu, bsize = %lu, dfree = %lu, dsize = %lu\n",
 			(unsigned long)diskfree,
 			(unsigned long)bsize,
@@ -131,14 +122,14 @@ static NTSTATUS cmd_disk_free(struct vfs
 }
 
 
-static NTSTATUS cmd_opendir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_opendir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	if (argc != 2) {
 		printf("Usage: opendir <fname>\n");
 		return NT_STATUS_OK;
 	}
 
-	vfs->currentdir = vfs->conn->vfs_ops.opendir(vfs->conn, argv[1]);
+	vfs->currentdir = vfs_opendir(vfs->conn, argv[1]);
 	if (vfs->currentdir == NULL) {
 		printf("opendir error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
@@ -149,7 +140,7 @@ static NTSTATUS cmd_opendir(struct vfs_s
 }
 
 
-static NTSTATUS cmd_readdir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_readdir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	struct dirent *dent;
 
@@ -158,7 +149,7 @@ static NTSTATUS cmd_readdir(struct vfs_s
 		return NT_STATUS_UNSUCCESSFUL;
 	}
 
-	dent = vfs->conn->vfs_ops.readdir(vfs->conn, vfs->currentdir);
+	dent = vfs_readdir(vfs->conn, vfs->currentdir);
 	if (dent == NULL) {
 		printf("readdir: NULL\n");
 		return NT_STATUS_OK;
@@ -169,14 +160,14 @@ static NTSTATUS cmd_readdir(struct vfs_s
 }
 
 
-static NTSTATUS cmd_mkdir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_mkdir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	if (argc != 2) {
 		printf("Usage: mkdir <path>\n");
 		return NT_STATUS_OK;
 	}
 
-	if (vfs->conn->vfs_ops.mkdir(vfs->conn, argv[1], 00755) == -1) {
+	if (vfs_mkdir(vfs->conn, argv[1], 00755) == -1) {
 		printf("mkdir error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -186,7 +177,7 @@ static NTSTATUS cmd_mkdir(struct vfs_sta
 }
 
 
-static NTSTATUS cmd_closedir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_closedir(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int ret;
 	
@@ -195,7 +186,7 @@ static NTSTATUS cmd_closedir(struct vfs_
 		return NT_STATUS_UNSUCCESSFUL;
 	}
 
-	ret = vfs->conn->vfs_ops.closedir(vfs->conn, vfs->currentdir);
+	ret = vfs_closedir(vfs->conn, vfs->currentdir);
 	if (ret == -1) {
 		printf("closedir failure: %s\n", strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
@@ -207,11 +198,11 @@ static NTSTATUS cmd_closedir(struct vfs_
 }
 
 
-static NTSTATUS cmd_open(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_open(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int flags, fd;
 	mode_t mode;
-	char *flagstr;
+	const char *flagstr;
 
 	mode = 00400;
 
@@ -287,7 +278,7 @@ static NTSTATUS cmd_open(struct vfs_stat
 		}
 	}
 
-	fd = vfs->conn->vfs_ops.open(vfs->conn, argv[1], flags, mode);
+	fd = vfs_open(vfs->conn, argv[1], flags, mode);
 	if (fd == -1) {
 		printf("open: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
@@ -302,7 +293,7 @@ static NTSTATUS cmd_open(struct vfs_stat
 }
 
 
-static NTSTATUS cmd_pathfunc(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_pathfunc(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int ret = -1;
 
@@ -312,11 +303,11 @@ static NTSTATUS cmd_pathfunc(struct vfs_
 	}
 
 	if (strcmp("rmdir", argv[0]) == 0 ) {
-		ret = vfs->conn->vfs_ops.rmdir(vfs->conn, argv[1]);
+		ret = vfs_rmdir(vfs->conn, argv[1]);
 	} else if (strcmp("unlink", argv[0]) == 0 ) {
-		ret = vfs->conn->vfs_ops.unlink(vfs->conn, argv[1]);
+		ret = vfs_unlink(vfs->conn, argv[1]);
 	} else if (strcmp("chdir", argv[0]) == 0 ) {
-		ret = vfs->conn->vfs_ops.chdir(vfs->conn, argv[1]);
+		ret = vfs_chdir(vfs->conn, argv[1]);
 	} else {
 		printf("%s: error=%d (invalid function name!)\n", argv[0], errno);
 		return NT_STATUS_UNSUCCESSFUL;
@@ -332,7 +323,7 @@ static NTSTATUS cmd_pathfunc(struct vfs_
 }
 
 
-static NTSTATUS cmd_close(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_close(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int fd, ret;
 
@@ -347,7 +338,7 @@ static NTSTATUS cmd_close(struct vfs_sta
 		return NT_STATUS_OK;
 	}
 
-	ret = vfs->conn->vfs_ops.close(vfs->files[fd], fd);
+	ret = vfs_close(vfs->files[fd], fd);
 	if (ret == -1 )
 		printf("close: error=%d (%s)\n", errno, strerror(errno));
 	else
@@ -360,7 +351,7 @@ static NTSTATUS cmd_close(struct vfs_sta
 }
 
 
-static NTSTATUS cmd_read(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_read(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int fd;
 	size_t size, rsize;
@@ -380,7 +371,7 @@ static NTSTATUS cmd_read(struct vfs_stat
 	}
 	vfs->data_size = size;
 	
-	rsize = vfs->conn->vfs_ops.read(vfs->files[fd], fd, vfs->data, size);
+	rsize = vfs_read(vfs->files[fd], fd, vfs->data, size);
 	if (rsize == -1) {
 		printf("read: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
@@ -391,7 +382,7 @@ static NTSTATUS cmd_read(struct vfs_stat
 }
 
 
-static NTSTATUS cmd_write(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_write(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int fd, size, wsize;
 
@@ -413,7 +404,7 @@ static NTSTATUS cmd_write(struct vfs_sta
 		return NT_STATUS_UNSUCCESSFUL;
 	}
 
-	wsize = vfs->conn->vfs_ops.write(vfs->files[fd], fd, vfs->data, size);
+	wsize = vfs_write(vfs->files[fd], fd, vfs->data, size);
 
 	if (wsize == -1) {
 		printf("write: error=%d (%s)\n", errno, strerror(errno));
@@ -425,7 +416,7 @@ static NTSTATUS cmd_write(struct vfs_sta
 }
 
 
-static NTSTATUS cmd_lseek(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_lseek(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int fd, offset, whence;
 	SMB_OFF_T pos;
@@ -444,7 +435,7 @@ static NTSTATUS cmd_lseek(struct vfs_sta
 		default:	whence = SEEK_END;
 	}
 
-	pos = vfs->conn->vfs_ops.lseek(vfs->files[fd], fd, offset, whence);
+	pos = vfs_lseek(vfs->files[fd], fd, offset, whence);
 	if (pos == (SMB_OFF_T)-1) {
 		printf("lseek: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
@@ -455,7 +446,7 @@ static NTSTATUS cmd_lseek(struct vfs_sta
 }
 
 
-static NTSTATUS cmd_rename(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_rename(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int ret;
 	if (argc != 3) {
@@ -463,7 +454,7 @@ static NTSTATUS cmd_rename(struct vfs_st
 		return NT_STATUS_OK;
 	}
 
-	ret = vfs->conn->vfs_ops.rename(vfs->conn, argv[1], argv[2]);
+	ret = vfs_rename(vfs->conn, argv[1], argv[2]);
 	if (ret == -1) {
 		printf("rename: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
@@ -474,7 +465,7 @@ static NTSTATUS cmd_rename(struct vfs_st
 }
 
 
-static NTSTATUS cmd_fsync(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_fsync(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int ret, fd;
 	if (argc != 2) {
@@ -483,7 +474,7 @@ static NTSTATUS cmd_fsync(struct vfs_sta
 	}
 
 	fd = atoi(argv[1]);
-	ret = vfs->conn->vfs_ops.fsync(vfs->files[fd], fd);
+	ret = vfs_fsync(vfs->files[fd], fd);
 	if (ret == -1) {
 		printf("fsync: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
@@ -494,11 +485,11 @@ static NTSTATUS cmd_fsync(struct vfs_sta
 }
 
 
-static NTSTATUS cmd_stat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_stat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int ret;
-	char *user;
-	char *group;
+	const char *user;
+	const char *group;
 	struct passwd *pwd;
 	struct group *grp;
 	SMB_STRUCT_STAT st;
@@ -508,17 +499,17 @@ static NTSTATUS cmd_stat(struct vfs_stat
 		return NT_STATUS_OK;
 	}
 
-	ret = vfs->conn->vfs_ops.stat(vfs->conn, argv[1], &st);
+	ret = vfs_stat(vfs->conn, argv[1], &st);
 	if (ret == -1) {
 		printf("stat: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
 
 	pwd = sys_getpwuid(st.st_uid);
-	if (pwd != NULL) user = strdup(pwd->pw_name);
+	if (pwd != NULL) user = pwd->pw_name;
 	else user = null_string;
 	grp = sys_getgrgid(st.st_gid);
-	if (grp != NULL) group = strdup(grp->gr_name);
+	if (grp != NULL) group = grp->gr_name;
 	else group = null_string;
 
 	printf("stat: ok\n");
@@ -541,17 +532,17 @@ static NTSTATUS cmd_stat(struct vfs_stat
 	printf("  Access: %s", ctime(&(st.st_atime)));
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
-	if (user != null_string) SAFE_FREE(user);
-	if (group!= null_string) SAFE_FREE(group);
+	SAFE_FREE(pwd);
+	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
 
-static NTSTATUS cmd_fstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_fstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int fd;
-	char *user;
-	char *group;
+	const char *user;
+	const char *group;
 	struct passwd *pwd;
 	struct group *grp;
 	SMB_STRUCT_STAT st;
@@ -572,16 +563,16 @@ static NTSTATUS cmd_fstat(struct vfs_sta
 		return NT_STATUS_OK;
 	}
 
-	if (vfs->conn->vfs_ops.fstat(vfs->files[fd], fd, &st) == -1) {
+	if (vfs_fstat(vfs->files[fd], fd, &st) == -1) {
 		printf("fstat: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
 
 	pwd = sys_getpwuid(st.st_uid);
-	if (pwd != NULL) user = strdup(pwd->pw_name);
+	if (pwd != NULL) user = pwd->pw_name;
 	else user = null_string;
 	grp = sys_getgrgid(st.st_gid);
-	if (grp != NULL) group = strdup(grp->gr_name);
+	if (grp != NULL) group = grp->gr_name;
 	else group = null_string;
 
 	printf("fstat: ok\n");
@@ -603,16 +594,16 @@ static NTSTATUS cmd_fstat(struct vfs_sta
 	printf("  Access: %s", ctime(&(st.st_atime)));
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
-	if (user != null_string) SAFE_FREE(user);
-	if (group!= null_string) SAFE_FREE(group);
+	SAFE_FREE(pwd);
+	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
 
-static NTSTATUS cmd_lstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_lstat(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
-	char *user;
-	char *group;
+	const char *user;
+	const char *group;
 	struct passwd *pwd;
 	struct group *grp;
 	SMB_STRUCT_STAT st;
@@ -622,16 +613,16 @@ static NTSTATUS cmd_lstat(struct vfs_sta
 		return NT_STATUS_OK;
 	}
 
-	if (vfs->conn->vfs_ops.lstat(vfs->conn, argv[1], &st) == -1) {
+	if (vfs_lstat(vfs->conn, argv[1], &st) == -1) {
 		printf("lstat: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
 
 	pwd = sys_getpwuid(st.st_uid);
-	if (pwd != NULL) user = strdup(pwd->pw_name);
+	if (pwd != NULL) user = pwd->pw_name;
 	else user = null_string;
 	grp = sys_getgrgid(st.st_gid);
-	if (grp != NULL) group = strdup(grp->gr_name);
+	if (grp != NULL) group = grp->gr_name;
 	else group = null_string;
 
 	printf("lstat: ok\n");
@@ -653,13 +644,13 @@ static NTSTATUS cmd_lstat(struct vfs_sta
 	printf("  Access: %s", ctime(&(st.st_atime)));
 	printf("  Modify: %s", ctime(&(st.st_mtime)));
 	printf("  Change: %s", ctime(&(st.st_ctime)));
-	if (user != null_string) SAFE_FREE(user);
-	if (group!= null_string) SAFE_FREE(group);
+	SAFE_FREE(pwd);
+	SAFE_FREE(grp);
 	return NT_STATUS_OK;
 }
 
 
-static NTSTATUS cmd_chmod(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_chmod(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	mode_t mode;
 	if (argc != 3) {
@@ -668,7 +659,7 @@ static NTSTATUS cmd_chmod(struct vfs_sta
 	}
 
 	mode = atoi(argv[2]);
-	if (vfs->conn->vfs_ops.chmod(vfs->conn, argv[1], mode) == -1) {
+	if (vfs_chmod(vfs->conn, argv[1], mode) == -1) {
 		printf("chmod: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -678,7 +669,7 @@ static NTSTATUS cmd_chmod(struct vfs_sta
 }
 
 
-static NTSTATUS cmd_fchmod(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_fchmod(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int fd;
 	mode_t mode;
@@ -698,7 +689,7 @@ static NTSTATUS cmd_fchmod(struct vfs_st
 		return NT_STATUS_OK;
 	}
 
-	if (vfs->conn->vfs_ops.fchmod(vfs->files[fd], fd, mode) == -1) {
+	if (vfs_fchmod(vfs->files[fd], fd, mode) == -1) {
 		printf("fchmod: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -708,7 +699,7 @@ static NTSTATUS cmd_fchmod(struct vfs_st
 }
 
 
-static NTSTATUS cmd_chown(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_chown(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	uid_t uid;
 	gid_t gid;
@@ -719,7 +710,7 @@ static NTSTATUS cmd_chown(struct vfs_sta
 
 	uid = atoi(argv[2]);
 	gid = atoi(argv[3]);
-	if (vfs->conn->vfs_ops.chown(vfs->conn, argv[1], uid, gid) == -1) {
+	if (vfs_chown(vfs->conn, argv[1], uid, gid) == -1) {
 		printf("chown: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -729,7 +720,7 @@ static NTSTATUS cmd_chown(struct vfs_sta
 }
 
 
-static NTSTATUS cmd_fchown(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_fchown(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	uid_t uid;
 	gid_t gid;
@@ -750,7 +741,7 @@ static NTSTATUS cmd_fchown(struct vfs_st
 		printf("fchown: error=%d (invalid file descriptor)\n", EBADF);
 		return NT_STATUS_OK;
 	}
-	if (vfs->conn->vfs_ops.fchown(vfs->files[fd], fd, uid, gid) == -1) {
+	if (vfs_fchown(vfs->files[fd], fd, uid, gid) == -1) {
 		printf("fchown error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -760,10 +751,10 @@ static NTSTATUS cmd_fchown(struct vfs_st
 }
 
 
-static NTSTATUS cmd_getwd(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_getwd(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	char buf[PATH_MAX];
-	if (vfs->conn->vfs_ops.getwd(vfs->conn, buf) == NULL) {
+	if (vfs_getwd(vfs->conn, buf) == NULL) {
 		printf("getwd: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -772,7 +763,7 @@ static NTSTATUS cmd_getwd(struct vfs_sta
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_utime(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_utime(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	struct utimbuf times;
 	if (argc != 4) {
@@ -781,7 +772,7 @@ static NTSTATUS cmd_utime(struct vfs_sta
 	}
 	times.actime = atoi(argv[2]);
 	times.modtime = atoi(argv[3]);
-	if (vfs->conn->vfs_ops.utime(vfs->conn, argv[1], &times) != 0) {
+	if (vfs_utime(vfs->conn, argv[1], &times) != 0) {
 		printf("utime: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -790,7 +781,7 @@ static NTSTATUS cmd_utime(struct vfs_sta
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_ftruncate(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_ftruncate(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	int fd;
 	SMB_OFF_T off;
@@ -810,7 +801,7 @@ static NTSTATUS cmd_ftruncate(struct vfs
 		return NT_STATUS_OK;
 	}
 
-	if (vfs->conn->vfs_ops.ftruncate(vfs->files[fd], fd, off) == -1) {
+	if (vfs_ftruncate(vfs->files[fd], fd, off) == -1) {
 		printf("ftruncate: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -819,7 +810,7 @@ static NTSTATUS cmd_ftruncate(struct vfs
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_lock(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_lock(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	BOOL ret;
 	int fd;
@@ -827,7 +818,7 @@ static NTSTATUS cmd_lock(struct vfs_stat
 	long offset;
 	long count;
 	int type;
-	char *typestr;
+	const char *typestr;
 	
 	if (argc != 6) {
 		printf("Usage: lock <fd> <op> <offset> <count> <type>\n");
@@ -893,7 +884,7 @@ static NTSTATUS cmd_lock(struct vfs_stat
 
 	printf("lock: debug lock(fd=%d, op=%d, offset=%ld, count=%ld, type=%d))\n", fd, op, offset, count, type);
 
-	if ((ret = vfs->conn->vfs_ops.lock(vfs->files[fd], fd, op, offset, count, type)) == False) {
+	if ((ret = vfs_lock(vfs->files[fd], fd, op, offset, count, type)) == False) {
 		printf("lock: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -902,14 +893,14 @@ static NTSTATUS cmd_lock(struct vfs_stat
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_symlink(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_symlink(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	if (argc != 3) {
 		printf("Usage: symlink <path> <link>\n");
 		return NT_STATUS_OK;
 	}
 
-	if (vfs->conn->vfs_ops.symlink(vfs->conn, argv[1], argv[2]) == -1) {
+	if (vfs_symlink(vfs->conn, argv[1], argv[2]) == -1) {
 		printf("symlink: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -919,7 +910,7 @@ static NTSTATUS cmd_symlink(struct vfs_s
 }
 
 
-static NTSTATUS cmd_readlink(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_readlink(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	char buffer[PATH_MAX];
 	int size;
@@ -929,7 +920,7 @@ static NTSTATUS cmd_readlink(struct vfs_
 		return NT_STATUS_OK;
 	}
 
-	if ((size = vfs->conn->vfs_ops.readlink(vfs->conn, argv[1], buffer, PATH_MAX)) == -1) {
+	if ((size = vfs_readlink(vfs->conn, argv[1], buffer, PATH_MAX)) == -1) {
 		printf("readlink: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -940,14 +931,14 @@ static NTSTATUS cmd_readlink(struct vfs_
 }
 
 
-static NTSTATUS cmd_link(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_link(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	if (argc != 3) {
 		printf("Usage: link <path> <link>\n");
 		return NT_STATUS_OK;
 	}
 
-	if (vfs->conn->vfs_ops.link(vfs->conn, argv[1], argv[2]) == -1) {
+	if (vfs_link(vfs->conn, argv[1], argv[2]) == -1) {
 		printf("link: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -956,7 +947,7 @@ static NTSTATUS cmd_link(struct vfs_stat
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_mknod(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_mknod(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	mode_t mode;
 	unsigned int dev_val;
@@ -980,7 +971,7 @@ static NTSTATUS cmd_mknod(struct vfs_sta
 	}
 	dev = (SMB_DEV_T)dev_val;
 
-	if (vfs->conn->vfs_ops.mknod(vfs->conn, argv[1], mode, dev) == -1) {
+	if (vfs_mknod(vfs->conn, argv[1], mode, dev) == -1) {
 		printf("mknod: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -989,7 +980,7 @@ static NTSTATUS cmd_mknod(struct vfs_sta
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_realpath(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_realpath(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	char respath[PATH_MAX];
 	
@@ -998,7 +989,7 @@ static NTSTATUS cmd_realpath(struct vfs_
 		return NT_STATUS_OK;
 	}
 
-	if (vfs->conn->vfs_ops.realpath(vfs->conn, argv[1], respath) == NULL) {
+	if (vfs_realpath(vfs->conn, argv[1], respath) == NULL) {
 		printf("realpath: error=%d (%s)\n", errno, strerror(errno));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/torture/vfstest.c 3_0-vfs/source/torture/vfstest.c
--- 3_0/source/torture/vfstest.c	Tue Apr 15 00:01:46 2003
+++ 3_0-vfs/source/torture/vfstest.c	Wed Apr 23 08:55:01 2003
@@ -106,7 +106,7 @@ static char* next_command(char** cmdstr)
 
 /* Load specified configuration file */
 static NTSTATUS cmd_conf(struct vfs_state *vfs, TALLOC_CTX *mem_ctx,
-			int argc, char **argv)
+			int argc, const char **argv)
 {
 	if (argc != 2) {
 		printf("Usage: %s <smb.conf>\n", argv[0]);
@@ -181,7 +181,7 @@ static NTSTATUS cmd_help(struct vfs_stat
 }
 
 /* Change the debug level */
-static NTSTATUS cmd_debuglevel(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_debuglevel(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	if (argc > 2) {
 		printf("Usage: %s [debuglevel]\n", argv[0]);
@@ -197,7 +197,7 @@ static NTSTATUS cmd_debuglevel(struct vf
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_freemem(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_freemem(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	/* Cleanup */
 	talloc_destroy(mem_ctx);
@@ -207,7 +207,7 @@ static NTSTATUS cmd_freemem(struct vfs_s
 	return NT_STATUS_OK;
 }
 
-static NTSTATUS cmd_quit(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, char **argv)
+static NTSTATUS cmd_quit(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, const char **argv)
 {
 	/* Cleanup */
 	talloc_destroy(mem_ctx);
@@ -311,7 +311,7 @@ static NTSTATUS do_cmd(struct vfs_state 
 		}
 
 		/* Run command */
-		result = cmd_entry->fn(vfs, mem_ctx, argc, argv);
+		result = cmd_entry->fn(vfs, mem_ctx, argc, (const char **)argv);
 
 	} else {
 		fprintf (stderr, "Invalid command\n");
@@ -478,7 +478,7 @@ int main(int argc, char *argv[])
 	struct cmd_set 		**cmd_set;
 	static struct vfs_state vfs;
 	int i;
-	static const char	*filename = NULL;
+	static char		*filename = NULL;
 
 	/* make sure the vars that get altered (4th field) are in
 	   a fixed location or certain compilers complain */
@@ -522,7 +522,7 @@ int main(int argc, char *argv[])
 	/* some basic initialization stuff */
 	conn_init();
 	vfs.conn = conn_new();
-	vfs.conn->user = "vfstest";
+	string_set(&vfs.conn->user,"vfstest");
 	for (i=0; i < 1024; i++)
 		vfs.files[i] = NULL;
 
diff -Npur --exclude=CVS --exclude=*.bak --exclude=*.o --exclude=*.po --exclude=*.so --exclude=.#* --exclude=Makefile --exclude=stamp-h --exclude=configure --exclude=findsmb --exclude=*proto*.h --exclude=build_env.h --exclude=tdbsam2_parse_info.h --exclude=config.* --exclude=bin --exclude=*.configure --exclude=autom4te.cache 3_0/source/torture/vfstest.h 3_0-vfs/source/torture/vfstest.h
--- 3_0/source/torture/vfstest.h	Sun Jan  5 12:21:01 2003
+++ 3_0-vfs/source/torture/vfstest.h	Wed Apr 23 08:47:11 2003
@@ -39,7 +39,7 @@ struct vfs_state {
 struct cmd_set {
 	const char *name;
 	NTSTATUS (*fn)(struct vfs_state *vfs, TALLOC_CTX *mem_ctx, int argc, 
-                       char **argv);
+                       const char **argv);
 	const char *description;
 	const char *usage;
 };
